<?php

//---------------------------------------------------------------------------//
// author: Yanick Bourbeau <ybourbeau@mrgtech.ca> 
// date:   2006.01.01
// web:    http://www.mrgtech.ca
// info:   Solar Imperium
//---------------------------------------------------------------------------//
// copyleft license
//
// this software is provided 'as-is', without any express or implied
// warranty. in no event will the authors be held liable for any damages
// arising from the use of this software.
//
// permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions: 
//
// 1. the origin of this software must not be misrepresented;
//	  you must not claim that you wrote the original software. 
//	  if you use this software in a product, an acknowledgment 
//	  in the product documentation would be appreciated but is not required.
//
// 2. altered source versions must be plainly marked as such, 
//	  and must not be misrepresented as being the original software.
//
// 3. mail about the fact of using this class in production 
//	  would be very appreciated.
//
// 4. this notice may not be removed or altered from any source distribution.
//
//---------------------------------------------------------------------------//
// CHANGELOG:
//
// 2006.06.01 : ybo : Changed name: Imperium Galactica is (c), moved to Solar Imperium
// 2007.02.04 : ybo : Reviewed the code for public release
//
//---------------------------------------------------------------------------//

class AgriculturalAdvisor
{
	
	var $empire;
	
	/* Constructor */
	function AgriculturalAdvisor($empire)
	{
		$this->empire = $empire;
	}
	
	/*  Get advise */
	function getAdvise()
	{
		global $GAME;
	
		$comment = LANG_NOTHING_TO_SAY;
		$action = "nothing";
	

		/******************************************************************************************/	
		// Condition 1 : No enough food for the next turn
		/******************************************************************************************/	
		// first, raw production calculated.
		

		$food_produced_prime = $this->empire->planets->data["food_planets"] * CONF_FOOD_PRODUCTION;
		$food_produced_prime = $GAME["system"]->alterNumber(round(($food_produced_prime/100) * $this->empire->production->data["food_short"]),5);
		$food_produced_civilunrest = ($food_produced_prime * ($this->empire->data["civil_status"] * CONF_POP_CIVILSTATUS));
	
		// calculating procuced food
		$food_produced = $food_produced_prime - $food_produced_civilunrest;

		$food_eaten_by_population = $GAME["system"]->alterNumber(round($this->empire->data["population"] * CONF_FOOD_POPULATION_EATING),5);
		$food_eaten_by_soldiers = $GAME["system"]->alterNumber(round($this->empire->army->data["soldiers"] * CONF_FOOD_SOLDIERS_EATING),5);

		// total
		$food_final = $food_produced - $food_eaten_by_population - $food_eaten_by_soldiers;
		
		if ($food_final < 0) {
			$comment = LANG_PRODUCE_MORE_FOOD;
			
			// tell the AI to buy more food planets
			$required_planets = floor((abs($food_final) / CONF_FOOD_PRODUCTION) * 1.5); 
			$action = "buy_food_planets=".$required_planets;
		
			return(array("comment"=>$comment,"action"=>$action));
		}


		/******************************************************************************************/	
		// Condition 2 : Just enough
		/******************************************************************************************/	
		if (($food_final > 0) && ($food_final < 100))  {
		
			$comment = LANG_YOUR_FOOD_PRODUCTION_IS_JUST_FINE;
			return(array("comment"=>$comment,"action"=>$action));
		}

		/******************************************************************************************/	
		// Condition 3 : Sell surpluses on market if the market prices are high
		/******************************************************************************************/	
		$prices = $GAME["gameplay_costs"]->getFoodPrices();
		if ( $prices["demand"] == LANG_DEMAND_HIGH) {
			$comment = LANG_YOU_CAN_SELL_FOOD_SURPLUSES_WHILE_THE_DEMAND_IS_HIGH;
			$action="sell_food=".($food_final-100);
			return(array("comment"=>$comment,"action"=>$action));
		
		}


		/******************************************************************************************/	
		// Condition 4 : Do not sell surpluses on market if the market prices are low
		/******************************************************************************************/	
		$prices = $GAME["gameplay_costs"]->getFoodPrices();
		if ( $prices["demand"] == LANG_DEMAND_HIGH) {
			$comment = LANG_DONT_SELL_FOOD_SURPLUSES_WHILE_THE_DEMAND_IS_LOW;
			return(array("comment"=>$comment,"action"=>$action));
		
		}

		
		return(array("comment"=>$comment,"action"=>$action));

	}
}

?>