<?php

//---------------------------------------------------------------------------//
// author: Yanick Bourbeau <ybourbeau@mrgtech.ca> 
// date:   2006.01.01
// web:    http://www.mrgtech.ca
// info:   Solar Imperium
//---------------------------------------------------------------------------//
// copyleft license
//
// this software is provided 'as-is', without any express or implied
// warranty. in no event will the authors be held liable for any damages
// arising from the use of this software.
//
// permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions: 
//
// 1. the origin of this software must not be misrepresented;
//	  you must not claim that you wrote the original software. 
//	  if you use this software in a product, an acknowledgment 
//	  in the product documentation would be appreciated but is not required.
//
// 2. altered source versions must be plainly marked as such, 
//	  and must not be misrepresented as being the original software.
//
// 3. mail about the fact of using this class in production 
//	  would be very appreciated.
//
// 4. this notice may not be removed or altered from any source distribution.
//
//---------------------------------------------------------------------------//
// CHANGELOG:
//
// 2006.06.01 : ybo : Changed name: Imperium Galactica is (c), moved to Solar Imperium
// 2007.02.04 : ybo : Reviewed the code for public release
//
//---------------------------------------------------------------------------//

class MilitaryAdvisor
{
	
	var $empire;
	
	/* Constructor */
	function MilitaryAdvisor($empire)
	{
		$this->empire = $empire;
	}
	
	/*  Get advise */
	function getAdvise()
	{
		global $GAME,$DB;
	
		$comment = LANG_NOTHING_TO_SAY;
		$action = "nothing";
	

		/******************************************************************************************/	
		// Condition 1 : If you are under protection, just tell to leave military unchanged
		/******************************************************************************************/	
		if ($this->empire->data["protection_turns_left"] > 2) {

			$comment = LANG_ITS_NOT_REQUIRED_TO_HAVE_A_ARMY_YET;
		///	return(array("comment"=>$comment,"action"=>$action));
			
		}

		/******************************************************************************************/	
		// Condition 2 : 2 Turns before end of protection, tell to start building forces
		/******************************************************************************************/	
		if (($this->empire->data["protection_turns_left"] > 0) && ($this->empire->data["protection_turns_left"] <= 2)) {

			$comment = LANG_START_STOCKPILING_UNITS;
			$action = "buy_random_units";

		//	return(array("comment"=>$comment,"action"=>$action));
			
		}
		
		$results = array();

		$total_army = 0;
		$total_army += $this->empire->army->data["soldiers"];
		$total_army += $this->empire->army->data["fighters"];
		$total_army += $this->empire->army->data["stations"];
		$total_army += $this->empire->army->data["lightcruisers"];
		$total_army += $this->empire->army->data["heavycruisers"];
		$slice = ($total_army / 5);

	
		/******************************************************************************************/	
		// Condition 3 : Check if you have enough soldiers
		/******************************************************************************************/	
		if ($this->empire->army->data["soldiers"] < $slice) {

			$action = "buy_soldiers=".round($slice - $this->empire->army->data["soldiers"]);
			$comment = LANG_YOU_NEED_TO_TRAIN_MORE_SOLDIERS;
			$results[] = array("comment"=>$comment,"action"=>$action);	
			
		}


		/******************************************************************************************/	
		// Condition 4 : Check if you have enough generals
		/******************************************************************************************/	
		$generals_needed = ceil($this->empire->army->data["soldiers"] / 50);
		if ($generals_needed > $this->empire->army->data["generals"]) {
			$action = "buy_generals=".round($generals_needed - $this->empire->army->data["generals"]);
			$comment = LANG_YOU_NEED_TO_TRAIN_MORE_GENERALS;
			$results[] = array("comment"=>$comment,"action"=>$action);	
		}


		/******************************************************************************************/	
		// Condition 5 : Check if you have enough fighters
		/******************************************************************************************/	
		if ($this->empire->army->data["fighters"] < $slice) {

			$action = "buy_fighters=".round($slice - $this->empire->army->data["fighters"]);
			$comment = LANG_YOU_NEED_TO_TRAIN_MORE_FIGHTERS;
			$results[] = array("comment"=>$comment,"action"=>$action);	
			
		}

		/******************************************************************************************/	
		// Condition 6 : Check if you have enough stations
		/******************************************************************************************/	
		if ($this->empire->army->data["stations"] < $slice) {

			$action = "buy_stations=".round($slice - $this->empire->army->data["stations"]);
			$comment = LANG_YOU_NEED_TO_TRAIN_MORE_STATIONS;
			$results[] = array("comment"=>$comment,"action"=>$action);	
			
		}

		/******************************************************************************************/	
		// Condition 7 : Check if you have enough lightcruisers
		/******************************************************************************************/	
		if ($this->empire->army->data["lightcruisers"] < $slice) {

			$action = "buy_lightcruisers=".round($slice - $this->empire->army->data["lightcruisers"]);
			$comment = LANG_YOU_NEED_TO_TRAIN_MORE_LIGHTCRUISERS;
			$results[] = array("comment"=>$comment,"action"=>$action);	
			
		}

		/******************************************************************************************/	
		// Condition 8 : Check if you have enough heavycruisers
		/******************************************************************************************/	
		if ($this->empire->army->data["heavycruisers"] < $slice) {

			$action = "buy_heavycruisers=".round($slice - $this->empire->army->data["heavycruisers"]);
			$comment = LANG_YOU_NEED_TO_TRAIN_MORE_HEAVYCRUISERS;
			$results[] = array("comment"=>$comment,"action"=>$action);	
			
		}

		/******************************************************************************************/	
		// Condition 9 : Check if you have enough carriers
		/******************************************************************************************/	
		$carriers_needed = 0;
		$carriers_needed += ($this->empire->army->data["soldiers"] * CONF_CARRIER_SOLDIER);
		$carriers_needed += ($this->empire->army->data["generals"] * CONF_CARRIER_GENERAL);
		$carriers_needed += ($this->empire->army->data["fighters"] * CONF_CARRIER_FIGHTER);
		$carriers_needed += ($this->empire->army->data["lightcruisers"] * CONF_CARRIER_LIGHTCRUISER);
		$carriers_needed += ($this->empire->army->data["heavycruisers"] * CONF_CARRIER_HEAVYCRUISER);

		if ($carriers_needed > $this->empire->army->data["carriers"]) {
			$action = "buy_carriers=".round($carriers_needed - $this->empire->army->data["carriers"]);
			$comment = LANG_YOU_NEED_TO_TRAIN_MORE_CARRIERS;
			$results[] = array("comment"=>$comment,"action"=>$action);	
		}

		/******************************************************************************************/	
		// Condition 10 : Check if enough covert agents
		/******************************************************************************************/	
		$covert_required = ($this->empire->planets->GetCount()*2);
		if ($covert_required > $this->empire->army->data["covertagents"]) {
			$action = "buy_covertagents=".round($covert_required - $this->empire->army->data["covertagents"]);
			$comment = LANG_YOU_NEED_TO_TRAIN_MORE_COVERTAGENTS;
			$results[] = array("comment"=>$comment,"action"=>$action);	
		}


		/******************************************************************************************/	
		// Condition 11 : Overall performance overview
		/******************************************************************************************/	
		$avg = 0;
		$count = 0;
		$max = 0;
		$rs = $DB->Execute("SELECT * FROM tb_army WHERE NOT id=".$this->empire->data["id"]);
		while(!$rs->EOF)
		{
			$count++;
			$army = 0;
			$army += $rs->fields["soldiers"];
			$army += $rs->fields["fighters"];
			$army += $rs->fields["stations"];
			$army += $rs->fields["lightcruisers"];
			$army += $rs->fields["heavycruisers"];
			if ($army > $max) $max = $army;
			$avg += $army;
			$rs->MoveNext();
		}
		$avg = floor($avg/($count>0?$count:1));
		
		if ($total_army < $avg) {
			$comment = LANG_YOU_HAVE_A_WEAK_ARMY_PLEASE_INCREASE_ITS_SIZE;
			$action = "nothing";
			$results[] = array("comment"=>$comment,"action"=>$action);	
		} else {

			$avg += (($max - $avg) / 2);
			if ($total_army < $avg) {
				$action = "nothing";
				$comment = LANG_YOU_HAVE_A_NORMAL_ARMY;
				$results[] = array("comment"=>$comment,"action"=>$action);	
			} else {
				$action = "nothing";
				$comment = LANG_YOU_HAVE_A_STRONG_ARMY;
				$results[] = array("comment"=>$comment,"action"=>$action);	
			}

		}

		srand(time(NULL));
		return $results[rand(0,count($results)-1)];
	}
}

?>
