<?php
//---------------------------------------------------------------------------//
// author: Yanick Bourbeau <ybourbeau@mrgtech.ca> 
// date:   2006.01.01
// web:    http://www.mrgtech.ca
// info:   Solar Imperium
//---------------------------------------------------------------------------//
// copyleft license
//
// this software is provided 'as-is', without any express or implied
// warranty. in no event will the authors be held liable for any damages
// arising from the use of this software.
//
// permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions: 
//
// 1. the origin of this software must not be misrepresented;
//	  you must not claim that you wrote the original software. 
//	  if you use this software in a product, an acknowledgment 
//	  in the product documentation would be appreciated but is not required.
//
// 2. altered source versions must be plainly marked as such, 
//	  and must not be misrepresented as being the original software.
//
// 3. mail about the fact of using this class in production 
//	  would be very appreciated.
//
// 4. this notice may not be removed or altered from any source distribution.
//
//---------------------------------------------------------------------------//
// CHANGELOG:
//
// 2006.06.01 : ybo : Changed name: Imperium Galactica is (c), moved to Solar Imperium
// 2006.09.01 : ybo : Major code changes, OOP
// 2007.02.04 : ybo : Reviewed the code for public release
//
//---------------------------------------------------------------------------//

class Empire
{

	var $DB;
	var $TEMPLATE;
	var $data;
	var $data_footprint;
	var $gameplay_costs;
	var $army;
	var $planets;
	var $coalition;
	var $production;
	var $supply;
	var $diplomacy;
	var $research;


	/////////////////////////////////////////////////////////////
	//
	/////////////////////////////////////////////////////////////
	function Empire($DB,$TEMPLATE,$gameplay_costs)
	{
		$this->DB = $DB;
		$this->TEMPLATE = $TEMPLATE;
		$this->gameplay_costs = $gameplay_costs;
		$this->army = new Army($DB,$TEMPLATE);
		$this->planets = new Planets($DB,$TEMPLATE);
		$this->coalition = new Coalition($DB);
		$this->production = new Production($DB,$TEMPLATE);
		$this->supply = new Supply($DB,$TEMPLATE);
		$this->diplomacy = new Diplomacy($DB);
		$this->research = new Research($DB,$TEMPLATE);
		
	}

	/////////////////////////////////////////////////////////////
	//
	/////////////////////////////////////////////////////////////
	function load($id)
	{
		$this->data = $this->DB->Execute("SELECT * FROM tb_empire WHERE id=".addslashes($id));
		if (!$this->data) trigger_error($this->DB->ErrorMsg());
		if ($this->data->EOF) return array("code"=>false,"desc"=>LANG_FATAL_ERROR_WHILE_LOADING_EMPIRE_OBJECT);
		$this->data = $this->data->fields;

		$this->data_footprint = md5(serialize($this->data));

		if (!$this->army->load($id)) return array("code"=>false,"desc"=>LANG_FATAL_ERROR_WHILE_LOADING_ARMY_OBJECT);
		if (!$this->planets->load($id)) return array("code"=>false,"desc"=>LANG_FATAL_ERROR_WHILE_LOADING_PLANETS_OBJECT);
		if (!$this->coalition->load($id)) return array("code"=>false,"desc"=>LANG_FATAL_ERROR_WHILE_LOADING_COALITION_OBJECT);
		if (!$this->production->load($id)) return array("code"=>false,"desc"=>LANG_FATAL_ERROR_WHILE_LOADING_PRODUCTION_OBJECT);
		if (!$this->supply->load($id)) return array("code"=>false,"desc"=>LANG_FATAL_ERROR_WHILE_LOADING_SUPPLY_OBJECT);
		if (!$this->diplomacy->load($id)) return array("code"=>false,"desc"=>LANG_FATAL_ERROR_WHILE_LOADING_DIPLOMACY_OBJECT);
		if (!$this->research->load($id)) return array("code"=>false,"desc"=>LANG_FATAL_ERROR_WHILE_LOADING_RESEARCH_OBJECT);
		return array("code"=>true,"desc"=>null);
	}

	/////////////////////////////////////////////////////////////
	// save, but only if data have changed !
	/////////////////////////////////////////////////////////////
	function save()
	{

		$query = "UPDATE tb_empire SET ";
		reset($this->data);
		while (list($key,$value) = each($this->data))
		{
			if ($key == "id") continue;
			if (is_numeric($key)) continue;
			if ((is_numeric($value)) && ($key != "logo"))
				$query .= "$key=$value,";
			else
				$query .= "$key='".addslashes($value)."',";
			
		}

		$query = substr($query,0,strlen($query)-1); // removing remaining ,
		$query .= " WHERE id=".$this->data["id"];

		$this->DB->beginTrans();		
		if (!$this->DB->Execute($query)) trigger_error($this->DB->ErrorMsg());
		
		$this->army->save();
		$this->planets->save();
		$this->coalition->save();
		$this->production->save();
		$this->supply->save();

		$this->DB->completeTrans();
	}

	/////////////////////////////////////////////////////////////
	//
	/////////////////////////////////////////////////////////////
	function collapse()
	{	
		global $NEWS_DATA;

		$this->data["active"] = 2;
		$this->data["population"] = 0;
		
		$evt = new EventCreator($this->DB);
		$evt->type = CONF_EVENT_COLLAPSEDEMPIRE;
		$evt->from = -1;
		$empire_data = $this->data;
		$evt->params  = array("empire_emperor"=>$this->data["emperor"],"empire_name"=>$this->data["name"],"gender"=>$this->data["gender"]);
		$evt->broadcast();

	
		$rs = $this->DB->Execute("SELECT * FROM tb_tradeconvoy WHERE empire_from=".$this->data["id"]);
		if (!$rs) trigger_error($this->DB->ErrorMsg());
		while(!$rs->EOF) {

			if (!$this->DB->Execute("DELETE FROM tb_tradeconvoy WHERE id=".$rs->fields["id"])) trigger_error($this->DB->ErrorMsg());
			$rs->MoveNext();
		}

		$rs = $this->DB->Execute("SELECT * FROM tb_tradeconvoy WHERE empire_to=".$this->data["id"]);
		if (!$rs) trigger_error($this->DB->ErrorMsg());
		while(!$rs->EOF) {
			if (!$this->DB->Execute("DELETE FROM tb_tradeconvoy WHERE id=".$rs->fields["id"])) trigger_error($this->DB->ErrorMsg());
			$rs->MoveNext();
		}

	
		$rs = $this->DB->Execute("SELECT * FROM tb_armyconvoy WHERE empire_from=".$this->data["id"]);
		if (!$rs) trigger_error($this->DB->ErrorMsg());
		while(!$rs->EOF) {
			if (!$this->DB->Execute("DELETE FROM tb_armyconvoy WHERE id=".$rs->fields["id"])) trigger_error($this->DB->ErrorMsg());
			$rs->MoveNext();
		}

		$rs = $this->DB->Execute("SELECT * FROM tb_armyconvoy WHERE empire_to=".$this->data["id"]);
		if (!$rs) trigger_error($this->DB->ErrorMsg());
		while(!$rs->EOF) {
			if (($rs->fields["convoy_type"] == CONF_CONVOY_INVASION) || ($rs->fields["convoy_type"] == CONF_CONVOY_DEFENSE)) {
				$time_left = ($rs->fields["time_total"] - $rs->fields["time_left"]) + 5;
				$convoy_type = CONF_CONVOY_INVASION_RETREAT;
				if ($rs->fields["convoy_type"] == CONF_CONVOY_DEFENSE)
				$convoy_type = CONF_CONVOY_DEFENSE_RETREAT;

				$query = "UPDATE tb_armyconvoy SET convoy_type=$convoy_type,time_left=$time_left WHERE id=" . $rs->fields["id"];
				$DB->Execute($query);
			}


			$rs->MoveNext();
		}

		$this->planets->releaseAll();
		$this->coalition->transferRandomOwnership();

		// collapse news event
		$news = new NewsCreator($this->DB);
		$news_data = $NEWS_DATA["collapsed_empire"];
		$news_data = $news_data[rand(0,count($news_data)-1)];
	
		$news->title = $news_data["title"];
		$news->text = $news_data["text"];
		$news->text = str_replace("{empire}",$this->data["id"],$news->text);
		$news->text = str_replace("{emperor}",$this->data["emperor"],$news->text);
		$news->text = str_replace("{gender}",$this->data["gender"]=="M"?LANG_EMPEROR:LANG_EMPRESS,$news->text);
		$news->image = $news_data["image"];
		$news->send();

	}

	/////////////////////////////////////////////////////////////
	//
	/////////////////////////////////////////////////////////////
	function delete()
	{
		
		if (!$this->DB->Execute("DELETE FROM tb_army WHERE empire=".$this->data["id"])) trigger_error($this->DB->ErrorMsg());
		if (!$this->DB->Execute("DELETE FROM tb_empire WHERE id=".$this->data["id"])) trigger_error($this->DB->ErrorMsg());
		if (!$this->DB->Execute("DELETE FROM tb_event WHERE event_from=".$this->data["id"])) trigger_error($this->DB->ErrorMsg());
		if (!$this->DB->Execute("DELETE FROM tb_event WHERE event_to=".$this->data["id"])) trigger_error($this->DB->ErrorMsg());
		if (!$this->DB->Execute("DELETE FROM tb_loan WHERE empire=".$this->data["id"])) trigger_error($this->DB->ErrorMsg());
		if (!$this->DB->Execute("DELETE FROM tb_planets WHERE empire=".$this->data["id"])) trigger_error($this->DB->ErrorMsg());
		if (!$this->DB->Execute("DELETE FROM tb_production WHERE empire=".$this->data["id"])) trigger_error($this->DB->ErrorMsg());
		if (!$this->DB->Execute("DELETE FROM tb_stats WHERE empire=".$this->data["id"])) trigger_error($this->DB->ErrorMsg());
		if (!$this->DB->Execute("DELETE FROM tb_supply WHERE empire=".$this->data["id"]))trigger_error($this->DB->ErrorMsg());
		if (!$this->DB->Execute("DELETE FROM tb_tradeconvoy WHERE empire_from=".$this->data["id"])) trigger_error($this->DB->ErrorMsg());
		if (!$this->DB->Execute("DELETE FROM tb_tradeconvoy WHERE empire_to=".$this->data["id"])) trigger_error($this->DB->ErrorMsg());
		if (!$this->DB->Execute("DELETE FROM tb_armyconvoy WHERE empire_from=".$this->data["id"])) trigger_error($this->DB->ErrorMsg());
		/*
			if (!$this->DB->Execute("DELETE FROM tb_armyconvoy WHERE empire_to=".$this->data["id"])) trigger_error($this->DB->ErrorMsg());
		*/

		$rs = $this->DB->Execute("SELECT * FROM tb_armyconvoy WHERE empire_to=".$this->data["id"]);
		if (!$rs) trigger_error($this->DB->ErrorMsg());
		while(!$rs->EOF) {
			if (($rs->fields["convoy_type"] == CONF_CONVOY_INVASION) || ($rs->fields["convoy_type"] == CONF_CONVOY_DEFENSE)) {
				$time_left = ($rs->fields["time_total"] - $rs->fields["time_left"]) + 5;
				$convoy_type = CONF_CONVOY_INVASION_RETREAT;
				if ($rs->fields["convoy_type"] == CONF_CONVOY_DEFENSE)
				$convoy_type = CONF_CONVOY_DEFENSE_RETREAT;

				$query = "UPDATE tb_armyconvoy SET convoy_type=$convoy_type,time_left=$time_left WHERE id=" . $rs->fields["id"];
				$DB->Execute($query);
			}
			$rs->MoveNext();
		}


		if (!$this->DB->Execute("DELETE FROM tb_treaty WHERE empire_from=".$this->data["id"])) trigger_error($this->DB->ErrorMsg());
		if (!$this->DB->Execute("DELETE FROM tb_treaty WHERE empire_to=".$this->data["id"])) trigger_error($this->DB->ErrorMsg());
				
	}

	/////////////////////////////////////////////////////////////
	//
	/////////////////////////////////////////////////////////////
	function checkForEnoughFood()
	{
		global $CONF_CIVIL_STATUS;

		if ($this->data["active"] != 1) return "";
		
		if ($this->data["food"] >= 0) return "";
 
		
		$msg = "<table width=\"100%\"><tr><td><img src=\"images/starving.gif\" style=\"border:0px solid yellow\"></td><td align=\"center\" width=\"100%\">&nbsp;<b style=\"color:yellow\">".LANG_YOUR_POPULATION_IS_STARVING."</b><br/>";

		// First we evaluate how much food is needed
		$food_needed = abs($this->data["food"]);
		
		// we get the foodmarket supply
		$foodmarket = $this->DB->Execute("SELECT food FROM tb_coordinator");
		if (!$foodmarket) trigger_error($this->DB->ErrorMsg());
		$foodmarket = $foodmarket->fields[0];
		// Now we check if the food market have what we need
		if ($foodmarket >= $food_needed)
		{
			$food_prices = $this->gameplay_costs->getFoodPrices();
			$food_total_cost = $food_needed * ($food_prices["buy"]*4);
			
			// then we check for credits
			if ($this->data["credits"] >= $food_total_cost)
			{
				$msg_food = LANG_BOUGHT_FOOD;
					
				$msg_food = str_replace("{food}",
					$this->TEMPLATE->formatFood($food_needed),
					$msg_food
					);

				$msg_food = str_replace("{credits}",
					$this->TEMPLATE->formatCredits($food_total_cost),
					$msg_food
					);

				$msg .= $msg_food;
				
				$this->data["credits"] -= $food_total_cost;
				$this->data["food"] = 0;
				$msg .= "</td></tr></table>";
				
				return $msg;	
			}
		}
		
		$pop_lost = round(($this->data["population"]/100
		*CONF_POPULATION_STARVING));

		$msg_death = LANG_POPULATION_STARVED_TO_DEATH;
		$msg_death = str_replace("{percent}",CONF_POPULATION_STARVING,$msg_death);
		$msg_death = str_replace("{population}",
			$this->TEMPLATE->formatNumber($pop_lost),$msg_death);

		$msg .= $msg_death;
		$this->data["population"] -= $pop_lost;
		$soldiers_lost = round(($this->army->data["soldiers"]/100)
		*CONF_SOLDIERS_STARVING);

		$msg_death = LANG_SOLDIERS_STARVED_TO_DEATH;
		$msg_death = str_replace("{percent}",CONF_SOLDIERS_STARVING,$msg_death);
		$msg_death = str_replace("{soldiers}",
			$this->TEMPLATE->formatNumber($soldiers_lost),$msg_death);
		$msg .= $msg_death;


		$this->army->data["soldiers"] -= $soldiers_lost;


		$this->data["civil_status"]++;

		if ($this->data["civil_status"] > 7) 
			$this->data["civil_status"] = 7;

		$msg .= str_replace("{civil}",
		$CONF_CIVIL_STATUS[$this->data["civil_status"]],
		LANG_POPULATION_IS_ANGRY)."<br/>\r\n";

		$this->data["food"] = 0;
		
		$msg .= "</td></tr></table>";
		return $msg;

	}

	/////////////////////////////////////////////////////////////
	//
	/////////////////////////////////////////////////////////////
	function checkForEnoughCredits()
	{
		if ($this->data["active"] != 1) return "";
		
		global $CONF_CIVIL_STATUS;
		
		if ($this->data["credits"] >= 0) return ""; 
	
		srand(time(NULL));
	
		$msg = "<table width=\"100%\"><tr><td><img src=\"images/bankrupt.gif\" style=\"border:0px solid yellow\"></td><td valign=\"top\" align=\"center\" width=\"100%\">&nbsp;<b style=\"color:yellow\">".LANG_CANNOT_AFFORD_MAINTENANCE_COSTS."<br/>\r\n";

	
		$msg .= str_replace("{percent}",CONF_BANKRUPT_PLANETS,LANG_PLANETS_RELEASED)."<br/>\r\n";
	
		$this->planets->data["food_planets"] -= ceil(($this->planets->data["food_planets"]/100)*CONF_BANKRUPT_PLANETS);
		$this->planets->data["ore_planets"] -= ceil(($this->planets->data["ore_planets"]/100)*CONF_BANKRUPT_PLANETS);
		$this->planets->data["tourism_planets"] -= ceil(($this->planets->data["tourism_planets"]/100)*CONF_BANKRUPT_PLANETS);
		$this->planets->data["supply_planets"] -= ceil(($this->planets->data["supply_planets"]/100)*CONF_BANKRUPT_PLANETS);
		$this->planets->data["gov_planets"] -= ceil(($this->planets->data["gov_planets"]/100)*CONF_BANKRUPT_PLANETS);
		$this->planets->data["edu_planets"] -= ceil(($this->planets->data["edu_planets"]/100)*CONF_BANKRUPT_PLANETS);
		$this->planets->data["research_planets"] -= ceil(($this->planets->data["research_planets"]/100)*CONF_BANKRUPT_PLANETS);
		$this->planets->data["urban_planets"] -= ceil(($this->planets->data["urban_planets"]/100)*CONF_BANKRUPT_PLANETS);
		$this->planets->data["petro_planets"] -= ceil(($this->planets->data["petro_planets"]/100)*CONF_BANKRUPT_PLANETS);
		$this->planets->data["antipollu_planets"] -= ceil(($this->planets->data["antipollu_planets"]/100)*CONF_BANKRUPT_PLANETS);

		

		$msg .= str_replace("{percent}",CONF_BANKRUPT_MILITARY,LANG_ARMY_DISBANDED)."<br/>\r\n";

		$this->army->data["soldiers"] -= ceil(($this->army->data["soldiers"]/100) * CONF_BANKRUPT_MILITARY);
		$this->army->data["fighters"] -= ceil(($this->army->data["fighters"]/100) * CONF_BANKRUPT_MILITARY);
		$this->army->data["stations"] -= ceil(($this->army->data["stations"]/100) * CONF_BANKRUPT_MILITARY);
		$this->army->data["lightcruisers"] -= ceil(($this->army->data["lightcruisers"]/100) * CONF_BANKRUPT_MILITARY);
		$this->army->data["heavycruisers"] -= ceil(($this->army->data["heavycruisers"]/100) * CONF_BANKRUPT_MILITARY);
		$this->army->data["carriers"] -= ceil(($this->army->data["carriers"]/100) * CONF_BANKRUPT_MILITARY);
		
		
		if (rand(0,5) == 0)
		{
			$this->army->data["effectiveness"] -= 5;
			if ($this->army->data["effectiveness"] < 0) $this->army->data["effectiveness"];
			$msg .= str_replace("{percent}",$this->army->data["effectiveness"],LANG_ARMY_EFFECTIVENESS_DROPPED)."<br/>\r\n";

		}

		
		if (rand(0,5) == 0)
		{
			$this->data["civil_status"]++;
			if ($this->data["civil_status"] > 7) $this->data["civil_status"] = 7;
			$msg .= str_replace("{civil}",
			$CONF_CIVIL_STATUS[$this->data["civil_status"]],
			LANG_POPULATION_IS_ANGRY)."<br/>\r\n";
		}
				
		$this->data["credits"] = 0;

		$msg .="</td></tr></table>";
		return $msg;

	}

	/////////////////////////////////////////////////////////////
	//
	/////////////////////////////////////////////////////////////
	function checkForEnoughPopulation()
	{
		if ($this->data["admin"] == 1) return true;


		// you are dead, jim
		if (($this->data["population"] < 10) || ($this->planets->getCount() == 0))
			return false;

		return true;
		
	
	}


	/////////////////////////////////////////////////////////
	//
	/////////////////////////////////////////////////////////
	function updateNetworth()
	{

		$networth = 0;
		$networth += ($this->data["population"] * CONF_NETWORTH_POPULATION);
		$networth += ($this->data["credits"] * CONF_NETWORTH_CREDITS);
	
		$planets = 0;
		$planets += $this->planets->data["food_planets"];
		$planets += $this->planets->data["ore_planets"];
		$planets += $this->planets->data["tourism_planets"];
		$planets += $this->planets->data["supply_planets"];
		$planets += $this->planets->data["gov_planets"];
		$planets += $this->planets->data["edu_planets"];
		$planets += $this->planets->data["research_planets"];
		$planets += $this->planets->data["urban_planets"];
		$planets += $this->planets->data["petro_planets"];
		$planets += $this->planets->data["antipollu_planets"];
		$networth += ($planets * CONF_NETWORTH_PLANETS);

		$army = 0;
		$army += $this->army->data["soldiers"] * CONF_NETWORTH_MILITARY_SOLDIER;
		$army += $this->army->data["fighters"]* CONF_NETWORTH_MILITARY_FIGHTER;
		$army += $this->army->data["stations"]* CONF_NETWORTH_MILITARY_STATION;
		$army += $this->army->data["lightcruisers"]* CONF_NETWORTH_MILITARY_LIGHTCRUISER;
		$army += $this->army->data["heavycruisers"]* CONF_NETWORTH_MILITARY_HEAVYCRUISER;
		$army += $this->army->data["carriers"]* CONF_NETWORTH_MILITARY_CARRIER;
		$army += $this->army->data["generals"]* CONF_NETWORTH_MILITARY_GENERAL;
		$army += $this->army->data["covertagents"]* CONF_NETWORTH_MILITARY_COVERT;

		$networth += ($army);
		$this->data["networth"] = floor($networth);

                if ($this->coalition->IsMember())
               		$this->coalition->updateNetworth($this->data["networth"]);
	}


	/////////////////////////////////////////////////////////
	//
	/////////////////////////////////////////////////////////
	function updateStats()
	{
		$pollution = ($this->planets->data["petro_planets"] * CONF_PETRO_POLLUTION);
		$pollution += ($this->data["population"] * CONF_POP_POLLUTION);
		$pollution = round($pollution,2);

		$army = 0;
		$army += $this->army->data["soldiers"] * CONF_NETWORTH_MILITARY_SOLDIER;
		$army += $this->army->data["fighters"]* CONF_NETWORTH_MILITARY_FIGHTER;
		$army += $this->army->data["stations"]* CONF_NETWORTH_MILITARY_STATION;
		$army += $this->army->data["lightcruisers"]* CONF_NETWORTH_MILITARY_LIGHTCRUISER;
		$army += $this->army->data["heavycruisers"]* CONF_NETWORTH_MILITARY_HEAVYCRUISER;
		$army += $this->army->data["carriers"]* CONF_NETWORTH_MILITARY_CARRIER;
		$army += $this->army->data["generals"]* CONF_NETWORTH_MILITARY_GENERAL;
		$army += $this->army->data["covertagents"]* CONF_NETWORTH_MILITARY_COVERT;


		$query = "INSERT INTO tb_stats (empire,date,credits,food,networth,military,planets,population,pollution,turn) 
		VALUES(".
		$this->data["id"].",".
		time(NULL).",".
		$this->data["credits"].",".
		$this->data["food"].",".
		$this->data["networth"].",".
		$army.",".
		$this->planets->getCount().",".
		$this->data["population"].",".
		$pollution.",".
		$this->data["turns_played"].");";

		if (!$this->DB->Execute($query)) trigger_error($this->DB->ErrorMsg());
	}


}



?>
