<?php
//---------------------------------------------------------------------------//
// author: Yanick Bourbeau <ybourbeau@mrgtech.ca> 
// date:   2006.01.01
// web:    http://www.mrgtech.ca
// info:   Solar Imperium
//---------------------------------------------------------------------------//
// copyleft license
//
// this software is provided 'as-is', without any express or implied
// warranty. in no event will the authors be held liable for any damages
// arising from the use of this software.
//
// permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions: 
//
// 1. the origin of this software must not be misrepresented;
//	  you must not claim that you wrote the original software. 
//	  if you use this software in a product, an acknowledgment 
//	  in the product documentation would be appreciated but is not required.
//
// 2. altered source versions must be plainly marked as such, 
//	  and must not be misrepresented as being the original software.
//
// 3. mail about the fact of using this class in production 
//	  would be very appreciated.
//
// 4. this notice may not be removed or altered from any source distribution.
//
//---------------------------------------------------------------------------//
// CHANGELOG:
//
// 2006.06.01 : ybo : Changed name: Imperium Galactica is (c), moved to Solar Imperium
// 2006.09.01 : ybo : Major code changes, OOP
// 2007.02.04 : ybo : Reviewed the code for public release
//
//---------------------------------------------------------------------------//

class EventRenderer
{
	var $DB;
	var $TEMPLATE;
	var $GAME_TPL;
	var $height;

	//////////////////////////////////////////////////////////////////////////////
	//
	//////////////////////////////////////////////////////////////////////////////
	function EventRenderer($DB,$TEMPLATE,$GAME_TPL)
	{
		$this->DB = $DB;
		$this->TEMPLATE = $TEMPLATE;
		$this->GAME_TPL = $GAME_TPL;
		$this->height = 160;
	}

	//////////////////////////////////////////////////////////////////////////////
	//
	//////////////////////////////////////////////////////////////////////////////
	function displayUnseenEvents($empire_data)
	{
		$output = array();
		
		$rs = $this->DB->Execute("SELECT * FROM tb_event WHERE event_to=".$empire_data["id"]." AND seen=0 ORDER BY id DESC LIMIT 0,30");
		if (!$rs) trigger_error($this->DB->ErrorMsg());
		$total_height = 0;
		while(!$rs->EOF)
		{
			$output[] = $this->renderEvent($rs->fields,$empire_data);
			$total_height += $this->height;	
			$rs->MoveNext();
		}
		return array("total_height"=>$total_height,"events_output"=>$output);
	}

	//////////////////////////////////////////////////////////////////////////////
	//
	//////////////////////////////////////////////////////////////////////////////
	function renderEvent($event_data,$empire_data)
	{
		global $CONF_DIPLOMACY_TREATIES,$CONF_CIVIL_STATUS;
		
		$output = "";
		$this->height = $event_data["height"];
				
		switch($event_data["event_type"])
		{

			/********************** NEW EMPIRE ! *********************************/
			case CONF_EVENT_NEWEMPIRE:
				$this->TEMPLATE->set_file("event_data","event/newempire.html");
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_var("gender",$params["gender"]=="M"?LANG_EMPEROR:LANG_EMPRESS);
				$this->TEMPLATE->set_var("emperor",$params["empire_emperor"]);
				$this->TEMPLATE->set_var("empire",$params["empire_name"]);
				if (!file_exists("images/empires/".$event_data["event_from"].".jpg"))
					$this->TEMPLATE->set_var("logo","img_logo.php?empire=".$event_data["event_from"]);
				else
					$this->TEMPLATE->set_var("logo","images/empires/".$event_data["event_from"].".jpg");
				
			break;			

			/********************** COLLAPSED EMPIRE ! *********************************/
			case CONF_EVENT_COLLAPSEDEMPIRE:
				$this->TEMPLATE->set_file("event_data","event/collapsedempire.html");
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_var("gender",$params["gender"]=="M"?LANG_EMPEROR:LANG_EMPRESS);
				$this->TEMPLATE->set_var("emperor",$params["empire_emperor"]);
				$this->TEMPLATE->set_var("empire",$params["empire_name"]);
				
				if (!file_exists("images/empires/".$event_data["event_from"].".jpg"))
					$this->TEMPLATE->set_var("logo","img_logo.php?empire=".$event_data["event_from"]);
				else
					$this->TEMPLATE->set_var("logo","images/empires/".$event_data["event_from"].".jpg");
					
				
			break;			


			/*********************** EMPIRE INVADED ! *********************************/
			case CONF_EVENT_INVADED:
				
				$this->TEMPLATE->set_file("event_data","event/invaded.html");
				$params = unserialize($event_data["params"]);
				if ($params["won"]==false) $params["won"] = LANG_INVASION_FAILED;
					else $params["won"] = LANG_INVASION_WON;

				while(list($key,$value) = each($params))
					$this->TEMPLATE->set_var($key,$value);
			break;


			/*********************** EMPIRE ELIMINATED ! *********************************/
			case CONF_EVENT_ELIMINATED:

				$this->TEMPLATE->set_file("event_data","event/eliminated.html");
				$params = unserialize($event_data["params"]);

				while(list($key,$value) = each($params))
					$this->TEMPLATE->set_var($key,$value);
			break;


			/*********************** EMPIRE ATTACKED ! *********************************/
			case CONF_EVENT_EMPIREATTACKED:
				
				$this->TEMPLATE->set_file("event_data","event/empire_attacked_guerilla.html");
				$params = unserialize($event_data["params"]);
				if ($params["won"]==false) $params["won"] = LANG_ASSAULT_FAILED;
					else $params["won"] = LANG_ASSAULT_WON;
				while(list($key,$value) = each($params))
					$this->TEMPLATE->set_var($key,$value);
			break;
			

			/********************** PLANETS RELEASED ! *********************************/
			case CONF_EVENT_PLANETS_RELEASED:
				$this->TEMPLATE->set_file("event_data","event/planets_released.html");
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_var("emperor",$params["empire_emperor"]);
				$this->TEMPLATE->set_var("empire",$params["empire_name"]);
				$this->TEMPLATE->set_var("gender",$params["gender"]=="M"?LANG_EMPEROR:LANG_EMPRESS);
				
				if (!file_exists("images/empires/".$params["empire_id"].".jpg"))
					$this->TEMPLATE->set_var("logo","img_logo.php?empire=".$params["empire_id"]);
				else
					$this->TEMPLATE->set_var("logo","images/empires/".$params["empire_id"].".jpg");
				
				$this->TEMPLATE->set_var("food_planets",$this->GAME_TPL->formatNumber($params["planets_data"]["food_planets"]));
				$this->TEMPLATE->set_var("ore_planets",$this->GAME_TPL->formatNumber($params["planets_data"]["ore_planets"]));
				$this->TEMPLATE->set_var("tourism_planets",$this->GAME_TPL->formatNumber($params["planets_data"]["tourism_planets"]));
				$this->TEMPLATE->set_var("supply_planets",$this->GAME_TPL->formatNumber($params["planets_data"]["supply_planets"]));
				$this->TEMPLATE->set_var("gov_planets",$this->GAME_TPL->formatNumber($params["planets_data"]["gov_planets"]));
				$this->TEMPLATE->set_var("edu_planets",$this->GAME_TPL->formatNumber($params["planets_data"]["edu_planets"]));
				$this->TEMPLATE->set_var("research_planets",$this->GAME_TPL->formatNumber($params["planets_data"]["research_planets"]));
				$this->TEMPLATE->set_var("urban_planets",$this->GAME_TPL->formatNumber($params["planets_data"]["urban_planets"]));
				$this->TEMPLATE->set_var("petro_planets",$this->GAME_TPL->formatNumber($params["planets_data"]["petro_planets"]));
				$this->TEMPLATE->set_var("antipollu_planets",$this->GAME_TPL->formatNumber($params["planets_data"]["antipollu_planets"]));
			
			break;
			
			/********************** MESSAGE RECEIVED ! *********************************/
			case CONF_EVENT_MESSAGE:
				
				if (($event_data["sticky"]==0) && ($event_data["seen"]==0)) {
					if (!$this->DB->Execute("UPDATE tb_event SET seen=1 WHERE id=".$event_data["id"])) trigger_error($this->DB->ErrorMsg());
				}

				return $this->GAME_TPL->showMessage($event_data);
			break;

			/********************** COALITION CREATED ! *********************************/
			case CONF_EVENT_COALITION_CREATED:
			
				$this->TEMPLATE->set_file("event_data","event/coalition_created.html");
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_var("emperor",$params["empire_emperor"]);
				$this->TEMPLATE->set_var("empire",$params["empire_name"]);
				$this->TEMPLATE->set_var("empire_id",$params["empire_id"]);
				$this->TEMPLATE->set_var("gender",$params["gender"]=="M"?LANG_EMPEROR:LANG_EMPRESS);
				$this->TEMPLATE->set_var("coalition_name",$params["coalition_name"]);
				
				
			break;
			
			/********************** COALITION KICKED ! *********************************/
			case CONF_EVENT_COALITION_KICKED:
			
				$this->TEMPLATE->set_file("event_data","event/coalition_kicked.html");
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_var("emperor",$params["empire_emperor"]);
				$this->TEMPLATE->set_var("empire",$params["empire_name"]);
				$this->TEMPLATE->set_var("empire_id",$params["empire_id"]);
				$this->TEMPLATE->set_var("gender",$params["gender"]=="M"?LANG_EMPEROR:LANG_EMPRESS);
				$this->TEMPLATE->set_var("coalition_name",$params["coalition_name"]);
				
			break;
			
			/********************** COALITION JOINED ! *********************************/
			case CONF_EVENT_COALITION_JOINED:
			
				$this->TEMPLATE->set_file("event_data","event/coalition_joined.html");
				$params = unserialize($event_data["params"]);
				
				$this->TEMPLATE->set_var("emperor",$params["empire_emperor"]);
				$this->TEMPLATE->set_var("empire",$params["empire_name"]);
				$this->TEMPLATE->set_var("empire_id",$params["empire_id"]);
				$this->TEMPLATE->set_var("gender",$params["gender"]=="M"?LANG_EMPEROR:LANG_EMPRESS);
				
			break;
			
			/********************** COALITION REFUSED ! *********************************/
			case CONF_EVENT_COALITION_REFUSED:
			
				$this->TEMPLATE->set_file("event_data","event/coalition_refused.html");
				$params = unserialize($event_data["params"]);
				
				$this->TEMPLATE->set_var("emperor",$params["empire_emperor"]);
				$this->TEMPLATE->set_var("empire",$params["empire_name"]);
				$this->TEMPLATE->set_var("empire_id",$params["empire_id"]);
				$this->TEMPLATE->set_var("gender",$params["gender"]=="M"?LANG_EMPEROR:LANG_EMPRESS);
				
			break;
			

			/********************** COALITION DISBANDED ! *********************************/
			case CONF_EVENT_COALITION_DISBANDED:
			
				$this->TEMPLATE->set_file("event_data","event/coalition_disbanded.html");
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_var("emperor",$params["empire_emperor"]);
				$this->TEMPLATE->set_var("empire_id",$params["empire_id"]);
				$this->TEMPLATE->set_var("empire",$params["empire_name"]);
				$this->TEMPLATE->set_var("gender",$params["gender"]=="M"?LANG_EMPEROR:LANG_EMPRESS);
				$this->TEMPLATE->set_var("coalition_name",$params["coalition_name"]);
				
			break;
			
			/********************** NEW TURN ! *********************************/
			case CONF_EVENT_NEWTURN:
			
				$this->TEMPLATE->set_file("event_data","event/new_turn.html");
				$params = unserialize($event_data["params"]);
				while(list($key,$value) = each($params))
					$this->TEMPLATE->set_var($key,$value);
			break;
			
			/**********************  TURN COMPLETED ! *********************************/
			case CONF_EVENT_TURNCOMPLETED:
			
				$this->TEMPLATE->set_file("event_data","event/turn_completed.html");
				$params = unserialize($event_data["params"]);
				while(list($key,$value) = each($params))
					$this->TEMPLATE->set_var($key,$value);
			break;
			
			/**********************  INCOMING_INVASION ! *********************************/
			case CONF_EVENT_INCOMING_INVASION:
			
				$this->TEMPLATE->set_file("event_data","event/incoming_invasion.html");
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_var("soldiers",$this->GAME_TPL->formatNumber($params["soldiers"]));
				$this->TEMPLATE->set_var("generals",$this->GAME_TPL->formatNumber($params["generals"]));
				$this->TEMPLATE->set_var("fighters",$this->GAME_TPL->formatNumber($params["fighters"]));
				$this->TEMPLATE->set_var("lightcruisers",$this->GAME_TPL->formatNumber($params["lightcruisers"]));
				$this->TEMPLATE->set_var("heavycruisers",$this->GAME_TPL->formatNumber($params["heavycruisers"]));
				$rs = $this->DB->Execute("SELECT * FROM tb_empire WHERE id=".addslashes($event_data["event_from"]));
				if (!$rs) trigger_error($this->DB->ErrorMsg());
				$empire = $this->GAME_TPL->displayEmpireHTML($rs->fields["id"],$rs->fields["emperor"],$rs->fields["name"],$empire_data["networth"]);
				$this->TEMPLATE->set_var("empire",$empire);

			break;

			/**********************  INCOMING_DEFENSE ! *********************************/
			case CONF_EVENT_INCOMING_DEFENSE:
			
				$this->TEMPLATE->set_file("event_data","event/incoming_defense.html");
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_var("soldiers",$this->GAME_TPL->formatNumber($params["soldiers"]));
				$this->TEMPLATE->set_var("generals",$this->GAME_TPL->formatNumber($params["generals"]));
				$this->TEMPLATE->set_var("fighters",$this->GAME_TPL->formatNumber($params["fighters"]));
				$this->TEMPLATE->set_var("lightcruisers",$this->GAME_TPL->formatNumber($params["lightcruisers"]));
				$this->TEMPLATE->set_var("heavycruisers",$this->GAME_TPL->formatNumber($params["heavycruisers"]));
				$rs = $this->DB->Execute("SELECT * FROM tb_empire WHERE id=".addslashes($event_data["event_from"]));
				$empire = $this->GAME_TPL->displayEmpireHTML($rs->fields["id"],$rs->fields["emperor"],$rs->fields["name"],$empire_data["networth"]);
				$this->TEMPLATE->set_var("empire",$empire);

			break;


			/**********************  SENDING_DEFENSE ! *********************************/
			case CONF_EVENT_SENDING_DEFENSE:
			
				$this->TEMPLATE->set_file("event_data","event/sending_defense.html");
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_var("soldiers",$this->GAME_TPL->formatNumber($params["soldiers"]));
				$this->TEMPLATE->set_var("generals",$this->GAME_TPL->formatNumber($params["generals"]));
				$this->TEMPLATE->set_var("fighters",$this->GAME_TPL->formatNumber($params["fighters"]));
				$this->TEMPLATE->set_var("lightcruisers",$this->GAME_TPL->formatNumber($params["lightcruisers"]));
				$this->TEMPLATE->set_var("heavycruisers",$this->GAME_TPL->formatNumber($params["heavycruisers"]));
				$rs = $this->DB->Execute("SELECT * FROM tb_empire WHERE id=".addslashes($event_data["event_from"]));
				$empire = $this->GAME_TPL->displayEmpireHTML($rs->fields["id"],$rs->fields["emperor"],$rs->fields["name"],$empire_data["networth"]);
				$this->TEMPLATE->set_var("empire",$empire);

			break;


			/********************** PIRATE RAID *********************************/
			case CONF_EVENT_PIRATERAID:
		
				$this->TEMPLATE->set_file("event_data","event/pirateraid.html");
				$params = unserialize($event_data["params"]);
				while(list($key,$value) = each($params))
					$this->TEMPLATE->set_var($key,$value);
			
			break;
			
			/********************** RESEARCH DONE *********************************/
			case CONF_EVENT_RESEARCHDONE:
				$this->TEMPLATE->set_file("event_data","event/researchdone.html");
				$params = unserialize($event_data["params"]);
				while(list($key,$value) = each($params))
					$this->TEMPLATE->set_var($key,$value);
		
			break;

			/********************** FOOD GROWTH *********************************/
			case CONF_EVENT_FOODGROWTH:
				$this->TEMPLATE->set_file("event_data","event/foodgrowth.html");
				$params = unserialize($event_data["params"]);
				if ($params["growrate"] > 0) $params["grow_color"] = "lightgreen"; else $params["grow_color"] = "#FFAAAA";

				while(list($key,$value) = each($params))
					$this->TEMPLATE->set_var($key,$value);
					
			break;
			
			/********************** MONEY GROWTH *********************************/
			case CONF_EVENT_MONEYGROWTH:
				$this->TEMPLATE->set_file("event_data","event/moneygrowth.html");
				$params = unserialize($event_data["params"]);
				if ($params["growrate"] > 0) $params["grow_color"] = "lightgreen"; else $params["grow_color"] = "#FFAAAA";

				while(list($key,$value) = each($params))
					$this->TEMPLATE->set_var($key,$value);
		
			break;

			/********************** FUNDAMENTAL COMPLETED *********************************/
			case CONF_EVENT_FUNDAMENTAL_COMPLETED:
				$this->TEMPLATE->set_file("event_data","event/fundamental_research.html");
				$params = unserialize($event_data["params"]);

				while(list($key,$value) = each($params))
					$this->TEMPLATE->set_var($key,$value);
		
			break;

			/********************** MILITARY PRODUCTION *********************************/
			case CONF_EVENT_MILITARYPRODUCTION:
				$this->TEMPLATE->set_file("event_data","event/military_production.html");
				$params = unserialize($event_data["params"]);
				while(list($key,$value) = each($params))
					$this->TEMPLATE->set_var($key,$value);
					
			break;
	
			/********************** NOTICE *********************************/
			case CONF_EVENT_NOTICE:
				$this->TEMPLATE->set_file("event_data","event/notice.html");
				$params = unserialize($event_data["params"]);
				while(list($key,$value) = each($params))
					$this->TEMPLATE->set_var($key,$value);
					
			break;
			
			
			/********************** POPULATION GROWTH *********************************/
			case CONF_EVENT_POPULATIONGROWTH:
				$this->TEMPLATE->set_file("event_data","event/populationgrowth.html");
				$params = unserialize($event_data["params"]);
				if ($params["growrate"] > 0) $params["grow_color"] = "lightgreen"; else $params["grow_color"] = "#FFAAAA";
				while(list($key,$value) = each($params))
					$this->TEMPLATE->set_var($key,$value);
					
			break;

			/********************** BREAK TREATY *********************************/
			case CONF_EVENT_BREAKTREATY:
				$this->TEMPLATE->set_file("event_data","event/diplomacy_breaktreaty.html");
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_var("empire",$this->GAME_TPL->displayEmpireHTML($params["empire_id"],$params["empire_emperor"],$params["empire_name"],$empire_data["networth"]));
				if (!$this->DB->Execute("DELETE FROM tb_treaty WHERE empire_from=".$event_data["event_to"]." AND empire_to=".$event_data["event_from"])) trigger_error($this->DB->ErrorMsg());
				if (!$this->DB->Execute("DELETE FROM tb_treaty WHERE empire_to=".$event_data["event_to"]." AND empire_from=".$event_data["event_from"])) trigger_error($this->DB->ErrorMsg());
			break;
			
			/********************** GUERILLA REVEALED *********************************/
			case CONF_EVENT_GUERILLA_REVEALED:
				$this->TEMPLATE->set_file("event_data","event/guerilla_revealed.html");
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_var("emperor",$params["empire_emperor"]);
				$this->TEMPLATE->set_var("empire_id",$params["empire_id"]);
				$this->TEMPLATE->set_var("empire",$params["empire_name"]);
				$this->TEMPLATE->set_var("lost_soldiers",$this->GAME_TPL->formatNumber($params["lost_soldiers"]));
				$this->TEMPLATE->set_var("gender",$params["gender"]=="M"?LANG_EMPEROR:LANG_EMPRESS);
			break;
			
			/********************** GUERILLA STEALTH *********************************/
			case CONF_EVENT_GUERILLA_STEALTH:
				$this->TEMPLATE->set_file("event_data","event/guerilla_stealth.html");
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_var("emperor","???");
				$this->TEMPLATE->set_var("empire_id","new");
				$this->TEMPLATE->set_var("empire","???");
				$this->TEMPLATE->set_var("gender","???");
				$this->TEMPLATE->set_var("lost_soldiers",$this->GAME_TPL->formatNumber($params["lost_soldiers"]));
			break;
			
			/********************** RANDOM EVENT *********************************/
			case CONF_EVENT_RANDOMEVENT:
				$this->TEMPLATE->set_file("event_data","event/randomevent.html");
				$params = unserialize($event_data["params"]);
				while(list($key,$value) = each($params))
					$this->TEMPLATE->set_var($key,$value);
					
			break;
			
			/********************** CIVIL STATUS *********************************/
			case CONF_EVENT_CIVILSTATUS:
				$this->TEMPLATE->set_file("event_data","event/civilstatus.html");
				$params = unserialize($event_data["params"]);
				while(list($key,$value) = each($params))
					$this->TEMPLATE->set_var($key,$value);
					
			break;
			
			/********************** COVERT OP RESULT *********************************/
			case CONF_EVENT_COVERTOP_RESULT:

				$params = unserialize($event_data["params"]);
				if (($event_data["sticky"]==0) && ($event_data["seen"]==0))
					if (!$this->DB->Execute("UPDATE tb_event SET seen=1 WHERE id=".$event_data["id"])) trigger_error($this->DB->ErrorMsg());
				return $params["result"];									
			break;


			/********************** PIRATE BUST **********************************/
			case CONF_EVENT_PIRATEBUST:

				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_file("event_data","event/piratebust.html");
				$this->TEMPLATE->set_var("result",$params["result"]);
			break;

			/********************** TRADE CONVOY RECV  ********************************/
			case CONF_EVENT_TRADECONVOY_RECEIVED:

				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_file("event_data","event/tradeconvoy_received.html");
				while(list($key,$value) = each($params)) {
				
					if (($key == "empire_to") || ($key == "empire_from")) {
						$rs = $this->DB->Execute("SELECT * FROM tb_empire WHERE id=".addslashes($value));
						if (!$rs) trigger_error($this->DB->ErrorMsg());
						$value = $this->GAME_TPL->displayEmpireHTML($rs->fields["id"],$rs->fields["emperor"],$rs->fields["name"],$empire_data["networth"]);
					}
					
					if (is_numeric($value)) $value = $this->GAME_TPL->formatNumber($value);
					$this->TEMPLATE->set_var($key,$value);
				}
					
				if (($event_data["sticky"]==0) && ($event_data["seen"]==0)) {
					if (!$this->DB->Execute("UPDATE tb_event SET seen=1 WHERE id=".$event_data["id"])) trigger_error($this->DB->ErrorMsg());
				}
			break;

			/********************** CONF_EVENT_COALITION_INVITE  ********************************/
			case CONF_EVENT_COALITION_INVITE:
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_file("event_data","event/coalition_invite.html");
				$this->TEMPLATE->set_var("empire",$this->GAME_TPL->displayEmpireHTML($params["empire_id"],$params["empire_emperor"],$params["empire_name"],$empire_data["networth"]));
				$this->TEMPLATE->set_var("coalition_name",$params["coalition_name"]);			
				$this->TEMPLATE->set_var("event_from",$event_data["event_from"]);			
			break;
			
			/********************** CONF_EVENT_CONVOY_RETREAT  ********************************/
			case CONF_EVENT_CONVOY_RETREAT:
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_file("event_data","event/convoy_retreat.html");
				$rs = $this->DB->Execute("SELECT id,networth,emperor,name FROM tb_empire WHERE id=".$params);
				$this->TEMPLATE->set_var("empire",
				$this->GAME_TPL->displayEmpireHTML(
				$rs->fields["id"],
				$rs->fields["emperor"],
				$rs->fields["name"],
				$rs->fields["networth"]));
			break;
			
			/********************** CONF_EVENT_COALITION_OWNERSHIP_CHANGED *********************/
			case CONF_EVENT_COALITION_OWNERSHIP_CHANGED:
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_file("event_data","event/coalition_ownership_changed.html");
				$this->TEMPLATE->set_var("coalition_name",$params["coalition_name"]);			
				$this->TEMPLATE->set_var("empire",$this->GAME_TPL->displayEmpireHTML($params["empire_id"],$params["empire_emperor"],$params["empire_name"],$empire_data["networth"]));
			break;
			
			/********************** CONF_EVENT_PENDINGTREATY  ********************************/
			case CONF_EVENT_PENDINGTREATY:
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_file("event_data","event/diplomacy_pendingtreaty.html");
				$this->TEMPLATE->set_var("empire",$this->GAME_TPL->displayEmpireHTML($params["empire_id"],$params["empire_emperor"],$params["empire_name"],$empire_data["networth"]));
				$this->TEMPLATE->set_var("event_from",$event_data["event_from"]);			
				$this->TEMPLATE->set_var("treaty",$CONF_DIPLOMACY_TREATIES[$params["treaty"]]);
			break;


			/********************** CONF_EVENT_TRADECONVOY_RAIDEDBYPIRATE  ********************************/
			case CONF_EVENT_TRADECONVOY_RAIDEDBYPIRATE:
				$this->TEMPLATE->set_file("event_data","event/pirate_ambush.html");
				$params = unserialize($event_data["params"]);
				while(list($key,$value) = each($params))
					$this->TEMPLATE->set_var($key,$this->GAME_TPL->formatNumber($value));
			break;


			/********************** CONF_EVENT_SPYCAUGHT  ********************************/
			case CONF_EVENT_SPYCAUGHT:
				$this->TEMPLATE->set_file("event_data","event/spy_caught.html");
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_var("empire",$this->GAME_TPL->displayEmpireHTML($params["empire_id"],$params["empire_emperor"],$params["empire_name"],$empire_data["networth"]));
				$this->TEMPLATE->set_var("opname",$params["opname"]);			
			break;


			/********************** CONF_EVENT_DISSENSION  ********************************/
			case CONF_EVENT_DISSENSION:
				$this->TEMPLATE->set_file("event_data","event/dissension.html");
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_var("content",$params["content"]);			
			break;

			/********************** CONF_EVENT_FOODBOMBED  ********************************/
			case CONF_EVENT_FOODBOMBED:
				$this->TEMPLATE->set_file("event_data","event/foodbombed.html");
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_var("content",$params["content"]);			
			break;

			/********************** CONF_EVENT_CARRIERS_SABOTAGE  ********************************/
			case CONF_EVENT_CARRIERS_SABOTAGE:
				$this->TEMPLATE->set_file("event_data","event/carriers_sabotage.html");
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_var("content",$params["content"]);			
			break;

			/********************** CONF_EVENT_HOSTAGES  ********************************/
			case CONF_EVENT_HOSTAGES:
				$this->TEMPLATE->set_file("event_data","event/hostages.html");
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_var("credits",$this->GAME_TPL->formatCredits($params["credits_lost"]));			
			break;


			/********************** CONF_EVENT_ACCEPTEDTREATY  ********************************/
			case CONF_EVENT_ACCEPTEDTREATY:
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_file("event_data","event/diplomacy_acceptedtreaty.html");
				$this->TEMPLATE->set_var("empire",$this->GAME_TPL->displayEmpireHTML($params["empire_id"],$params["empire_emperor"],$params["empire_name"],$empire_data["networth"]));
				$this->TEMPLATE->set_var("event_from",$event_data["event_from"]);			
			break;

			/********************** CONF_EVENT_REFUSEDTREATY  ********************************/
			case 	CONF_EVENT_REFUSEDTREATY:
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_file("event_data","event/diplomacy_refusedtreaty.html");
				$this->TEMPLATE->set_var("empire",$this->GAME_TPL->displayEmpireHTML($params["empire_id"],$params["empire_emperor"],$params["empire_name"],$empire_data["networth"]));
				$this->TEMPLATE->set_var("event_from",$event_data["event_from"]);			
			break;


			/********************** CONF_EVENT_BOND_PAYOUT ********************************/
			case CONF_EVENT_BOND_PAYOUT:
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_file("event_data","event/bond_payout.html");
				$this->TEMPLATE->set_var("current_credits",$this->GAME_TPL->formatCredits($params["current_credits"]));
			
			break;
			
			/********************** CONF_EVENT_LOTTERYWINNER ********************************/
			case CONF_EVENT_LOTTERYWINNER:
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_file("event_data","event/lottery_winner.html");
				$this->TEMPLATE->set_var("lottery_cash",$this->GAME_TPL->formatCredits($params["lottery_cash"]));
				$this->TEMPLATE->set_var("gender",($params["gender"]=="M"?LANG_EMPEROR:LANG_EMPRESS));
				$this->TEMPLATE->set_var("emperor",stripslashes($params["empire_emperor"]));
				$this->TEMPLATE->set_var("empire",stripslashes($params["empire_name"]));
				
				$logo = "images/empires/".$params["empire_id"].".jpg";
				if (!file_exists($logo)) $logo = "images/empires/new.jpg";
				$this->TEMPLATE->set_var("logo",$logo);
			
			break;
	

			/********************** CONF_EVENT_EMPIRE_TELEPORTED ********************************/
			case CONF_EVENT_EMPIRE_TELEPORTED:
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_file("event_data","event/empire_teleported.html");
				$this->TEMPLATE->set_var("empire",$this->GAME_TPL->displayEmpireHTML($params["empire_id"],$params["empire_emperor"],$params["empire_name"],$empire_data["networth"]));
			break;


			/********************** CONF_EVENT_EMPIRE_BUY_NUKES ********************************/
			case CONF_EVENT_EMPIRE_BUY_NUKES:
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_file("event_data","event/empire_buy_nukes.html");
			break;

			/********************** CONF_EVENT_NUCLEARWARFARE_BUSTED ********************************/
			case CONF_EVENT_NUCLEARWARFARE_BUSTED:
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_var("empire",$this->GAME_TPL->displayEmpireHTML($params["empire_id"],$params["empire_emperor"],$params["empire_name"],$empire_data["networth"]));
				$this->TEMPLATE->set_file("event_data","event/nuclearwarfare_busted.html");
			break;

			/********************** CONF_EVENT_NUCLEARWARFARE_FOILED ********************************/
			case CONF_EVENT_NUCLEARWARFARE_FOILED:
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_var("empire",$this->GAME_TPL->displayEmpireHTML($params["empire_id"],$params["empire_emperor"],$params["empire_name"],$empire_data["networth"]));
				$this->TEMPLATE->set_file("event_data","event/nuclearwarfare_foiled.html");
			break;

			/********************** CONF_EVENT_NUCLEARWARFARE_ATTACKED ********************************/
			case CONF_EVENT_NUCLEARWARFARE_ATTACKED:
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_var("empire",$this->GAME_TPL->displayEmpireHTML($params["empire_id"],$params["empire_emperor"],$params["empire_name"],$empire_data["networth"]));
				$this->TEMPLATE->set_var("total_damage",$params["total_damage"]);
				$this->TEMPLATE->set_file("event_data","event/nuclearwarfare_attacked.html");
			break;


			/********************** CONF_EVENT_INVASION_REPORT ********************************/
			case CONF_EVENT_INVASION_REPORT:
				$params = unserialize($event_data["params"]);

				$this->TEMPLATE->set_file("event_data","event/invasion_report.html",true);

				$this->TEMPLATE->set_var("attack_empire_id",$params["army_attack"][0]["empire_from"]);
				$this->TEMPLATE->set_var("defense_empire_id",$params["army_defense"][0]["empire"]);
				
				$attack_empires = "";
				$attack_soldiers_qty = $params["army_attack"]["0"]["convoy_soldiers"];
				$attack_fighters_qty = $params["army_attack"]["0"]["convoy_fighters"];
				$attack_lightcruisers_qty = $params["army_attack"]["0"]["convoy_lightcruisers"];
				$attack_heavycruisers_qty = $params["army_attack"]["0"]["convoy_heavycruisers"];
				$attack_soldiers_total = 0;
				$attack_fighters_total = 0;
				$attack_lightcruisers_total = 0;
				$attack_heavycruisers_total = 0;


				for ($i=0;$i<count($params["army_attack"]);$i++) {
					$rs = $this->DB->Execute("SELECT emperor,name FROM tb_empire WHERE id=".addslashes($params["army_attack"][$i]["empire_from"]));
					if (!$rs) trigger_error($this->DB->ErrorMsg());
					$attack_soldiers_total += $params["army_attack"][$i]["convoy_soldiers"];
					$attack_fighters_total += $params["army_attack"][$i]["convoy_fighters"];
					$attack_lightcruisers_total += $params["army_attack"][$i]["convoy_lightcruisers"];
					$attack_heavycruisers_total += $params["army_attack"][$i]["convoy_heavycruisers"];

					$attack_empires .= $rs->fields["emperor"]."@".$rs->fields["name"]." (".$params["army_attack"][$i]["effectiveness"]."%)";
				}


				$this->TEMPLATE->set_var("attack_empires",$attack_empires);
				$this->TEMPLATE->set_var("attack_soldiers_qty",$this->GAME_TPL->formatNumber($attack_soldiers_qty) ." / <b>".$this->GAME_TPL->formatNumber($attack_soldiers_total))."</b>";
				$this->TEMPLATE->set_var("attack_fighters_qty",$this->GAME_TPL->formatNumber($attack_fighters_qty) ." / <b>".$this->GAME_TPL->formatNumber($attack_fighters_total))."</b>";
				$this->TEMPLATE->set_var("attack_lightcruisers_qty",$this->GAME_TPL->formatNumber($attack_lightcruisers_qty) ." / <b>".$this->GAME_TPL->formatNumber($attack_lightcruisers_total))."</b>";
				$this->TEMPLATE->set_var("attack_heavycruisers_qty",$this->GAME_TPL->formatNumber($attack_heavycruisers_qty) ." / <b>".$this->GAME_TPL->formatNumber($attack_heavycruisers_total))."</b>";
				$this->TEMPLATE->set_var("attack_soldiers_level",$params["army_attack"]["0"]["convoy_soldiers_level"]);
				$this->TEMPLATE->set_var("attack_fighters_level",$params["army_attack"]["0"]["convoy_fighters_level"]);
				$this->TEMPLATE->set_var("attack_lightcruisers_level",$params["army_attack"]["0"]["convoy_lightcruisers_level"]);
				$this->TEMPLATE->set_var("attack_heavycruisers_level",$params["army_attack"]["0"]["convoy_heavycruisers_level"]);

				$defense_empires = "";
				$defense_soldiers_qty = $params["army_defense"]["0"]["soldiers"];
				$defense_fighters_qty = $params["army_defense"]["0"]["fighters"];
				$defense_stations_qty = $params["army_defense"]["0"]["stations"];
				$defense_lightcruisers_qty = $params["army_defense"]["0"]["lightcruisers"];
				$defense_heavycruisers_qty = $params["army_defense"]["0"]["heavycruisers"];
				$defense_soldiers_total = 0;
				$defense_fighters_total = 0;
				$defense_lightcruisers_total = 0;
				$defense_heavycruisers_total = 0;

				for ($i=0;$i<count($params["army_defense"]);$i++) {
					$rs = $this->DB->Execute("SELECT emperor,name FROM tb_empire WHERE id=".addslashes($params["army_defense"][$i][($i==0?"empire":"empire_from")]));
					if (!$rs) trigger_error($this->DB->ErrorMsg());
					$defense_empires .= $rs->fields["emperor"]."@".$rs->fields["name"]." (".$params["army_defense"][$i]["effectiveness"]."%)";
					$defense_soldiers_total += $params["army_defense"][$i]["soldiers"];
					$defense_fighters_total += $params["army_defense"][$i]["fighters"];
					$defense_lightcruisers_total += $params["army_defense"][$i]["lightcruisers"];
					$defense_heavycruisers_total += $params["army_defense"][$i]["heavycruisers"];
				}

				$this->TEMPLATE->set_var("defense_empires",$defense_empires);
				$this->TEMPLATE->set_var("defense_soldiers_qty",$this->GAME_TPL->formatNumber($defense_soldiers_qty) ." / <b>".$this->GAME_TPL->formatNumber($defense_soldiers_total))."</b>";
				$this->TEMPLATE->set_var("defense_fighters_qty",$this->GAME_TPL->formatNumber($defense_fighters_qty) ." / <b>".$this->GAME_TPL->formatNumber($defense_fighters_total))."</b>";
				$this->TEMPLATE->set_var("defense_stations_qty",$this->GAME_TPL->formatNumber($defense_stations_qty));
				$this->TEMPLATE->set_var("defense_lightcruisers_qty",$this->GAME_TPL->formatNumber($defense_lightcruisers_qty) ." / <b>".$this->GAME_TPL->formatNumber($defense_lightcruisers_total))."</b>";
				$this->TEMPLATE->set_var("defense_heavycruisers_qty",$this->GAME_TPL->formatNumber($defense_heavycruisers_qty) ." / <b>".$this->GAME_TPL->formatNumber($defense_heavycruisers_total))."</b>";
				$this->TEMPLATE->set_var("defense_soldiers_level",$params["army_defense"]["0"]["soldiers_level"]);
				$this->TEMPLATE->set_var("defense_fighters_level",$params["army_defense"]["0"]["fighters_level"]);
				$this->TEMPLATE->set_var("defense_stations_level",$params["army_defense"]["0"]["stations_level"]);
				$this->TEMPLATE->set_var("defense_lightcruisers_level",$params["army_defense"]["0"]["lightcruisers_level"]);
				$this->TEMPLATE->set_var("defense_heavycruisers_level",$params["army_defense"]["0"]["heavycruisers_level"]);

				if ($params["space_won"] == true) {
				
					$this->TEMPLATE->set_var("background_space","images/background/invasion_attack.jpg");
					$this->TEMPLATE->set_var("color_space","darkred");
				
				} else {
				
					$this->TEMPLATE->set_var("background_space","images/background/invasion_defense.jpg");
					$this->TEMPLATE->set_var("color_space","darkblue");
				
				}
				
				if ($params["orbital_won"] == true) {
					$this->TEMPLATE->set_var("color_orbital","darkred");
					$this->TEMPLATE->set_var("background_orbital","images/background/invasion_attack.jpg");
				} else {
					$this->TEMPLATE->set_var("color_orbital","darkblue");
					$this->TEMPLATE->set_var("background_orbital","images/background/invasion_defense.jpg");
				}
				if ($params["ground_won"] == true) {
					$this->TEMPLATE->set_var("color_ground","darkred");
					$this->TEMPLATE->set_var("background_ground","images/background/invasion_attack.jpg");
				} else {
					$this->TEMPLATE->set_var("color_ground","darkblue");
					$this->TEMPLATE->set_var("background_ground","images/background/invasion_defense.jpg");
				}
				



				if (($params["space_won"] == true) && ($params["orbital_won"] == true) && ($params["ground_won"] == true)) {
					// attackers won


					$this->TEMPLATE->set_var("lost_population",$this->GAME_TPL->formatNumber($params["lost_population"]));
					$this->TEMPLATE->set_var("lost_credits",$this->GAME_TPL->formatCredits($params["lost_credits"]));
					$this->TEMPLATE->set_var("lost_food_planets",$this->GAME_TPL->formatNumber($params["lost_food_planets"]));
					$this->TEMPLATE->set_var("lost_ore_planets",$this->GAME_TPL->formatNumber($params["lost_ore_planets"]));
					$this->TEMPLATE->set_var("lost_supply_planets",$this->GAME_TPL->formatNumber($params["lost_supply_planets"]));
					$this->TEMPLATE->set_var("lost_tourism_planets",$this->GAME_TPL->formatNumber($params["lost_tourism_planets"]));
					$this->TEMPLATE->set_var("lost_gov_planets",$this->GAME_TPL->formatNumber($params["lost_gov_planets"]));
					$this->TEMPLATE->set_var("lost_edu_planets",$this->GAME_TPL->formatNumber($params["lost_edu_planets"]));
					$this->TEMPLATE->set_var("lost_research_planets",$this->GAME_TPL->formatNumber($params["lost_research_planets"]));
					$this->TEMPLATE->set_var("lost_urban_planets",$this->GAME_TPL->formatNumber($params["lost_urban_planets"]));
					$this->TEMPLATE->set_var("lost_petro_planets",$this->GAME_TPL->formatNumber($params["lost_petro_planets"]));
					$this->TEMPLATE->set_var("lost_antipollu_planets",$this->GAME_TPL->formatNumber($params["lost_antipollu_planets"]));


					$rs = $this->DB->Execute("SELECT civil_status FROM tb_empire WHERE id=".$params["army_defense"][0]["empire"]);
					if (!$rs) trigger_error($this->DB->ErrorMsg());
					$civil_status =abs($rs->fields["civil_status"]);
					if ($civil_status < 0) $civil_status = 0;
					if ($civil_status > 7) $civil_status = 7;

					$this->TEMPLATE->set_var("civil_status",$CONF_CIVIL_STATUS[$civil_status]);

					$rs = $this->DB->Execute("SELECT effectiveness FROM tb_army WHERE id=".$params["army_defense"][0]["empire"]);
					if (!$rs) trigger_error($this->DB->ErrorMsg());
					$this->TEMPLATE->set_var("military_effectiveness",$rs->fields["effectiveness"]);



					$this->TEMPLATE->set_var("battle_result",$this->TEMPLATE->process("","battle_result_won",2));

				} else {
					// attackers lost
					$this->TEMPLATE->set_var("battle_result",$this->TEMPLATE->process("","battle_result_lost",2));
				}


				$attack_casualties = array();
				for ($i=0;$i<count($params["army_attack"]);$i++) {

					$casualty = array();
					$rs = $this->DB->Execute("SELECT emperor,name FROM tb_empire WHERE id=".addslashes($params["army_attack"][$i]["empire_from"]));
					if (!$rs) trigger_error($this->DB->ErrorMsg());					
					$casualty["emperor"] = $rs->fields["emperor"];
					$casualty["name"] = $rs->fields["name"];
					$casualty["soldiers"] = $this->GAME_TPL->formatNumber($params["army_attack"][$i]["casualties_soldiers"]);
					$casualty["generals"] = $this->GAME_TPL->formatNumber(ceil($params["army_attack"][$i]["casualties_soldiers"]/50));
					$casualty["fighters"] = $this->GAME_TPL->formatNumber($params["army_attack"][$i]["casualties_fighters"]);
					$casualty["lightcruisers"] = $this->GAME_TPL->formatNumber($params["army_attack"][$i]["casualties_lightcruisers"]);
					$casualty["heavycruisers"] = $this->GAME_TPL->formatNumber($params["army_attack"][$i]["casualties_heavycruisers"]);
					$casualty["carriers"] = $this->GAME_TPL->formatNumber($params["army_attack"][$i]["casualties_carriers"]);
					$attack_casualties[] = $casualty;

				}

				$this->TEMPLATE->set_loop("attack_casualties",$attack_casualties);

				$defense_casualties = array();
				for ($i=0;$i<count($params["army_defense"]);$i++) {

					$casualty = array();
					$rs = $this->DB->Execute("SELECT emperor,name FROM tb_empire WHERE id=".addslashes($params["army_defense"][$i][($i==0?"empire":"empire_from")]));
					if (!$rs) trigger_error($this->DB->ErrorMsg());
					$casualty["emperor"] = $rs->fields["emperor"];
					$casualty["name"] = $rs->fields["name"];
					$casualty["soldiers"] = $this->GAME_TPL->formatNumber($params["army_defense"][$i]["casualties_soldiers"]);
					$casualty["generals"] = $this->GAME_TPL->formatNumber(ceil($params["army_defense"][$i]["casualties_soldiers"]/50));
					$casualty["fighters"] = $this->GAME_TPL->formatNumber($params["army_defense"][$i]["casualties_fighters"]);
					$casualty["lightcruisers"] = $this->GAME_TPL->formatNumber($params["army_defense"][$i]["casualties_lightcruisers"]);
					$casualty["heavycruisers"] = $this->GAME_TPL->formatNumber($params["army_defense"][$i]["casualties_heavycruisers"]);
					$casualty["stations"] = $this->GAME_TPL->formatNumber($params["army_defense"][$i]["casualties_stations"]);
					$defense_casualties[] = $casualty;

				}

				$this->TEMPLATE->set_loop("defense_casualties",$defense_casualties);

			break;

			/********************** EVENT_CONVOY_BACK *********************************/
			case CONF_EVENT_CONVOY_BACK:
				$params = unserialize($event_data["params"]);
				$this->TEMPLATE->set_file("event_data","event/convoy_back.html",true);
				$this->TEMPLATE->set_var("soldiers",$this->GAME_TPL->formatNumber($params["convoy_soldiers"]));
				$this->TEMPLATE->set_var("generals",$this->GAME_TPL->formatNumber($params["convoy_generals"]));
				$this->TEMPLATE->set_var("fighters",$this->GAME_TPL->formatNumber($params["convoy_fighters"]));
				$this->TEMPLATE->set_var("lightcruisers",$this->GAME_TPL->formatNumber($params["convoy_lightcruisers"]));
				$this->TEMPLATE->set_var("heavycruisers",$this->GAME_TPL->formatNumber($params["convoy_heavycruisers"]));
				$this->TEMPLATE->set_var("carriers",$this->GAME_TPL->formatNumber($params["carriers"]));

			break;


			/********************** UNKNOWN EVENT TYPE! *********************************/
			default:
				return LANG_UNKNOWN_EVENT_TYPE." = ".$event_data["event_type"];
			break;
		}
		
		$this->TEMPLATE->set_var("date",$this->GAME_TPL->formatTime(time(NULL)-$event_data["date"]));
		$output = $this->TEMPLATE->process("","event_data",2);
		
		if (($event_data["sticky"]==0) && ($event_data["seen"]==0)) {
			if (!$this->DB->Execute("UPDATE tb_event SET seen=1 WHERE id=".$event_data["id"])) trigger_error($this->DB->ErrorMsg());
		}

		return $output;
	}

}


?>
