<?php


//---------------------------------------------------------------------------//
// author: Yanick Bourbeau <ybourbeau@mrgtech.ca> 
// date:   2006.01.01
// web:    http://www.mrgtech.ca
// info:   Solar Imperium
//---------------------------------------------------------------------------//
// copyleft license
//
// this software is provided 'as-is', without any express or implied
// warranty. in no event will the authors be held liable for any damages
// arising from the use of this software.
//
// permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions: 
//
// 1. the origin of this software must not be misrepresented;
//	  you must not claim that you wrote the original software. 
//	  if you use this software in a product, an acknowledgment 
//	  in the product documentation would be appreciated but is not required.
//
// 2. altered source versions must be plainly marked as such, 
//	  and must not be misrepresented as being the original software.
//
// 3. mail about the fact of using this class in production 
//	  would be very appreciated.
//
// 4. this notice may not be removed or altered from any source distribution.
//
//---------------------------------------------------------------------------//
// CHANGELOG:
//
// 2006.06.01 : ybo : Changed name: Imperium Galactica is (c), moved to Solar Imperium
// 2006.09.01 : ybo : Major code changes, OOP
// 2007.02.04 : ybo : Reviewed the code for public release
//
//---------------------------------------------------------------------------//

class GameplayCosts {

	var $DB;

	///////////////////////////////////////////////////////////////////////
	// constructor
	///////////////////////////////////////////////////////////////////////
	function GameplayCosts($DB) {
		$this->DB = $DB;
	}

	///////////////////////////////////////////////////////////////////////
	// Retrieve detailled food price
	///////////////////////////////////////////////////////////////////////
	function getFoodPrices() {
		$demand = array (
			LANG_DEMAND_LOW,
			LANG_DEMAND_NORMAL,
			LANG_DEMAND_HIGH
		);

		$price_buy = CONF_FOODMARKET_PRICE_BUY;
		$price_sell = CONF_FOODMARKET_PRICE_SELL;

		$foodmarket = $this->DB->Execute("SELECT food FROM tb_coordinator");
		$foodmarket = $foodmarket->fields[0];

		$avg = round(($foodmarket / CONF_FOODMARKET_AVG) * 100);
		$a = floor($avg / 33.33);
		if ($a < 0)
			$a = 0;
		if ($a > 2)
			$a = 2;
		$avg = 2 - $a;
		$demand = $demand[$avg];

		if ($avg == 0) {
			$price_buy /= 10;
			$price_sell /= 10;
			$price_buy = floor($price_buy);
			$price_sell = floor($price_sell);

			if ($price_buy < 0)
				$price_buy = 1;
			if ($price_sell < 0)
				$price_sell = 1;
		}

		if ($avg == 2) {
			$price_buy *= 10;
			$price_sell *= 10;
			$price_buy = floor($price_buy);
			$price_sell = floor($price_sell);

		}

		return array (
			"buy" => $price_buy,
			"sell" => $price_sell,
			"demand" => $demand
		);
	}

	///////////////////////////////////////////////////////////////////////
	// Retrieve detailed planets prices
	///////////////////////////////////////////////////////////////////////
	function getPlanetsPrices($planets_data, $turns_played, $inflation) {

		$costs = array ();

		if ($planets_data["max_ore"] < $planets_data["ore_planets"])
			$planets_data["max_ore"] = $planets_data["ore_planets"];
		if ($planets_data["max_tourism"] < $planets_data["tourism_planets"])
			$planets_data["max_tourism"] = $planets_data["tourism_planets"];
		if ($planets_data["max_petro"] < $planets_data["petro_planets"])
			$planets_data["max_petro"] = $planets_data["petro_planets"];

		if ($planets_data["max_food"] < $planets_data["food_planets"])
			$planets_data["max_food"] = $planets_data["food_planets"];
		if ($planets_data["max_supply"] < $planets_data["supply_planets"])
			$planets_data["max_supply"] = $planets_data["supply_planets"];
		if ($planets_data["max_gov"] < $planets_data["gov_planets"])
			$planets_data["max_gov"] = $planets_data["gov_planets"];
		if ($planets_data["max_edu"] < $planets_data["edu_planets"])
			$planets_data["max_edu"] = $planets_data["edu_planets"];
		if ($planets_data["max_research"] < $planets_data["research_planets"])
			$planets_data["max_research"] = $planets_data["research_planets"];
		if ($planets_data["max_urban"] < $planets_data["urban_planets"])
			$planets_data["max_urban"] = $planets_data["urban_planets"];
		if ($planets_data["max_antipollu"] < $planets_data["antipollu_planets"])
			$planets_data["max_antipollu"] = $planets_data["antipollu_planets"];

		$food_planets = $planets_data["food_planets"];
		if ($food_planets == 0)
			$food_planets = 1;
		$costs["planet_food"] = round((CONF_COST_PLANET_FOOD + $turns_played) + (CONF_COST_PLANET_FOOD * $inflation * 0.0008));
		$costs["planet_food"] = $this->calcUnitCost($costs["planet_food"], $planets_data["max_food"], CONF_COST_INCREMENT);

		$supply_planets = $planets_data["supply_planets"];
		if ($supply_planets == 0)
			$supply_planets = 1;
		$costs["planet_supply"] = round((CONF_COST_PLANET_SUPPLY + $turns_played) + (CONF_COST_PLANET_SUPPLY * $inflation * 0.0008));
		$costs["planet_supply"] = $this->calcUnitCost($costs["planet_supply"], $planets_data["max_supply"], CONF_COST_INCREMENT);

		$gov_planets = $planets_data["gov_planets"];
		if ($gov_planets == 0)
			$gov_planets = 1;
		$costs["planet_gov"] = round((CONF_COST_PLANET_GOV + $turns_played) + (CONF_COST_PLANET_GOV * $inflation * 0.0008));
		$costs["planet_gov"] = $this->calcUnitCost($costs["planet_gov"], $planets_data["max_gov"], CONF_COST_INCREMENT);

		$edu_planets = $planets_data["edu_planets"];
		if ($edu_planets == 0)
			$edu_planets = 1;
		$costs["planet_edu"] = round((CONF_COST_PLANET_EDU + $turns_played) + (CONF_COST_PLANET_EDU * $inflation * 0.0008));
		$costs["planet_edu"] = $this->calcUnitCost($costs["planet_edu"], $planets_data["max_edu"], CONF_COST_INCREMENT);

		$research_planets = $planets_data["research_planets"];
		if ($research_planets == 0)
			$research_planets = 1;
		$costs["planet_research"] = round((CONF_COST_PLANET_RESEARCH + $turns_played) + (CONF_COST_PLANET_RESEARCH * $inflation * 0.0008));
		$costs["planet_research"] = $this->calcUnitCost($costs["planet_research"], $planets_data["max_research"], CONF_COST_INCREMENT);

		$urban_planets = $planets_data["urban_planets"];
		if ($urban_planets == 0)
			$urban_planets = 1;
		$costs["planet_urban"] = round((CONF_COST_PLANET_URBAN + $turns_played) + (CONF_COST_PLANET_URBAN * $inflation * 0.0008));
		$costs["planet_urban"] = $this->calcUnitCost($costs["planet_urban"], $planets_data["max_urban"], CONF_COST_INCREMENT);

		$ore_planets = $planets_data["ore_planets"];
		if ($ore_planets == 0)
			$ore_planets = 1;
		$costs["planet_ore"] = round((CONF_COST_PLANET_ORE + $turns_played) + (CONF_COST_PLANET_ORE * $inflation * 0.0008));
		$costs["planet_ore"] = $this->calcUnitCost($costs["planet_ore"], $planets_data["max_ore"], CONF_COST_INCREMENT);

		$tourism_planets = $planets_data["tourism_planets"];
		if ($tourism_planets == 0)
			$tourism_planets = 1;
		$costs["planet_tourism"] = round((CONF_COST_PLANET_TOURISM + $turns_played) + (CONF_COST_PLANET_TOURISM * $inflation * 0.0008));
		$costs["planet_tourism"] = $this->calcUnitCost($costs["planet_tourism"], $planets_data["max_tourism"], CONF_COST_INCREMENT);

		$petro_planets = $planets_data["petro_planets"];
		if ($petro_planets == 0)
			$petro_planets = 1;
		$costs["planet_petro"] = round((CONF_COST_PLANET_PETROLEUM + $turns_played) + (CONF_COST_PLANET_PETROLEUM * $inflation * 0.0008));
		$costs["planet_petro"] = $this->calcUnitCost($costs["planet_petro"], $planets_data["max_petro"], CONF_COST_INCREMENT);

		$antipollu_planets = $planets_data["antipollu_planets"];
		if ($antipollu_planets == 0)
			$antipollu_planets = 1;
		$costs["planet_antipollu"] = round((CONF_COST_PLANET_ANTIPOLLU + $turns_played) + (CONF_COST_PLANET_ANTIPOLLU * $inflation * 0.0008));
		$costs["planet_antipollu"] = $this->calcUnitCost($costs["planet_antipollu"], $planets_data["max_antipollu"], CONF_COST_INCREMENT);

		return $costs;

	}

	///////////////////////////////////////////////////////////////////////
	//
	///////////////////////////////////////////////////////////////////////
	function calcTotalCost($base, $qty, $increment) {
		return floor(($base * $qty) + ((($qty -1) * $qty) * $increment) / 2);
	}

	///////////////////////////////////////////////////////////////////////
	//
	///////////////////////////////////////////////////////////////////////
	function calcUnitCost($base, $pos, $increment) {
		return floor($base + (($pos -1) * ($base * $increment)));
	}

	///////////////////////////////////////////////////////////////////////
	//
	///////////////////////////////////////////////////////////////////////
	function getMaxQty($cash, $baseprice, $increment) {

		$qty = 0;
		$level = 1;
		$q = 0;
		$pool = ($cash / $baseprice);
		$slice = ($pool / 10);
		$count = 0;
		while (1) {

			if ($count++ > 80)
				break;
			$q += $slice;
			$total = $this->calcTotalCost($baseprice, $q, $increment);
			if ($total > $cash) {
				$q -= $slice;
				$level++;
				$slice = ($slice / 2);

			} else {
				$qty += $slice;
			}
			if ($level == 20)
				break;
		}

		return floor($qty);
	}

	///////////////////////////////////////////////////////////////////////
	//
	///////////////////////////////////////////////////////////////////////
	function getUnitsPrices($army_data,$empire_data,$turns_played,$inflation) {

		$costs = array();
		$convoy_soldiers = 0;
		$convoy_fighters = 0;
		$convoy_generals = 0;
		$convoy_lightcruisers = 0;
		$convoy_heavycruisers = 0;
		$convoy_carriers = 0;

		$rs = $this->DB->Execute("SELECT * FROM tb_armyconvoy WHERE empire_from=".$empire_data["id"]);
		if (!$rs) trigger_error($this->DB->ErrorMsg());
		while(!$rs->EOF)
		{
			$convoy_soldiers += $rs->fields["convoy_soldiers"];
			$convoy_fighters += $rs->fields["convoy_fighters"];
			$convoy_generals += $rs->fields["convoy_generals"];
			$convoy_lightcruisers += $rs->fields["convoy_lightcruisers"];
			$convoy_heavycruisers += $rs->fields["convoy_heavycruisers"];
			$convoy_carriers += $rs->fields["carriers"];
			$rs->MoveNext();
		}


		$costs["buy_soldiers"] = round((CONF_COST_SOLDIER + $turns_played) + ((CONF_COST_SOLDIER + $turns_played) * $inflation * 0.005));
		$costs["sell_soldiers"] = floor($costs["buy_soldiers"] / CONF_COST_SELLRATIO);
		$total = $army_data["soldiers"] + $convoy_soldiers;
		$costs["buy_soldiers"] = $this->calcUnitCost($costs["buy_soldiers"],$total, CONF_COST_INCREMENT);
			
		$costs["buy_fighters"] = round((CONF_COST_FIGHTER + $turns_played) + (CONF_COST_FIGHTER * $inflation * 0.005));
		$costs["sell_fighters"] = floor($costs["buy_fighters"] / CONF_COST_SELLRATIO);
		$total = $army_data["fighters"] + $convoy_fighters;
		$costs["buy_fighters"] = $this->calcUnitCost($costs["buy_fighters"], $total, CONF_COST_INCREMENT);

		$costs["buy_stations"] = round((CONF_COST_STATION + $turns_played) + (CONF_COST_STATION * $inflation * 0.005));
		$costs["sell_stations"] = floor($costs["buy_stations"] / CONF_COST_SELLRATIO);
		$costs["buy_stations"] = $this->calcUnitCost($costs["buy_stations"], $army_data["stations"], CONF_COST_INCREMENT);

		$costs["buy_generals"] = round((CONF_COST_GENERAL + $turns_played) + (CONF_COST_GENERAL * $inflation * 0.005));
		$costs["sell_generals"] = floor($costs["buy_generals"] / CONF_COST_SELLRATIO);
		$total = $army_data["generals"] + $convoy_generals;
		$costs["buy_generals"] = $this->calcUnitCost($costs["buy_generals"], $total, CONF_COST_INCREMENT);

		$costs["buy_commandship"] = CONF_COST_COMMANDSHIP + ((CONF_COST_COMMANDSHIP + $turns_played) * $army_data["commandship"] * $inflation * 50);

		$costs["buy_covertagents"] = round((CONF_COST_COVERT + $turns_played) + (CONF_COST_COVERT * $inflation * 0.005));
		$costs["sell_covertagents"] = floor($costs["buy_covertagents"] / CONF_COST_SELLRATIO);
		$costs["buy_covertagents"] = $this->calcUnitCost($costs["buy_covertagents"], $army_data["covertagents"], CONF_COST_INCREMENT);

		$costs["buy_heavycruisers"] = round((CONF_COST_HEAVYCRUISER + $turns_played) + (CONF_COST_HEAVYCRUISER * $inflation * 0.005));
		$costs["sell_heavycruisers"] = floor($costs["buy_heavycruisers"] / CONF_COST_SELLRATIO);
		$total = $army_data["heavycruisers"] + $convoy_heavycruisers;
		$costs["buy_heavycruisers"] = $this->calcUnitCost($costs["buy_heavycruisers"], $total, CONF_COST_INCREMENT);

		$costs["sell_lightcruisers"] = round(((CONF_COST_HEAVYCRUISER / 4) + $turns_played) + ((CONF_COST_HEAVYCRUISER / 4) * $empire_data["inflation"] * 0.005));

		$costs["buy_carriers"] = round((CONF_COST_CARRIER + $turns_played) + (CONF_COST_CARRIER * $inflation * 0.005));
		$costs["sell_carriers"] = floor($costs["buy_carriers"] / CONF_COST_SELLRATIO);
		$total = $army_data["carriers"] + $convoy_carriers;
		$costs["buy_carriers"] = $this->calcUnitCost($costs["buy_carriers"], $total, CONF_COST_INCREMENT);
	
		return $costs;
		
	}

}
?>
