<?php

//---------------------------------------------------------------------------//
// author: Yanick Bourbeau <ybourbeau@mrgtech.ca> 
// date:   2006.01.01
// web:    http://www.mrgtech.ca
// info:   Solar Imperium
//---------------------------------------------------------------------------//
// copyleft license
//
// this software is provided 'as-is', without any express or implied
// warranty. in no event will the authors be held liable for any damages
// arising from the use of this software.
//
// permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions: 
//
// 1. the origin of this software must not be misrepresented;
//	  you must not claim that you wrote the original software. 
//	  if you use this software in a product, an acknowledgment 
//	  in the product documentation would be appreciated but is not required.
//
// 2. altered source versions must be plainly marked as such, 
//	  and must not be misrepresented as being the original software.
//
// 3. mail about the fact of using this class in production 
//	  would be very appreciated.
//
// 4. this notice may not be removed or altered from any source distribution.
//
//---------------------------------------------------------------------------//
// CHANGELOG:
//
// 2006.06.01 : ybo : Changed name: Imperium Galactica is (c), moved to Solar Imperium
// 2006.09.01 : ybo : Major code changes, OOP
// 2007.02.04 : ybo : Reviewed the code for public release
// 2007.02.18 : ybo : Fixing division by zero error at line 726
// 2007.03.01 : ybo : Fixing convoy bugs.
// 2007.03.02 : ybo : Removed fighters from defense
//
//---------------------------------------------------------------------------//

class Invasion {

	var $DB;
	var $convoy;
	var $target;
	var $total_strength;
	var $total_casualties;

	///////////////////////////////////////////////////////////////////////
	//
	///////////////////////////////////////////////////////////////////////
	function Invasion($DB, $convoy) {
		$this->DB = $DB;
		$this->convoy = $convoy;
		$query = "SELECT * FROM tb_army WHERE empire=" . $this->convoy["empire_to"];
		$rs = $this->DB->Execute($query);
		if (!$rs)
			trigger_error($this->DB->ErrorMsg());
		$this->target = $rs->fields;
		if ($rs->EOF) {
	
			$query = "UPDATE tb_armyconvoy SET " .
				"convoy_type=" .CONF_CONVOY_INVASION_RETREAT . "," .
				"time_left=" . $convoy["time_total"] . " WHERE id=" . $convoy["id"];
	
			$DB->Execute($query);
			$convoy = NULL;
		}
		

		srand(time(NULL));
	}

	///////////////////////////////////////////////////////////////////////
	//
	///////////////////////////////////////////////////////////////////////
	function Attack() {
		global $units_info;
		if ($this->convoy == NULL) return;
		
		/////////////////////////////////////////////////////////////
		// Step 1: Build invasion force
		/////////////////////////////////////////////////////////////
		$army_attack = array ();
		$this->convoy["casualties_soldiers"] = 0;
		$this->convoy["casualties_fighters"] = 0;
		$this->convoy["casualties_lightcruisers"] = 0;
		$this->convoy["casualties_heavycruisers"] = 0;
		$this->convoy["casualties_carriers"] = 0;
		$rs2 = $this->DB->Execute("SELECT effectiveness FROM tb_army WHERE empire=" . $this->convoy["empire_from"]);
		if (!$rs2)
			trigger_error($this->DB->ErrorMsg());
		$this->convoy["effectiveness"] = $rs2->fields["effectiveness"];

		$army_attack[] = $this->convoy;

		$rs = $this->DB->Execute("SELECT * FROM tb_armyconvoy WHERE " .
		"convoy_type=" . CONF_CONVOY_INVASION . " AND " .
		"empire_to=" . $this->convoy["empire_to"] . " AND " .
		"NOT empire_from=" . $this->convoy["empire_from"] . " AND " .
		"time_left <= 10");
		if (!$rs)
			trigger_error($this->DB->ErrorMsg());

		while (!$rs->EOF) {
			$convoy_data = $rs->fields;

			$rs2 = $this->DB->Execute("SELECT id,emperor,name,effectiveness FROM tb_army WHERE empire=" . $rs->fields["empire_from"]);
			if (!$rs2)
				trigger_error($this->DB->ErrorMsg());
			$convoy_data["id"] = $rs2->fields["id"];
			$convoy_data["emperor"] = $rs2->fields["emperor"];
			$convoy_data["name"] = $rs2->fields["name"];
			$convoy_data["effectiveness"] = $rs2->fields["effectiveness"];
			$convoy_data["casualties_soldiers"] = 0;
			$convoy_data["casualties_fighters"] = 0;
			$convoy_data["casualties_lightcruisers"] = 0;
			$convoy_data["casualties_heavycruisers"] = 0;
			$convoy_data["casualties_carriers"] = 0;

			$army_attack[] = $convoy_data;
			$rs->MoveNext();
		}

		/////////////////////////////////////////////////////////////
		// Step 2: Build defense force
		/////////////////////////////////////////////////////////////
		$army_defense = array ();
		$this->target["casualties_soldiers"] = 0;
		$this->target["casualties_fighters"] = 0;
		$this->target["casualties_stations"] = 0;
		$this->target["casualties_lightcruisers"] = 0;
		$this->target["casualties_heavycruisers"] = 0;

		$army_defense[] = $this->target;

		$rs = $this->DB->Execute("SELECT * FROM tb_armyconvoy WHERE " .
		"convoy_type=" . CONF_CONVOY_DEFENSE . " AND " .
		"empire_to=" . $this->convoy["empire_to"] . " AND " .
		"time_left <= 10");

		if (!$rs)
			trigger_error($this->DB->ErrorMsg());

		while (!$rs->EOF) {
			$convoy_data = $rs->fields;
			$rs2 = $this->DB->Execute("SELECT effectiveness FROM tb_army WHERE empire=" . $rs->fields["empire_from"]);
			$rs3 = $this->DB->Execute("SELECT id,emperor,name FROM tb_empire WHERE id=" . $rs->fields["empire_from"]);
			if (!$rs2)
				trigger_error($this->DB->ErrorMsg());
			$convoy_data["id"] = $rs3->fields["id"];
			$convoy_data["emperor"] = $rs3->fields["emperor"];
			$convoy_data["name"] = $rs3->fields["name"];
			$convoy_data["effectiveness"] = $rs2->fields["effectiveness"];
			$convoy_data["soldiers"] = $rs->fields["convoy_soldiers"];
			$convoy_data["generals"] = $rs->fields["convoy_generals"];
			$convoy_data["fighters"] = $rs->fields["convoy_fighters"];
			$convoy_data["lightcruisers"] = $rs->fields["convoy_lightcruisers"];
			$convoy_data["heavycruisers"] = $rs->fields["convoy_heavycruisers"];
			$convoy_data["soldiers_level"] = $rs->fields["convoy_soldiers_level"];
			$convoy_data["fighters_level"] = $rs->fields["convoy_fighters_level"];
			$convoy_data["lightcruisers_level"] = $rs->fields["convoy_lightcruisers_level"];
			$convoy_data["heavycruisers_level"] = $rs->fields["convoy_heavycruisers_level"];
			$convoy_data["casualties_soldiers"] = 0;
			$convoy_data["casualties_fighters"] = 0;
			$convoy_data["casualties_stations"] = 0;
			$convoy_data["casualties_lightcruisers"] = 0;
			$convoy_data["casualties_heavycruisers"] = 0;
			$army_defense[] = $convoy_data;
			$rs->MoveNext();
		}

		$this->total_strength = array (
			"SPACE" => array (),
			"ORBITAL" => array (),
			"GROUND" => array ()
		);
		$this->total_casualties = array (
			"SPACE" => array (),
			"ORBITAL" => array (),
			"GROUND" => array ()
		);

		/////////////////////////////////////////////////////////////
		// Step 3: Do attacks
		/////////////////////////////////////////////////////////////

		///////////////////////////////////////////////////////////////////////////////////////////////////////
		// Space Attack!
		///////////////////////////////////////////////////////////////////////////////////////////////////////

		$space_won_attack = 0;
		$space_won_defense = 0;

		for ($round = 0; $round < 5; $round++) {

			// print "ROUND ".($round+1)."<br/>";
			$attack_strength = 1;
			$total_attack_strength = array ();
			$total_attack_strength["soldiers"] = 0;
			$total_attack_strength["fighters"] = 0;
			$total_attack_strength["lightcruisers"] = 0;
			$total_attack_strength["heavycruisers"] = 0;

			for ($i = 0; $i < count($army_attack); $i++) {

				$soldiers_strength = (($army_attack[$i]["convoy_soldiers"] - $army_attack[$i]["casualties_soldiers"]) * $units_info["soldiers_" . $army_attack[$i]["convoy_soldiers_level"]]["invasion_space"]);
				$soldiers_strength = floor(($soldiers_strength / 100) * $army_attack[$i]["effectiveness"]);
				$attack_strength += $soldiers_strength;

				$fighters_strength = (($army_attack[$i]["convoy_fighters"] - $army_attack[$i]["casualties_fighters"]) * $units_info["fighters_" . $army_attack[$i]["convoy_fighters_level"]]["invasion_space"]);
				$fighters_strength = floor(($fighters_strength / 100) * $army_attack[$i]["effectiveness"]);
				$attack_strength += $fighters_strength;

				$lightcruisers_strength = (($army_attack[$i]["convoy_lightcruisers"] - $army_attack[$i]["casualties_lightcruisers"]) * $units_info["lightcruisers_" . $army_attack[$i]["convoy_lightcruisers_level"]]["invasion_space"]);
				$lightcruisers_strength = floor(($lightcruisers_strength / 100) * $army_attack[$i]["effectiveness"]);
				$attack_strength += $lightcruisers_strength;

				$heavycruisers_strength = (($army_attack[$i]["convoy_heavycruisers"] - $army_attack[$i]["casualties_heavycruisers"]) * $units_info["heavycruisers_" . $army_attack[$i]["convoy_heavycruisers_level"]]["invasion_space"]);
				$heavycruisers_strength = floor(($heavycruisers_strength / 100) * $army_attack[$i]["effectiveness"]);
				$attack_strength += $heavycruisers_strength;

				$total_attack_strength["soldiers"] += $soldiers_strength;
				$total_attack_strength["fighters"] += $fighters_strength;
				$total_attack_strength["lightcruisers"] += $lightcruisers_strength;
				$total_attack_strength["heavycruisers"] += $heavycruisers_strength;
			}

			$defense_strength = 1;
			$total_defense_strength = array ();
			$total_defense_strength["soldiers"] = 0;
			$total_defense_strength["stations"] = 0;
			$total_defense_strength["fighters"] = 0;
			$total_defense_strength["lightcruisers"] = 0;
			$total_defense_strength["heavycruisers"] = 0;
			for ($i = 0; $i < count($army_defense); $i++) {

				$soldiers_strength = (($army_defense[$i]["soldiers"] - $army_defense[$i]["casualties_soldiers"]) * $units_info["soldiers_" . $army_defense[$i]["soldiers_level"]]["invasion_space"]);
				$soldiers_strength = floor(($soldiers_strength / 100) * $army_defense[$i]["effectiveness"]);
				$defense_strength += $soldiers_strength;

				$fighters_strength = (($army_defense[$i]["fighters"] - $army_defense[$i]["casualties_fighters"]) * $units_info["fighters_" . $army_defense[$i]["fighters_level"]]["invasion_space"]);
				$fighters_strength = floor(($fighters_strength / 100) * $army_defense[$i]["effectiveness"]);
				//$defense_strength += $fighters_strength;

				if (isset ($army_defense[$i]["stations"])) {
					$stations_strength = (($army_defense[$i]["stations"] - $army_defense[$i]["casualties_stations"]) * $units_info["fighters_" . $army_defense[$i]["stations_level"]]["invasion_space"]);
					$stations_strength = floor(($stations_strength / 100) * $army_defense[$i]["effectiveness"]);
					$defense_strength += $stations_strength;
				}

				$lightcruisers_strength = (($army_defense[$i]["lightcruisers"] - $army_defense[$i]["casualties_lightcruisers"]) * $units_info["lightcruisers_" . $army_defense[$i]["lightcruisers_level"]]["invasion_space"]);
				$lightcruisers_strength = floor(($lightcruisers_strength / 100) * $army_defense[$i]["effectiveness"]);
				$defense_strength += $lightcruisers_strength;

				$heavycruisers_strength = (($army_defense[$i]["heavycruisers"] - $army_defense[$i]["casualties_heavycruisers"]) * $units_info["heavycruisers_" . $army_defense[$i]["heavycruisers_level"]]["invasion_space"]);
				$heavycruisers_strength += ((($heavycruisers_strength / 100) * $this->target["commandship"]) * $units_info["commandship_" . $this->target["commandship_level"]]["bonus"]);
				$heavycruisers_strength = floor(($heavycruisers_strength / 100) * $army_defense[$i]["effectiveness"]);

				$defense_strength += $heavycruisers_strength;

				$total_defense_strength["soldiers"] += $soldiers_strength;
				$total_defense_strength["fighters"] += $fighters_strength;
				$total_defense_strength["stations"] += $stations_strength;
				$total_defense_strength["lightcruisers"] += $lightcruisers_strength;
				$total_defense_strength["heavycruisers"] += $heavycruisers_strength;

			}
			$defense_strength *= CONF_INVASION_DEFENSE_BONUS;

			$this->total_strength["SPACE"][$round] = array (
				"attack" => $total_attack_strength,
				"defense" => $total_defense_strength
			);

			//			$attack_strength = 8000;
			//			$defense_strength = 5000;	
			//			$attack_strength = 4000;
			//			$defense_strength = 5000;	

			// add randomness
			$attack_strength += floor(rand(0, CONF_INVASION_RANDOMNESS) * ($attack_strength / 100));
			$defense_strength += floor(rand(0, CONF_INVASION_RANDOMNESS) * ($defense_strength / 100));
			if ($attack_strength == 0)
				$attack_strength = 1;
			if ($defense_strength == 0)
				$defense_strength = 1;

			// print "ATTACK STRENGTH: ".$attack_strength."<br/>";
			// print "DEFENSE STRENGTH: ".$defense_strength."<br/>";
			$casualties = $attack_strength - $defense_strength;
			if ($casualties == 0)
				$casualties = 1;

			// print "RAW DAMAGE: ".$casualties."<br/>";
			$casualties_attack = 0;
			$casualties_defense = 0;

			if ($casualties <= 1) {
				// print "ATTACKERS LOOSE<br/>";
				$casualties_attack = (($defense_strength / $attack_strength) * 100);
				$casualties_defense = (($attack_strength / $defense_strength) * 100);
				$casualties_attack = ($casualties_attack) / 8;
				$casualties_defense = ($casualties_defense) / 12;
				$space_won_defense++;

			} else {
				// print "DEFENDERS LOOSE<br/>";
				$casualties_attack = (($defense_strength / $attack_strength) * 100);
				$casualties_defense = (($attack_strength / $defense_strength) * 100);
				$casualties_attack = ($casualties_attack) / 12;
				$casualties_defense = ($casualties_defense) / 8;
				$space_won_attack++;
			}

			if ($casualties_attack > 100)
				$casualties_attack = 100;
			if ($casualties_defense > 100)
				$casualties_defense = 100;

			$this->total_casualties["SPACE"][$round] = array (
				"attack" => $casualties_attack,
				"defense" => $casualties_defense
			);

			// print "CASUALTIES ATTACK: $casualties_attack<br/>";
			// print "CASUALTIES DEFENSE: $casualties_defense<br/>";
			// print "<br/>";

			// calculating casualties	
			for ($i = 0; $i < count($army_attack); $i++) {
				// print "ATTACK CONVOY: $i<br/>";
				if ($total_attack_strength["soldiers"] != 0) {
					$percent = (((($army_attack[$i]["convoy_soldiers"] - $army_attack[$i]["casualties_soldiers"]) / $total_attack_strength["soldiers"]) * 100) / 100) * $casualties_attack;
					if ($percent > 100)
						$percent = 100;

					$casualties = floor((($army_attack[$i]["convoy_soldiers"] - $army_attack[$i]["casualties_soldiers"]) / 100) * $percent);
					$army_attack[$i]["casualties_soldiers"] += $casualties;
					// print "SOLDIERS CASUALTIES: " .$percent." % (".$casualties.")<br/>";
				}
				if ($total_attack_strength["fighters"] != 0) {
					$percent = (((($army_attack[$i]["convoy_fighters"] - $army_attack[$i]["casualties_fighters"]) / $total_attack_strength["fighters"]) * 100) / 100) * $casualties_attack;
					if ($percent > 100)
						$percent = 100;
					$casualties = floor((($army_attack[$i]["convoy_fighters"] - $army_attack[$i]["casualties_fighters"]) / 100) * $percent);
					$army_attack[$i]["casualties_fighters"] += $casualties;
					// print "FIGHTERS CASUALTIES: " .$percent." % (".$casualties.")<br/>";
				}

				if ($round >= CONF_INVASION_LIGHTCRUISER_PROTECTION_ROUNDS)
					if ($total_attack_strength["lightcruisers"] != 0) {
						$percent = (((($army_attack[$i]["convoy_lightcruisers"] - $army_attack[$i]["casualties_lightcruisers"]) / $total_attack_strength["lightcruisers"]) * 100) / 100) * $casualties_attack;
						if ($percent > 100)
							$percent = 100;
						$casualties = floor((($army_attack[$i]["convoy_lightcruisers"] - $army_attack[$i]["casualties_lightcruisers"]) / 100) * $percent);
						$army_attack[$i]["casualties_lightcruisers"] += $casualties;
						// print "LIGHTCRUISERS CASUALTIES: " .$percent." %  (".$casualties.")<br/>";
					}

				if ($total_attack_strength["heavycruisers"] != 0) {
					$percent = (((($army_attack[$i]["convoy_heavycruisers"] - $army_attack[$i]["casualties_heavycruisers"]) / $total_attack_strength["heavycruisers"]) * 100) / 100) * $casualties_attack;
					if ($percent > 100)
						$percent = 100;
					$casualties = floor((($army_attack[$i]["convoy_heavycruisers"] - $army_attack[$i]["casualties_heavycruisers"]) / 100) * $percent);
					$army_attack[$i]["casualties_heavycruisers"] += $casualties;
					// print "HEAVYCRUISERS CASUALTIES: " .$percent." %  (".$casualties.")<br/>";
				}

				// Calculate carriers lost
				$percent = (($casualties_attack / 100) * CONF_INVASION_CARRIERS_DAMAGE) * $units_info["carriers_" . $army_attack[$i]["carriers_level"]]["toughness"];
				$casualties = floor((($army_attack[$i]["carriers"] - $army_attack[$i]["casualties_carriers"]) / 100) * $percent);

				// we need to calculate the casualties related to the carriers itself (soldiers, fighters and generals)
				$army_attack[$i]["casualties_soldiers"] += ($casualties * (1 / CONF_CARRIER_SOLDIER));

				if ($army_attack[$i]["casualties_soldiers"] > $army_attack[$i]["convoy_soldiers"])
					$army_attack[$i]["casualties_soldiers"] = $army_attack[$i]["convoy_soldiers"];

				$army_attack[$i]["casualties_fighters"] += ($casualties * (1 / CONF_CARRIER_FIGHTER));
				if ($army_attack[$i]["casualties_fighters"] > $army_attack[$i]["convoy_fighters"])
					$army_attack[$i]["casualties_fighters"] = $army_attack[$i]["convoy_fighters"];

				$army_attack[$i]["casualties_carriers"] += $casualties;
				// print "CARRIERS CASUALTIES: " .$percent." %  (".$casualties.")<br/>";

			}

			for ($i = 0; $i < count($army_defense); $i++) {
				// print "DEFENSE CONVOY: $i<br/>";
				if ($total_defense_strength["soldiers"] != 0) {
					$percent = (((($army_defense[$i]["soldiers"] - $army_defense[$i]["casualties_soldiers"]) / $total_defense_strength["soldiers"]) * 100) / 100) * $casualties_defense;
					if ($percent > 100)
						$percent = 100;
					$casualties = floor((($army_defense[$i]["soldiers"] - $army_defense[$i]["casualties_soldiers"]) / 100) * $percent);
					$army_defense[$i]["casualties_soldiers"] += $casualties;
					// print "SOLDIERS CASUALTIES: " .$percent." % (".$casualties.")<br/>";
				}
				if ($total_defense_strength["fighters"] != 0) {
					$percent = (((($army_defense[$i]["fighters"] - $army_defense[$i]["casualties_fighters"]) / $total_defense_strength["fighters"]) * 100) / 100) * $casualties_defense;
					if ($percent > 100)
						$percent = 100;
					$casualties = floor((($army_defense[$i]["fighters"] - $army_defense[$i]["casualties_fighters"]) / 100) * $percent);
					$army_defense[$i]["casualties_fighters"] += $casualties;
					// print "FIGHTERS CASUALTIES: " .$percent." % (".$casualties.")<br/>";
				}

				if (isset ($army_defense[$i]["stations"]))
					if ($total_defense_strength["stations"] != 0) {
						$percent = (((($army_defense[$i]["stations"] - $army_defense[$i]["casualties_stations"]) / $total_defense_strength["stations"]) * 100) / 100) * $casualties_defense;
						if ($percent > 100)
							$percent = 100;
						$casualties = floor((($army_defense[$i]["stations"] - $army_defense[$i]["casualties_stations"]) / 100) * $percent);
						$army_defense[$i]["casualties_stations"] += $casualties;
						// print "STATIONS CASUALTIES: " .$percent." % (".$casualties.")<br/>";
					}

				if ($round >= CONF_INVASION_LIGHTCRUISER_PROTECTION_ROUNDS)
					if ($total_defense_strength["lightcruisers"] != 0) {
						$percent = (((($army_defense[$i]["lightcruisers"] - $army_defense[$i]["casualties_lightcruisers"]) / $total_defense_strength["lightcruisers"]) * 100) / 100) * $casualties_defense;
						if ($percent > 100)
							$percent = 100;
						$casualties = floor((($army_defense[$i]["lightcruisers"] - $army_defense[$i]["casualties_lightcruisers"]) / 100) * $percent);
						$army_defense[$i]["casualties_lightcruisers"] += $casualties;
						// print "LIGHTCRUISERS CASUALTIES: " .$percent." %  (".$casualties.")<br/>";
					}

				if ($total_defense_strength["heavycruisers"] != 0) {
					$percent = (((($army_defense[$i]["heavycruisers"] - $army_defense[$i]["casualties_heavycruisers"]) / $total_defense_strength["heavycruisers"]) * 100) / 100) * $casualties_defense;
					if ($percent > 100)
						$percent = 100;
					$casualties = floor((($army_defense[$i]["heavycruisers"] - $army_defense[$i]["casualties_heavycruisers"]) / 100) * $percent);
					$army_defense[$i]["casualties_heavycruisers"] += $casualties;
					// print "HEAVYCRUISERS CASUALTIES: " .$percent." %  (".$casualties.")<br/>";
				}

			}

			// print "<HR>";			

		}
		// print "Space attack result: (ATTACK: ".$space_won_attack." / DEFENSE: ".$space_won_defense.")<br/>";
		if ($space_won_attack < 3)
			return $this->Finish(false, false, false, $army_attack, $army_defense);

		///////////////////////////////////////////////////////////////////////////////////////////////////////
		// Orbital attack!
		///////////////////////////////////////////////////////////////////////////////////////////////////////
		// print "<B>ORBITAL</B><br/>";
		$orbital_won_attack = 0;
		$orbital_won_defense = 0;

		for ($round = 0; $round < 5; $round++) {

			// print "ROUND ".($round+1)."<br/>";
			$attack_strength = 1;
			$total_attack_strength = array ();
			$total_attack_strength["soldiers"] = 0;
			$total_attack_strength["fighters"] = 0;
			$total_attack_strength["lightcruisers"] = 0;
			$total_attack_strength["heavycruisers"] = 0;

			for ($i = 0; $i < count($army_attack); $i++) {

				$soldiers_strength = (($army_attack[$i]["convoy_soldiers"] - $army_attack[$i]["casualties_soldiers"]) * $units_info["soldiers_" . $army_attack[$i]["convoy_soldiers_level"]]["invasion_orbital"]);
				$soldiers_strength = floor(($soldiers_strength / 100) * $army_attack[$i]["effectiveness"]);
				$attack_strength += $soldiers_strength;

				$fighters_strength = (($army_attack[$i]["convoy_fighters"] - $army_attack[$i]["casualties_fighters"]) * $units_info["fighters_" . $army_attack[$i]["convoy_fighters_level"]]["invasion_orbital"]);
				$fighters_strength = floor(($fighters_strength / 100) * $army_attack[$i]["effectiveness"]);
				$attack_strength += $fighters_strength;

				$lightcruisers_strength = (($army_attack[$i]["convoy_lightcruisers"] - $army_attack[$i]["casualties_lightcruisers"]) * $units_info["lightcruisers_" . $army_attack[$i]["convoy_lightcruisers_level"]]["invasion_orbital"]);
				$lightcruisers_strength = floor(($lightcruisers_strength / 100) * $army_attack[$i]["effectiveness"]);
				$attack_strength += $lightcruisers_strength;

				$heavycruisers_strength = (($army_attack[$i]["convoy_heavycruisers"] - $army_attack[$i]["casualties_heavycruisers"]) * $units_info["heavycruisers_" . $army_attack[$i]["convoy_heavycruisers_level"]]["invasion_orbital"]);
				$heavycruisers_strength = floor(($heavycruisers_strength / 100) * $army_attack[$i]["effectiveness"]);
				$attack_strength += $heavycruisers_strength;

				$total_attack_strength["soldiers"] += $soldiers_strength;
				$total_attack_strength["fighters"] += $fighters_strength;
				$total_attack_strength["lightcruisers"] += $lightcruisers_strength;
				$total_attack_strength["heavycruisers"] += $heavycruisers_strength;
			}

			$defense_strength = 1;
			$total_defense_strength = array ();
			$total_defense_strength["soldiers"] = 0;
			$total_defense_strength["stations"] = 0;
			$total_defense_strength["fighters"] = 0;
			$total_defense_strength["lightcruisers"] = 0;
			$total_defense_strength["heavycruisers"] = 0;
			for ($i = 0; $i < count($army_defense); $i++) {

				$soldiers_strength = (($army_defense[$i]["soldiers"] - $army_defense[$i]["casualties_soldiers"]) * $units_info["soldiers_" . $army_defense[$i]["soldiers_level"]]["invasion_orbital"]);
				$soldiers_strength = floor(($soldiers_strength / 100) * $army_defense[$i]["effectiveness"]);
				$defense_strength += $soldiers_strength;
				
								$fighters_strength = (($army_defense[$i]["fighters"]-$army_defense[$i]["casualties_fighters"]) * $units_info["fighters_".$army_defense[$i]["fighters_level"]]["invasion_orbital"]);
								$fighters_strength = floor(($fighters_strength/100)*$army_defense[$i]["effectiveness"]);
				//				$defense_strength += $fighters_strength;
				

				if (isset($army_defense[$i]["stations"])) {
				$stations_strength = (($army_defense[$i]["stations"] - $army_defense[$i]["casualties_stations"]) * $units_info["fighters_" . $army_defense[$i]["stations_level"]]["invasion_orbital"]);
				$stations_strength = floor(($stations_strength / 100) * $army_defense[$i]["effectiveness"]);
				$defense_strength += $stations_strength;
			}
			
				$lightcruisers_strength = (($army_defense[$i]["lightcruisers"] - $army_defense[$i]["casualties_lightcruisers"]) * $units_info["lightcruisers_" . $army_defense[$i]["lightcruisers_level"]]["invasion_orbital"]);
				$soldiers_strength = floor(($soldiers_strength / 100) * $army_defense[$i]["effectiveness"]);
				$defense_strength += $lightcruisers_strength;

				$heavycruisers_strength = (($army_defense[$i]["heavycruisers"] - $army_defense[$i]["casualties_heavycruisers"]) * $units_info["heavycruisers_" . $army_defense[$i]["heavycruisers_level"]]["invasion_orbital"]);
				$heavycruisers_strength += ((($heavycruisers_strength / 100) * $this->target["commandship"]) * $units_info["commandship_" . $this->target["commandship_level"]]["bonus"]);
				$heavycruisers_strength = floor(($heavycruisers_strength / 100) * $army_defense[$i]["effectiveness"]);

				$defense_strength += $heavycruisers_strength;

				$total_defense_strength["soldiers"] += $soldiers_strength;
				$total_defense_strength["fighters"] += $fighters_strength;
				$total_defense_strength["stations"] += $stations_strength;
				$total_defense_strength["lightcruisers"] += $lightcruisers_strength;
				$total_defense_strength["heavycruisers"] += $heavycruisers_strength;

			}
			$defense_strength *= CONF_INVASION_DEFENSE_BONUS;

			$this->total_strength["ORBITAL"][$round] = array (
				"attack" => $total_attack_strength,
				"defense" => $total_defense_strength
			);

			//			$attack_strength = 8000;
			//			$defense_strength = 5000;	
			//			$attack_strength = 4000;
			//			$defense_strength = 5000;	

			// add randomness
			$attack_strength += floor(rand(0, CONF_INVASION_RANDOMNESS) * ($attack_strength / 100));
			$defense_strength += floor(rand(0, CONF_INVASION_RANDOMNESS) * ($defense_strength / 100));
			if ($attack_strength == 0)
				$attack_strength = 1;
			if ($defense_strength == 0)
				$defense_strength = 1;

			// print "ATTACK STRENGTH: ".$attack_strength."<br/>";
			// print "DEFENSE STRENGTH: ".$defense_strength."<br/>";
			$casualties = $attack_strength - $defense_strength;
			if ($casualties == 0)
				$casualties = 1;

			// print "RAW DAMAGE: ".$casualties."<br/>";
			$casualties_attack = 0;
			$casualties_defense = 0;

			if ($casualties <= 1) {
				// print "ATTACKERS LOOSE<br/>";
				$casualties_attack = (($defense_strength / $attack_strength) * 100);
				$casualties_defense = (($attack_strength / $defense_strength) * 100);
				$casualties_attack = ($casualties_attack) / 8;
				$casualties_defense = ($casualties_defense) / 12;
				$orbital_won_defense++;

			} else {
				// print "DEFENDERS LOOSE<br/>";
				$casualties_attack = (($defense_strength / $attack_strength) * 100);
				$casualties_defense = (($attack_strength / $defense_strength) * 100);
				$casualties_attack = ($casualties_attack) / 12;
				$casualties_defense = ($casualties_defense) / 8;
				$orbital_won_attack++;
			}

			if ($casualties_attack > 100)
				$casualties_attack = 100;
			if ($casualties_defense > 100)
				$casualties_defense = 100;
			$this->total_casualties["ORBITAL"][$round] = array (
				"attack" => $casualties_attack,
				"defense" => $casualties_defense
			);

			// print "CASUALTIES ATTACK: $casualties_attack<br/>";
			// print "CASUALTIES DEFENSE: $casualties_defense<br/>";
			// print "<br/>";

			// calculating casualties	
			for ($i = 0; $i < count($army_attack); $i++) {
				// print "ATTACK CONVOY: $i<br/>";
				if ($total_attack_strength["soldiers"] != 0) {
					$percent = (((($army_attack[$i]["convoy_soldiers"] - $army_attack[$i]["casualties_soldiers"]) / $total_attack_strength["soldiers"]) * 100) / 100) * $casualties_attack;
					if ($percent > 100)
						$percent = 100;
					$casualties = floor((($army_attack[$i]["convoy_soldiers"] - $army_attack[$i]["casualties_soldiers"]) / 100) * $percent);
					$army_attack[$i]["casualties_soldiers"] += $casualties;
					// print "SOLDIERS CASUALTIES: " .$percent." % (".$casualties.")<br/>";
				}
				if ($total_attack_strength["fighters"] != 0) {
					$percent = (((($army_attack[$i]["convoy_fighters"] - $army_attack[$i]["casualties_fighters"]) / $total_attack_strength["fighters"]) * 100) / 100) * $casualties_attack;
					if ($percent > 100)
						$percent = 100;
					$casualties = floor((($army_attack[$i]["convoy_fighters"] - $army_attack[$i]["casualties_fighters"]) / 100) * $percent);
					$army_attack[$i]["casualties_fighters"] += $casualties;
					// print "FIGHTERS CASUALTIES: " .$percent." % (".$casualties.")<br/>";
				}

				if ($round >= CONF_INVASION_LIGHTCRUISER_PROTECTION_ROUNDS)
					if ($total_attack_strength["lightcruisers"] != 0) {
						$percent = (((($army_attack[$i]["convoy_lightcruisers"] - $army_attack[$i]["casualties_lightcruisers"]) / $total_attack_strength["lightcruisers"]) * 100) / 100) * $casualties_attack;
						if ($percent > 100)
							$percent = 100;
						$casualties = floor((($army_attack[$i]["convoy_lightcruisers"] - $army_attack[$i]["casualties_lightcruisers"]) / 100) * $percent);
						$army_attack[$i]["casualties_lightcruisers"] += $casualties;
						// print "LIGHTCRUISERS CASUALTIES: " .$percent." %  (".$casualties.")<br/>";
					}

				if ($total_attack_strength["heavycruisers"] != 0) {
					$percent = (((($army_attack[$i]["convoy_heavycruisers"] - $army_attack[$i]["casualties_heavycruisers"]) / $total_attack_strength["heavycruisers"]) * 100) / 100) * $casualties_attack;
					if ($percent > 100)
						$percent = 100;
					$casualties = floor((($army_attack[$i]["convoy_heavycruisers"] - $army_attack[$i]["casualties_heavycruisers"]) / 100) * $percent);
					$army_attack[$i]["casualties_heavycruisers"] += $casualties;
					// print "HEAVYCRUISERS CASUALTIES: " .$percent." %  (".$casualties.")<br/>";
				}

			}

			for ($i = 0; $i < count($army_defense); $i++) {
				// print "DEFENSE CONVOY: $i<br/>";
				if ($total_defense_strength["soldiers"] != 0) {
					$percent = (((($army_defense[$i]["soldiers"] - $army_defense[$i]["casualties_soldiers"]) / $total_defense_strength["soldiers"]) * 100) / 100) * $casualties_defense;
					if ($percent > 100)
						$percent = 100;
					$casualties = floor((($army_defense[$i]["soldiers"] - $army_defense[$i]["casualties_soldiers"]) / 100) * $percent);
					$army_defense[$i]["casualties_soldiers"] += $casualties;
					// print "SOLDIERS CASUALTIES: " .$percent." % (".$casualties.")<br/>";
				}
				if ($total_defense_strength["fighters"] != 0) {
					$percent = (((($army_defense[$i]["fighters"] - $army_defense[$i]["casualties_fighters"]) / $total_defense_strength["fighters"]) * 100) / 100) * $casualties_defense;
					if ($percent > 100)
						$percent = 100;
					$casualties = floor((($army_defense[$i]["fighters"] - $army_defense[$i]["casualties_fighters"]) / 100) * $percent);
					$army_defense[$i]["casualties_fighters"] += $casualties;
					// print "FIGHTERS CASUALTIES: " .$percent." % (".$casualties.")<br/>";
				}

				if (isset ($army_defense[$i]["stations"]))
					if ($total_defense_strength["stations"] != 0) {
						$percent = (((($army_defense[$i]["stations"] - $army_defense[$i]["casualties_stations"]) / $total_defense_strength["stations"]) * 100) / 100) * $casualties_defense;
						if ($percent > 100)
							$percent = 100;
						$casualties = floor((($army_defense[$i]["stations"] - $army_defense[$i]["casualties_stations"]) / 100) * $percent);
						$army_defense[$i]["casualties_stations"] += $casualties;
						// print "STATIONS CASUALTIES: " .$percent." % (".$casualties.")<br/>";
					}

				if ($round >= CONF_INVASION_LIGHTCRUISER_PROTECTION_ROUNDS)
					if ($total_defense_strength["lightcruisers"] != 0) {
						$percent = (((($army_defense[$i]["lightcruisers"] - $army_defense[$i]["casualties_lightcruisers"]) / $total_defense_strength["lightcruisers"]) * 100) / 100) * $casualties_defense;
						if ($percent > 100)
							$percent = 100;
						$casualties = floor((($army_defense[$i]["lightcruisers"] - $army_defense[$i]["casualties_lightcruisers"]) / 100) * $percent);
						$army_defense[$i]["casualties_lightcruisers"] += $casualties;
						// print "LIGHTCRUISERS CASUALTIES: " .$percent." %  (".$casualties.")<br/>";
					}

				if ($total_defense_strength["heavycruisers"] != 0) {
					$percent = (((($army_defense[$i]["heavycruisers"] - $army_defense[$i]["casualties_heavycruisers"]) / $total_defense_strength["heavycruisers"]) * 100) / 100) * $casualties_defense;
					if ($percent > 100)
						$percent = 100;
					$casualties = floor((($army_defense[$i]["heavycruisers"] - $army_defense[$i]["casualties_heavycruisers"]) / 100) * $percent);
					$army_defense[$i]["casualties_heavycruisers"] += $casualties;
					// print "HEAVYCRUISERS CASUALTIES: " .$percent." %  (".$casualties.")<br/>";
				}

			}

			// print "<HR>";			

		}
		// print "Space attack result: (ATTACK: ".$orbital_won_attack." / DEFENSE: ".$orbital_won_defense.")<br/>";
		if ($orbital_won_attack < 3)
			return $this->Finish(true, false, false, $army_attack, $army_defense);

		///////////////////////////////////////////////////////////////////////////////////////////////////////
		// Ground attack!
		///////////////////////////////////////////////////////////////////////////////////////////////////////
		// print "<B>GROUND</B><br/>";
		$ground_won_attack = 0;
		$ground_won_defense = 0;

		for ($round = 0; $round < 5; $round++) {

			// print "ROUND ".($round+1)."<br/>";
			$attack_strength = 1;
			$total_attack_strength = array ();
			$total_attack_strength["soldiers"] = 0;
			$total_attack_strength["fighters"] = 0;
			$total_attack_strength["lightcruisers"] = 0;
			$total_attack_strength["heavycruisers"] = 0;

			for ($i = 0; $i < count($army_attack); $i++) {

				$soldiers_strength = (($army_attack[$i]["convoy_soldiers"] - $army_attack[$i]["casualties_soldiers"]) * $units_info["soldiers_" . $army_attack[$i]["convoy_soldiers_level"]]["invasion_ground"]);
				$soldiers_strength = floor(($soldiers_strength / 100) * $army_attack[$i]["effectiveness"]);
				$attack_strength += $soldiers_strength;

				$fighters_strength = (($army_attack[$i]["convoy_fighters"] - $army_attack[$i]["casualties_fighters"]) * $units_info["fighters_" . $army_attack[$i]["convoy_fighters_level"]]["invasion_ground"]);
				$fighters_strength = floor(($fighters_strength / 100) * $army_attack[$i]["effectiveness"]);
				$attack_strength += $fighters_strength;

				$lightcruisers_strength = (($army_attack[$i]["convoy_lightcruisers"] - $army_attack[$i]["casualties_lightcruisers"]) * $units_info["lightcruisers_" . $army_attack[$i]["convoy_lightcruisers_level"]]["invasion_ground"]);
				$lightcruisers_strength = floor(($lightcruisers_strength / 100) * $army_attack[$i]["effectiveness"]);
				$attack_strength += $lightcruisers_strength;

				$heavycruisers_strength = (($army_attack[$i]["convoy_heavycruisers"] - $army_attack[$i]["casualties_heavycruisers"]) * $units_info["heavycruisers_" . $army_attack[$i]["convoy_heavycruisers_level"]]["invasion_ground"]);
				$heavycruisers_strength = floor(($heavycruisers_strength / 100) * $army_attack[$i]["effectiveness"]);
				$attack_strength += $heavycruisers_strength;

				$total_attack_strength["soldiers"] += $soldiers_strength;
				$total_attack_strength["fighters"] += $fighters_strength;
				$total_attack_strength["lightcruisers"] += $lightcruisers_strength;
				$total_attack_strength["heavycruisers"] += $heavycruisers_strength;
			}
			//				print "<HR>";
			//				print_r($total_attack_strength);
			//				print "<BR/>";

			$defense_strength = 1;
			$total_defense_strength = array ();
			$total_defense_strength["soldiers"] = 0;
			$total_defense_strength["stations"] = 0;
			$total_defense_strength["fighters"] = 0;
			$total_defense_strength["lightcruisers"] = 0;
			$total_defense_strength["heavycruisers"] = 0;
			for ($i = 0; $i < count($army_defense); $i++) {

				$soldiers_strength = (($army_defense[$i]["soldiers"] - $army_defense[$i]["casualties_soldiers"]) * $units_info["soldiers_" . $army_defense[$i]["soldiers_level"]]["invasion_ground"]);
				$soldiers_strength = floor(($soldiers_strength / 100) * $army_defense[$i]["effectiveness"]);
				$defense_strength += $soldiers_strength;

				$fighters_strength = (($army_defense[$i]["fighters"] - $army_defense[$i]["casualties_fighters"]) * $units_info["fighters_" . $army_defense[$i]["fighters_level"]]["invasion_ground"]);
				$fighters_strength = floor(($fighters_strength / 100) * $army_defense[$i]["effectiveness"]);
			//	$defense_strength += $fighters_strength;

				if (isset($army_defense[$i]["stations"])) {
				$stations_strength = (($army_defense[$i]["stations"] - $army_defense[$i]["casualties_stations"]) * $units_info["fighters_" . $army_defense[$i]["stations_level"]]["invasion_ground"]);
				$stations_strength = floor(($stations_strength / 100) * $army_defense[$i]["effectiveness"]);
				$defense_strength += $stations_strength;
			}
			

				$lightcruisers_strength = (($army_defense[$i]["lightcruisers"] - $army_defense[$i]["casualties_lightcruisers"]) * $units_info["lightcruisers_" . $army_defense[$i]["lightcruisers_level"]]["invasion_ground"]);
				$lightcruisers_strength = floor(($lightcruisers_strength / 100) * $army_defense[$i]["effectiveness"]);
				$defense_strength += $lightcruisers_strength;

				$heavycruisers_strength = (($army_defense[$i]["heavycruisers"] - $army_defense[$i]["casualties_heavycruisers"]) * $units_info["heavycruisers_" . $army_defense[$i]["heavycruisers_level"]]["invasion_ground"]);
				$heavycruisers_strength += ((($heavycruisers_strength / 100) * $this->target["commandship"]) * $units_info["commandship_" . $this->target["commandship_level"]]["bonus"]);
				$heavycruisers_strength = floor(($heavycruisers_strength / 100) * $army_defense[$i]["effectiveness"]);

				$defense_strength += $heavycruisers_strength;
				$total_defense_strength["soldiers"] += $soldiers_strength;
				$total_defense_strength["fighters"] += $fighters_strength;
				$total_defense_strength["stations"] += $stations_strength;
				$total_defense_strength["lightcruisers"] += $lightcruisers_strength;
				$total_defense_strength["heavycruisers"] += $heavycruisers_strength;

			}
			//				print_r($total_defense_strength);
			$defense_strength *= CONF_INVASION_DEFENSE_BONUS;

			$this->total_strength["GROUND"][$round] = array (
				"attack" => $total_attack_strength,
				"defense" => $total_defense_strength
			);

			// add randomness
			$attack_strength += floor(rand(0, CONF_INVASION_RANDOMNESS) * ($attack_strength / 100));
			$defense_strength += floor(rand(0, CONF_INVASION_RANDOMNESS) * ($defense_strength / 100));

			//			print "<HR>";
			//			 print "ATTACK STRENGTH: ".$attack_strength."<br/>";
			//			 print "DEFENSE STRENGTH: ".$defense_strength."<br/>";
			$casualties = $attack_strength - $defense_strength;
			if ($casualties == 0)
				$casualties = 1;

			//			 print "RAW DAMAGE: ".$casualties."<br/>";
			$casualties_attack = 0;
			$casualties_defense = 0;

			if ($attack_strength == 0)
				$attack_strength = 1;
			if ($defense_strength == 0)
				$defense_strength = 1;

			if ($casualties <= 1) {
				//				print "ATTACKERS LOOSE<br/>";
				$casualties_attack = (($defense_strength / $attack_strength) * 100);
				$casualties_defense = (($attack_strength / $defense_strength) * 100);
				$casualties_attack = ($casualties_attack) / 8;
				$casualties_defense = ($casualties_defense) / 12;
				$ground_won_defense++;

			} else {
				//				print "DEFENDERS LOOSE<br/>";
				$casualties_attack = (($defense_strength / $attack_strength) * 100);
				$casualties_defense = (($attack_strength / $defense_strength) * 100);
				$casualties_attack = ($casualties_attack) / 12;
				$casualties_defense = ($casualties_defense) / 8;
				$ground_won_attack++;
			}

			if ($casualties_attack > 100)
				$casualties_attack = 100;
			if ($casualties_defense > 100)
				$casualties_defense = 100;
			if ($casualties_attack < 0)
				$casualties_attack = 0;
			if ($casualties_defense < 0)
				$casualties_defense = 0;

			$this->total_casualties["GROUND"][$round] = array (
				"attack" => $casualties_attack,
				"defense" => $casualties_defense
			);

			//			print "CASUALTIES ATTACK: $casualties_attack<br/>";
			//			print "CASUALTIES DEFENSE: $casualties_defense<br/>";
			//			print "<br/>";

			// calculating casualties	
			for ($i = 0; $i < count($army_attack); $i++) {
				//				 print "ATTACK CONVOY: $i<br/>";
				if ($total_attack_strength["soldiers"] != 0) {
					$percent = (((($army_attack[$i]["convoy_soldiers"] - $army_attack[$i]["casualties_soldiers"]) / $total_attack_strength["soldiers"]) * 100) / 100) * $casualties_attack;
					if ($percent > 100)
						$percent = 100;
					$casualties = floor((($army_attack[$i]["convoy_soldiers"] - $army_attack[$i]["casualties_soldiers"]) / 100) * $percent);
					$army_attack[$i]["casualties_soldiers"] += $casualties;
					//					 print "SOLDIERS CASUALTIES: " .$percent." % (".$casualties.") ".$army_attack[$i]["casualties_soldiers"]."<br/>";
				}

				if ($total_attack_strength["fighters"] != 0) {
					$percent = (((($army_attack[$i]["convoy_fighters"] - $army_attack[$i]["casualties_fighters"]) / $total_attack_strength["fighters"]) * 100) / 100) * $casualties_attack;
					if ($percent > 100)
						$percent = 100;
					$casualties = floor((($army_attack[$i]["convoy_fighters"] - $army_attack[$i]["casualties_fighters"]) / 100) * $percent);
					$army_attack[$i]["casualties_fighters"] += $casualties;
					//					print "FIGHTERS CASUALTIES: " .$percent." % (".$casualties.")<br/>";
				}

				if ($round >= CONF_INVASION_LIGHTCRUISER_PROTECTION_ROUNDS)
					if ($total_attack_strength["lightcruisers"] != 0) {
						$percent = (((($army_attack[$i]["convoy_lightcruisers"] - $army_attack[$i]["casualties_lightcruisers"]) / $total_attack_strength["lightcruisers"]) * 100) / 100) * $casualties_attack;
						if ($percent > 100)
							$percent = 100;
						$casualties = floor((($army_attack[$i]["convoy_lightcruisers"] - $army_attack[$i]["casualties_lightcruisers"]) / 100) * $percent);
						$army_attack[$i]["casualties_lightcruisers"] += $casualties;
						//					print "LIGHTCRUISERS CASUALTIES: " .$percent." %  (".$casualties.")<br/>";
					}

				if ($total_attack_strength["heavycruisers"] != 0) {
					$percent = (((($army_attack[$i]["convoy_heavycruisers"] - $army_attack[$i]["casualties_heavycruisers"]) / $total_attack_strength["heavycruisers"]) * 100) / 100) * $casualties_attack;
					if ($percent > 100)
						$percent = 100;
					$casualties = floor((($army_attack[$i]["convoy_heavycruisers"] - $army_attack[$i]["casualties_heavycruisers"]) / 100) * $percent);
					$army_attack[$i]["casualties_heavycruisers"] += $casualties;
					//					print "HEAVYCRUISERS CASUALTIES: " .$percent." %  (".$casualties.")<br/>";
				}

			}

			for ($i = 0; $i < count($army_defense); $i++) {
				//				 print "DEFENSE CONVOY: $i<br/>";
				if ($total_defense_strength["soldiers"] != 0) {
					$percent = (((($army_defense[$i]["soldiers"] - $army_defense[$i]["casualties_soldiers"]) / $total_defense_strength["soldiers"]) * 100) / 100) * $casualties_defense;
					if ($percent > 100)
						$percent = 100;
					$casualties = floor((($army_defense[$i]["soldiers"] - $army_defense[$i]["casualties_soldiers"]) / 100) * $percent);
					$army_defense[$i]["casualties_soldiers"] += $casualties;
					//					print "SOLDIERS CASUALTIES: " .$percent." % (".$casualties.")<br/>";
				}
				if ($total_defense_strength["fighters"] != 0) {
					$percent = (((($army_defense[$i]["fighters"] - $army_defense[$i]["casualties_fighters"]) / $total_defense_strength["fighters"]) * 100) / 100) * $casualties_defense;
					if ($percent > 100)
						$percent = 100;
					$casualties = floor((($army_defense[$i]["fighters"] - $army_defense[$i]["casualties_fighters"]) / 100) * $percent);
					$army_defense[$i]["casualties_fighters"] += $casualties;
					//					print "FIGHTERS CASUALTIES: " .$percent." % (".$casualties.")<br/>";
				}

				if ($total_defense_strength["stations"] != 0) {
				
								if (isset($army_defense[$i]["stations"])) {

					$percent = (((($army_defense[$i]["stations"] - $army_defense[$i]["casualties_stations"]) / $total_defense_strength["stations"]) * 100) / 100) * $casualties_defense;
					if ($percent > 100)
						$percent = 100;
					$casualties = floor((($army_defense[$i]["stations"] - $army_defense[$i]["casualties_stations"]) / 100) * $percent);
					$army_defense[$i]["casualties_stations"] += $casualties;
					//					print "STATIONS CASUALTIES: " .$percent." % (".$casualties.")<br/>";
					}
					
				}

				if ($round >= CONF_INVASION_LIGHTCRUISER_PROTECTION_ROUNDS)
					if ($total_defense_strength["lightcruisers"] != 0) {
						$percent = (((($army_defense[$i]["lightcruisers"] - $army_defense[$i]["casualties_lightcruisers"]) / $total_defense_strength["lightcruisers"]) * 100) / 100) * $casualties_defense;
						if ($percent > 100)
							$percent = 100;
						$casualties = floor((($army_defense[$i]["lightcruisers"] - $army_defense[$i]["casualties_lightcruisers"]) / 100) * $percent);
						$army_defense[$i]["casualties_lightcruisers"] += $casualties;
						//					print "LIGHTCRUISERS CASUALTIES: " .$percent." %  (".$casualties.")<br/>";
					}

				if ($total_defense_strength["heavycruisers"] != 0) {
					$percent = (((($army_defense[$i]["heavycruisers"] - $army_defense[$i]["casualties_heavycruisers"]) / $total_defense_strength["heavycruisers"]) * 100) / 100) * $casualties_defense;
					if ($percent > 100)
						$percent = 100;
					$casualties = floor((($army_defense[$i]["heavycruisers"] - $army_defense[$i]["casualties_heavycruisers"]) / 100) * $percent);
					$army_defense[$i]["casualties_heavycruisers"] += $casualties;
					//					print "HEAVYCRUISERS CASUALTIES: " .$percent." %  (".$casualties.")<br/>";
				}

			}

			//			// print "<HR>";			

		}

		//		die();

		if ($ground_won_attack < 3)
			return $this->Finish(true, false, false, $army_attack, $army_defense);

		$this->Finish(true, true, true, $army_attack, $army_defense);

	}

	//////////////////////////////////////////////////////////////////////////////////////
	//
	//////////////////////////////////////////////////////////////////////////////////////
	function Finish($space, $orbital, $ground, $army_attack, $army_defense) {
		global $GAME;

		$strength = $this->total_strength["SPACE"][0];

		if ($ground == true) {
			// invasion worked

			$target_empire = new Empire($this->DB, $GAME["template"], $GAME["gameplay_costs"]);
			$target_empire->Load($army_defense[0]["empire"]);

			// Calculating losts for the target
			$percent = 0;
			for ($i = 0; $i < count($this->total_casualties["GROUND"]); $i++) {
				$percent = ($percent + $this->total_casualties["GROUND"][$i]["defense"]) / 2;
			}

			$percent += CONF_INVASION_PLANETS_MINIMUM;
			if ($percent > CONF_INVASION_PLANETS_MAXIMUM)
				$percent = CONF_INVASION_PLANETS_MAXIMUM;
			$percent = floor($percent);

			$lost_population = floor(($target_empire->data["population"] / 100) * $percent);
			$lost_credits = floor(($target_empire->data["credits"] / 100) * $percent);

			$target_empire->data["population"] -= $lost_population;
			$target_empire->data["credits"] -= $lost_credits;

			$lost_food_planets = floor(($target_empire->planets->data["food_planets"] / 100) * $percent);
			$lost_ore_planets = floor(($target_empire->planets->data["ore_planets"] / 100) * $percent);
			$lost_tourism_planets = floor(($target_empire->planets->data["tourism_planets"] / 100) * $percent);
			$lost_supply_planets = floor(($target_empire->planets->data["supply_planets"] / 100) * $percent);
			$lost_gov_planets = floor(($target_empire->planets->data["gov_planets"] / 100) * $percent);
			$lost_edu_planets = floor(($target_empire->planets->data["edu_planets"] / 100) * $percent);
			$lost_research_planets = floor(($target_empire->planets->data["research_planets"] / 100) * $percent);
			$lost_urban_planets = floor(($target_empire->planets->data["urban_planets"] / 100) * $percent);
			$lost_petro_planets = floor(($target_empire->planets->data["petro_planets"] / 100) * $percent);
			$lost_antipollu_planets = floor(($target_empire->planets->data["antipollu_planets"] / 100) * $percent);

			// distribute equally
			for ($i = 0; $i < count($army_attack); $i++) {

				$temp_empire = new Empire($this->DB, $GAME["template"], $GAME["gameplay_costs"]);
				$temp_empire->Load($army_attack[$i]["empire_from"]);
				$temp_empire->planets->data["food_planets"] += floor($lost_food_planets / count($army_attack));
				$temp_empire->planets->data["ore_planets"] += floor($lost_ore_planets / count($army_attack));
				$temp_empire->planets->data["tourism_planets"] += floor($lost_tourism_planets / count($army_attack));
				$temp_empire->planets->data["supply_planets"] += floor($lost_supply_planets / count($army_attack));
				$temp_empire->planets->data["gov_planets"] += floor($lost_gov_planets / count($army_attack));
				$temp_empire->planets->data["edu_planets"] += floor($lost_edu_planets / count($army_attack));
				$temp_empire->planets->data["research_planets"] += floor($lost_research_planets / count($army_attack));
				$temp_empire->planets->data["urban_planets"] += floor($lost_urban_planets / count($army_attack));
				$temp_empire->planets->data["petro_planets"] += floor($lost_petro_planets / count($army_attack));
				$temp_empire->planets->data["antipollu_planets"] += floor($lost_antipollu_planets / count($army_attack));
				$temp_empire->data["credits"] += floor($lost_credits / count($army_attack));
				$temp_empire->Save();
				unset ($temp_empire);
			}

			$target_empire->planets->data["food_planets"] -= $lost_food_planets;
			$target_empire->planets->data["ore_planets"] -= $lost_ore_planets;
			$target_empire->planets->data["tourism_planets"] -= $lost_tourism_planets;
			$target_empire->planets->data["supply_planets"] -= $lost_supply_planets;
			$target_empire->planets->data["gov_planets"] -= $lost_gov_planets;
			$target_empire->planets->data["edu_planets"] -= $lost_edu_planets;
			$target_empire->planets->data["research_planets"] -= $lost_research_planets;
			$target_empire->planets->data["urban_planets"] -= $lost_urban_planets;
			$target_empire->planets->data["petro_planets"] -= $lost_petro_planets;
			$target_empire->planets->data["antipollu_planets"] -= $lost_antipollu_planets;
			$target_empire->planets->data["max_ore"] = $target_empire->planets->data["ore_planets"];
			$target_empire->planets->data["max_food"] = $target_empire->planets->data["food_planets"];
			$target_empire->planets->data["max_tourism"] = $target_empire->planets->data["tourism_planets"];
			$target_empire->planets->data["max_supply"] = $target_empire->planets->data["supply_planets"];
			$target_empire->planets->data["max_gov"] = $target_empire->planets->data["gov_planets"];
			$target_empire->planets->data["max_edu"] = $target_empire->planets->data["edu_planets"];
			$target_empire->planets->data["max_research"] = $target_empire->planets->data["research_planets"];
			$target_empire->planets->data["max_urban"] = $target_empire->planets->data["urban_planets"];
			$target_empire->planets->data["max_antipollu"] = $target_empire->planets->data["antipollu_planets"];

			$target_empire->army->data["commandship"] -= 25;
			if ($target_empire->army->data["commandship"] < 0)
				$target_empire->army->data["commandship"] = 0;

			$target_empire->data["civil_status"]++;
			if ($target_empire->data["civil_status"] > 7) {
				$r = rand(0, 5);
				$target_empire->data["civil_status"] = 7;
				if ($r == 1) {

					$evt = new EventCreator($this->DB);
					$evt->type = CONF_EVENT_ELIMINATED;
					$evt->from = -1;

					$rs_from = $this->DB->Execute("SELECT id,emperor,name,empires_eliminated,networth FROM tb_empire WHERE id=" . $army_attack[0]["empire_from"]);
					if (!$rs_from)
						trigger_error($this->DB->ErrorMsg());
					$rs_to = $this->DB->Execute("SELECT id,emperor,name,networth FROM tb_empire WHERE id=" . $army_defense[0]["empire"]);
					if (!$rs_to)
						trigger_error($this->DB->ErrorMsg());
					$empire_from = $GAME["template"]->DisplayEmpireHTML($rs_from->fields["id"], $rs_from->fields["emperor"], $rs_from->fields["name"], "");
					$empire_to = $GAME["template"]->DisplayEmpireHTML($rs_to->fields["id"], $rs_to->fields["emperor"], $rs_to->fields["name"], "");
					$evt->params = array (
						"empire_from" => $empire_from,
						"empire_to" => $empire_to
					);
					if (!$this->DB->Execute("UPDATE tb_empire SET empires_eliminated=" . ($rs_from["empires_eliminated"] + 1) . " WHERE id=" . $rs_from["id"]))
						trigger_error($this->DB->ErrorMsg());

					$evt->Broadcast();

					$target_empire->UpdateNetworth();
					$target_empire->Save();
					$target_empire->Collapse();
				}

			} else {
				$target_empire->UpdateNetworth();
				$target_empire->Save();
			}

			// UPDATING EFFECTIVENESS
			for ($i = 0; $i < count($army_attack); $i++) {
				$army_attack[$i]["effectiveness"] += CONF_INVASION_EFFECTIVENESS_WON;
				if ($army_attack[$i]["effectiveness"] > 100)
					$army_attack[$i]["effectiveness"] = 100;
				if (!$this->DB->Execute("UPDATE tb_army SET effectiveness=" . $army_attack[$i]["effectiveness"] . " WHERE empire=" . $army_attack[$i]["empire_from"]))
					trigger_error($this->DB->ErrorMsg());
			}

			$army_defense[0]["effectiveness"] -= CONF_INVASION_EFFECTIVENESS_LOST;
			if ($army_attack[0]["effectiveness"] < 10)
				$army_attack[0]["effectiveness"] = 10;
			if (!$this->DB->Execute("UPDATE tb_army SET effectiveness=" . $army_defense[0]["effectiveness"] . " WHERE empire=" . $army_defense[0]["empire"]))
				trigger_error($this->DB->ErrorMsg());

			if (count($army_defense) != 1)
				for ($i = 1; $i < count($army_defense); $i++) {
					$army_defense[$i]["effectiveness"] -= CONF_INVASION_EFFECTIVENESS_LOST;
					if ($army_defense[$i]["effectiveness"] < 10)
						$army_defense[$i]["effectiveness"] = 10;
					if (!$this->DB->Execute("UPDATE tb_army SET effectiveness=" . $army_defense[$i]["effectiveness"] . " WHERE empire=" . $army_defense[$i]["empire_from"]))
						trigger_error($this->DB->ErrorMsg());

				}

			$evt = new EventCreator($this->DB);
			$evt->type = CONF_EVENT_INVADED;
			$evt->from = -1;
			$rs_from = $this->DB->Execute("SELECT id,emperor,name,networth FROM tb_empire WHERE id=" . $army_attack[0]["empire_from"]);
			if (!$rs_from)
				trigger_error($this->DB->ErrorMsg());
			$rs_to = $this->DB->Execute("SELECT id,emperor,name,networth FROM tb_empire WHERE id=" . $army_defense[0]["empire"]);
			if (!$rs_to)
				trigger_error($this->DB->ErrorMsg());
			$empire_from = $GAME["template"]->DisplayEmpireHTML($rs_from->fields["id"], $rs_from->fields["emperor"], $rs_from->fields["name"], "");
			$empire_to = $GAME["template"]->DisplayEmpireHTML($rs_to->fields["id"], $rs_to->fields["emperor"], $rs_to->fields["name"], "");
			$evt->params = array (
				"won" => true,
				"empire_from" => $empire_from,
				"empire_to" => $empire_to
			);
			$evt->broadcast();

			// Sending INVASION REPORT

			$evt = new EventCreator($this->DB);
			$evt->type = CONF_EVENT_INVASION_REPORT;
			$evt->from = -1;
			$evt->params = array (
				"space_won" => $space,
				"orbital_won" => $orbital,
				"ground_won" => $ground,
				"lost_population" => $lost_population,
				"lost_credits" => $lost_credits,
				"lost_food_planets" => $lost_food_planets,
				"lost_ore_planets" => $lost_ore_planets,
				"lost_supply_planets" => $lost_supply_planets,
				"lost_tourism_planets" => $lost_tourism_planets,
				"lost_gov_planets" => $lost_gov_planets,
				"lost_edu_planets" => $lost_edu_planets,
				"lost_research_planets" => $lost_research_planets,
				"lost_urban_planets" => $lost_urban_planets,
				"lost_petro_planets" => $lost_petro_planets,
				"lost_antipollu_planets" => $lost_antipollu_planets,
				"army_attack" => $army_attack,
				"army_defense" => $army_defense,
				"strength" => $strength
			);

			for ($i = 0; $i < count($army_attack); $i++) {
				$evt->to = $army_attack[$i]["empire_from"];
				$evt->Send();
			}

			$evt->to = $army_defense[0]["empire"];
			$evt->Send();

			if (count($army_defense) != 1)
				for ($i = 0; $i < count($army_defense); $i++) {
					if ($i == 0)
						$evt->to = $army_defense[$i]["empire"];
					else
						$evt->to = $army_defense[$i]["empire_from"];
					$evt->Send();
				}

		} else {
			// invasion not worked

			//			print "INVASION NOT WORKED";

			// UPDATING EFFECTIVENESS
			for ($i = 0; $i < count($army_attack); $i++) {
				$army_attack[$i]["effectiveness"] -= CONF_INVASION_EFFECTIVENESS_LOST;
				if ($army_attack[$i]["effectiveness"] < 5)
					$army_attack[$i]["effectiveness"] = 5;
				if (!$this->DB->Execute("UPDATE tb_army SET effectiveness=" . $army_attack[$i]["effectiveness"] . " WHERE empire=" . $army_attack[$i]["empire_from"]))
					trigger_error($this->DB->ErrorMsg());
			}

			$army_defense[0]["effectiveness"] += CONF_INVASION_EFFECTIVENESS_WON;
			if ($army_defense[0]["effectiveness"] > 100)
				$army_defense[0]["effectiveness"] = 100;
			if (!$this->DB->Execute("UPDATE tb_army SET effectiveness=" . $army_defense[0]["effectiveness"] . " WHERE empire=" . $army_defense[0]["empire"]))
				trigger_error($this->DB->ErrorMsg());

			if (count($army_defense) != 1)
				for ($i = 1; $i < count($army_defense); $i++) {
					$army_defense[$i]["effectiveness"] += CONF_INVASION_EFFECTIVENESS_WON;
					if ($army_defense[$i]["effectiveness"] > 100)
						$army_defense[$i]["effectiveness"] = 100;
					if (!$this->DB->Execute("UPDATE tb_army SET effectiveness=" . $army_defense[$i]["effectiveness"] . " WHERE empire=" . $army_defense[$i]["empire_from"]))
						trigger_error($this->DB->ErrorMsg());
				}

			$evt = new EventCreator($this->DB);
			$evt->type = CONF_EVENT_INVADED;
			$evt->from = -1;
			$rs_from = $this->DB->Execute("SELECT id,emperor,name,networth FROM tb_empire WHERE id=" . $army_attack[0]["empire_from"]);
			if (!$rs_from)
				trigger_error($this->DB->ErrorMsg());
			$rs_to = $this->DB->Execute("SELECT id,emperor,name,networth FROM tb_empire WHERE id=" . $army_defense[0]["empire"]);
			if (!$rs_to)
				trigger_error($this->DB->ErrorMsg());
			$empire_from = $GAME["template"]->DisplayEmpireHTML($rs_from->fields["id"], $rs_from->fields["emperor"], $rs_from->fields["name"], "");
			$empire_to = $GAME["template"]->DisplayEmpireHTML($rs_to->fields["id"], $rs_to->fields["emperor"], $rs_to->fields["name"], "");
			$evt->params = array (
				"won" => false,
				"empire_from" => $empire_from,
				"empire_to" => $empire_to
			);
			$evt->broadcast();

			// Sending INVASION REPORT

			$evt = new EventCreator($this->DB);
			$evt->type = CONF_EVENT_INVASION_REPORT;
			$evt->from = -1;
			$evt->params = array (
				"space_won" => $space,
				"orbital_won" => $orbital,
				"ground_won" => $ground,
				"army_attack" => $army_attack,
				"army_defense" => $army_defense,
				"strength" => $strength
			);

			for ($i = 0; $i < count($army_attack); $i++) {
				$evt->to = $army_attack[$i]["empire_from"];
				$evt->Send();
			}

			$evt->to = $army_defense[0]["empire"];
			$evt->Send();

			if (count($army_defense) != 1)
				for ($i = 0; $i < count($army_defense); $i++) {
					if ($i == 0)
						$evt->to = $army_defense[$i]["empire"];
					else
						$evt->to = $army_defense[$i]["empire_from"];

					$evt->Send();
				}

		}

		for ($i = 0; $i < count($army_attack); $i++) {

			$query = "UPDATE tb_armyconvoy SET " .
			"convoy_type=" .CONF_CONVOY_INVASION_RETREAT . "," .
			"convoy_generals=" . ($army_attack[$i]["convoy_generals"] - ceil($army_attack[$i]["casualties_soldiers"] / 50)) . "," .
			"convoy_soldiers=" . ($army_attack[$i]["convoy_soldiers"] - $army_attack[$i]["casualties_soldiers"]) . "," .
			"convoy_fighters=" . ($army_attack[$i]["convoy_fighters"] - $army_attack[$i]["casualties_fighters"]) . "," .
			"convoy_lightcruisers=" . ($army_attack[$i]["convoy_lightcruisers"] - $army_attack[$i]["casualties_lightcruisers"]) . "," .
			"convoy_heavycruisers=" . ($army_attack[$i]["convoy_heavycruisers"] - $army_attack[$i]["casualties_heavycruisers"]) . "," .
			"carriers=" . ($army_attack[$i]["carriers"] - $army_attack[$i]["casualties_carriers"]) . "," .
			"time_left=" . $army_attack[$i]["time_total"] . " WHERE id=" . $army_attack[$i]["id"];

			if (!$this->DB->Execute($query))
				trigger_error($this->DB->ErrorMsg());
				
				
			$evt = new EventCreator($this->DB);
			$evt->from = -1;
			$evt->to =  $army_attack[$i]["empire_to"];
			$evt->type = CONF_EVENT_CONVOY_RETREAT;
			$evt->params = $army_attack[$i]["empire_from"];
			$evt->send();
			
		}

		// updating target army
		$query = "UPDATE tb_army SET " .
		"generals=" . ($army_defense[0]["generals"] - ceil($army_defense[0]["casualties_soldiers"] / 50)) . "," .
		"soldiers=" . ($army_defense[0]["soldiers"] - $army_defense[0]["casualties_soldiers"]) . "," .
		"fighters=" . ($army_defense[0]["fighters"] - $army_defense[0]["casualties_fighters"]) . "," .
		"stations=" . ($army_defense[0]["stations"] - $army_defense[0]["casualties_stations"]) . "," .
		"lightcruisers=" . ($army_defense[0]["lightcruisers"] - $army_defense[0]["casualties_lightcruisers"]) . "," .
		"heavycruisers=" . ($army_defense[0]["heavycruisers"] - $army_defense[0]["casualties_heavycruisers"]) .
		" WHERE id=" . $army_defense[0]["id"];

		if (!$this->DB->Execute($query))
			trigger_error($this->DB->ErrorMsg());

		if (count($army_defense) != 1) {

			for ($i = 1; $i < count($army_defense); $i++) {

				$query = "UPDATE tb_armyconvoy SET " .
				"convoy_type=" .CONF_CONVOY_DEFENSE_RETREAT . "," .
				"convoy_generals=" . ($army_defense[$i]["convoy_generals"] - ceil($army_defense[$i]["casualties_soldiers"] / 50)) . "," .
				"convoy_soldiers=" . ($army_defense[$i]["convoy_soldiers"] - $army_defense[$i]["casualties_soldiers"]) . "," .
				"convoy_fighters=" . ($army_defense[$i]["convoy_fighters"] - $army_defense[$i]["casualties_fighters"]) . "," .
				"convoy_lightcruisers=" . ($army_defense[$i]["convoy_lightcruisers"] - $army_defense[$i]["casualties_lightcruisers"]) . "," .
				"convoy_heavycruisers=" . ($army_defense[$i]["convoy_heavycruisers"] - $army_defense[$i]["casualties_heavycruisers"]) . "," .
				"time_left=" . $army_defense[$i]["time_total"] . " WHERE id=" . $army_defense[$i]["id"];

				if (!$this->DB->Execute($query))
					trigger_error($this->DB->ErrorMsg());
					
						
				$evt = new EventCreator($this->DB);
				$evt->from = -1;
				$evt->to =  $army_defense[$i]["empire_to"];
				$evt->type = CONF_EVENT_CONVOY_RETREAT;
				$evt->params = $army_defense[$i]["empire_from"];
				$evt->send();
						
			}

		}

	}

}
?>
