<?php

//---------------------------------------------------------------------------//
// author: Yanick Bourbeau <ybourbeau@mrgtech.ca> 
// date:   2006.01.01
// web:    http://www.mrgtech.ca
// info:   Solar Imperium
//---------------------------------------------------------------------------//
// copyleft license
//
// this software is provided 'as-is', without any express or implied
// warranty. in no event will the authors be held liable for any damages
// arising from the use of this software.
//
// permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions: 
//
// 1. the origin of this software must not be misrepresented;
//	  you must not claim that you wrote the original software. 
//	  if you use this software in a product, an acknowledgment 
//	  in the product documentation would be appreciated but is not required.
//
// 2. altered source versions must be plainly marked as such, 
//	  and must not be misrepresented as being the original software.
//
// 3. mail about the fact of using this class in production 
//	  would be very appreciated.
//
// 4. this notice may not be removed or altered from any source distribution.
//
//---------------------------------------------------------------------------//
// CHANGELOG:
//
// 2006.06.01 : ybo : Changed name: Imperium Galactica is (c), moved to Solar Imperium
// 2006.09.01 : ybo : Major code changes, OOP
// 2007.02.04 : ybo : Reviewed the code for public release
//
//---------------------------------------------------------------------------//

class System {
	var $DB;
	var $game_status;

	//////////////////////////////////////////////////////////////////////
	// Constructor
	//////////////////////////////////////////////////////////////////////
	function System($DB) {

		$this->DB = $DB;
		$this->game_status = 0;
	}

	//////////////////////////////////////////////////////////////////////
	// Initialization
	//////////////////////////////////////////////////////////////////////
	function init() {

		// update turns if needed
		$this->DB->StartTrans();

		srand(time(NULL));
		$now = time(NULL);

		$timeslice = floor((60 * 60 * 24) / CONF_TURNS_PER_DAY);

		$rs = $this->DB->Execute("SELECT last_turns_update,last_time_update,game_status FROM tb_coordinator");
		$elapsed = $now - $rs->fields["last_turns_update"];
		$elapsed2 = $now - $rs->fields["last_time_update"];
		$this->game_status = $rs->fields["game_status"];

		// 60 second update)
		$delay = $timeslice;
		if ($delay > 60)
			$delay = 60;
		if ($elapsed2 >= $delay) {
			$this->handleTradeConvoys($elapsed2);
			$this->handleArmyConvoys($elapsed2);
			$query = "UPDATE tb_coordinator SET last_time_update=" . $now;
			$this->DB->Execute($query);

		}
		if ($elapsed >= $timeslice) {

			$turns = floor($elapsed / $timeslice);
			for ($i = 0; $i < count($turns); $i++) {

				$this->handleAITurns();

			}

			$rs = $this->DB->Execute("SELECT * FROM tb_empire WHERE active=1");
			while (!$rs->EOF) {
				$newturns = $rs->fields["turns_left"] + $turns;

				if ($newturns > CONF_MAX_STORED_TURNS)
					$newturns = CONF_MAX_STORED_TURNS;

				$query = "UPDATE tb_empire " .
				"SET turns_left=$newturns WHERE id=" . $rs->fields["id"];

				$this->DB->Execute($query);
				$rs->MoveNext();
			}

			$query = "UPDATE tb_coordinator SET last_turns_update=" . $now;
			$this->DB->Execute($query);

			$this->handleVictoryConditions();

		}

		$this->DB->CompleteTrans();

	}

	//////////////////////////////////////////////////////////////////////
	// Handle trade convoys
	//////////////////////////////////////////////////////////////////////
	function handleTradeConvoys($elapsed) {
		global $GAME;

		$rs = $this->DB->Execute("SELECT * FROM tb_tradeconvoy");
		while (!$rs->EOF) {

			$rs->fields["time_left"] -= $elapsed;
			if ($rs->fields["time_left"] <= 0) {

				$source_empire = new Empire($this->DB, $GAME["template"], $GAME["gameplay_costs"]);
				$retval1 = $source_empire->load($rs->fields["empire_from"]);
				$target_empire = new Empire($this->DB, $GAME["template"], $GAME["gameplay_costs"]);
				$retval2 = $target_empire->load($rs->fields["empire_to"]);
				if (($retval1["code"] == true) && ($retval2["code"] == true)) {

					// time to unload stuff
					$target_empire->data["credits"] += $rs->fields["trade_money"];
					$target_empire->data["food"] += $rs->fields["trade_food"];
					$target_empire->army->data["generals"] += $rs->fields["trade_generals"];
					$target_empire->army->data["covertagents"] += $rs->fields["trade_covertagents"];
					$target_empire->army->data["soldiers"] += $rs->fields["trade_generals"];
					$target_empire->army->data["fighters"] += $rs->fields["trade_fighters"];
					$target_empire->army->data["stations"] += $rs->fields["trade_stations"];
					$target_empire->army->data["lightcruisers"] += $rs->fields["trade_lightcruisers"];
					$target_empire->army->data["heavycruisers"] += $rs->fields["trade_heavycruisers"];
					$target_empire->save();

					$source_empire->army->data["carriers"] += $rs->fields["carriers"];
					$source_empire->save();

					// send notifications
					$evt = new EventCreator($this->DB);
					$evt->type = CONF_EVENT_TRADECONVOY_RECEIVED;
					$evt->from = -1;
					$evt->params = $rs->fields;
					$evt->to = $rs->fields["empire_from"];
					$evt->send();

					$evt->to = $rs->fields["empire_to"];
					$evt->send();
				}

				$this->DB->Execute("DELETE FROM tb_tradeconvoy WHERE id=" . $rs->fields["id"]);

			} else {

				// check if pirates attack
				$r = rand(0, CONF_TRADE_PIRATERAID);
				if ($r == 0) {

					$pirates = array ();
					$rs2 = $this->DB->Execute("SELECT * FROM tb_pirate");
					while (!$rs2->EOF) {
						$pirates[] = $rs2->fields;

						$rs2->MoveNext();
					}

					$pirate = $pirates[rand(0, count($pirates) - 1)];

					$stolen_food = floor(($rs->fields["trade_food"] / 100) * rand(0, 10));
					$stolen_credits = floor(($rs->fields["trade_money"] / 100) * rand(0, 10));
					$stolen_generals = floor(($rs->fields["trade_generals"] / 100) * rand(0, 10));
					$stolen_covertagents = floor(($rs->fields["trade_covertagents"] / 100) * rand(0, 10));
					$stolen_soldiers = floor(($rs->fields["trade_soldiers"] / 100) * rand(0, 10));
					$stolen_fighters = floor(($rs->fields["trade_fighters"] / 100) * rand(0, 10));
					$stolen_lightcruisers = floor(($rs->fields["trade_lightcruisers"] / 100) * rand(0, 10));
					$stolen_heavycruisers = floor(($rs->fields["trade_heavycruisers"] / 100) * rand(0, 10));

					$pirate["food"] += $stolen_food;
					$pirate["credits"] += $stolen_credits;
					$pirate["generals"] += $stolen_generals;
					$pirate["covertagents"] += $stolen_covertagents;
					$pirate["soldiers"] += $stolen_soldiers;
					$pirate["fighters"] += $stolen_fighters;
					$pirate["lightcruisers"] += $stolen_lightcruisers;
					$pirate["heavycruisers"] += $stolen_heavycruisers;

					$this->DB->Execute("UPDATE tb_pirate SET food=" . $pirate["food"] . ",credits=" . $pirate["credits"] . ",generals=" . $pirate["generals"] . ",covertagents=" . $pirate["covertagents"] . ",soldiers=" . $pirate["soldiers"] . ",fighters=" . $pirate["fighters"] . ",lightcruisers=" . $pirate["lightcruisers"] . ",heavycruisers=" . $pirate["heavycruisers"] . " WHERE id=" + $pirate["id"]);

					// send notifications
					$evt = new EventCreator($this->DB);
					$evt->type = CONF_EVENT_TRADECONVOY_RAIDEDBYPIRATE;
					$evt->from = -1;
					$evt->params = array (
						"stolen_food" => $stolen_food,
						"stolen_credits" => $stolen_credits,
						"stolen_generals" => $stolen_generals,
						"stolen_covertagents" => $stolen_covertagents,
						"stolen_soldiers" => $stolen_soldiers,
						"stolen_fighters" => $stolen_fighters,
						"stolen_lightcruisers" => $stolen_lightcruisers,
						"stolen_heavycruisers" => $stolen_heavycruisers
					);
					$evt->to = $rs->fields["empire_from"];
					$evt->send();

				}

				// update record
				$this->DB->Execute("UPDATE tb_tradeconvoy SET time_left=" . $rs->fields["time_left"] . " WHERE id=" . $rs->fields["id"]);

			}

			$rs->MoveNext();
		}

	}

	//////////////////////////////////////////////////////////////////////
	// AI TURNS
	//////////////////////////////////////////////////////////////////////
	function handleAITurns() {
		/*
				global $GAME,$DB;
		                
				require_once ("include/newturn/pirateraid.php");
				require_once ("include/newturn/researchgrowth.php");
				require_once ("include/newturn/foodgrowth.php");
				require_once ("include/newturn/moneygrowth.php");
				require_once ("include/newturn/military_production.php");
				require_once ("include/newturn/populationgrowth.php");
				require_once ("include/newturn/randomevent.php");
				require_once ("include/newturn/civilstatus.php");
				require_once ("include/newturn/lottery.php");
				require_once ("include/newturn/blackmarket.php");
		
				$previous_empire = $GAME["empire"];
		
		
				$rs = $DB->Execute("SELECT id FROM tb_empire WHERE NOT ai_level = -1 AND turns_left > 0");
		//		$rs = $DB->Execute("SELECT id FROM tb_empire WHERE NOT ai_level = -1"); // remove
				while(!$rs->EOF)
				{
					$ai_empire = new Empire($this->DB,$GAME["template"],$GAME["gameplay_costs"]);	
					$ai_empire->load($rs->fields["id"]);
					$GAME["empire"] = $ai_empire;
					$turns_left = $ai_empire->data["turns_left"];
					if ($turns_left > 1) $turns_left = 1;
					$turns_left = 5; // remove
					$GAME["empire"]->data["networth"]." : Turns played: ".$GAME["empire"]->data["turns_played"]." : POP: " .$GAME["empire"]->data["population"]." : FOOD:".$GAME["empire"]->data["food"]."\r\n";
					for ($i=0;$i<$turns_left;$i++) {
		
						$advisor = new AgriculturalAdvisor($GAME["empire"]);
						$reply = $advisor->getAdvise();
						
						if ($reply["action"] == "nothing") {
							// actually do nothing then. we may try to sell our food (HALF)
							$sell = floor($GAME["empire"]->data["food"] / 2);
							if ($sell > 0) {
		
								$prices = $GAME["gameplay_costs"]->getFoodPrices();
								$GAME["empire"]->data["credits"] += ($sell * ($prices["sell"]));
								$GAME["empire"]->data["food"] -= $sell;
								$GAME["empire"]->data["food_traded"] = 1;
								$GAME["template"]->coord["food"] += $sell;
								$query = "UPDATE tb_coordinator SET food=".$GAME["template"]->coord["food"];
								if (!$DB->Execute($query)) trigger_error($DB->ErrorMsg());
		
							}
						} else {
		
		
							$action = explode("=",$reply["action"]);
		
							if (count($action) == 2) {
								if ($action[0] == "buy_food_planets") {
									if ($GAME["template"]->coord["food_planets"] >= $action[1]) {
										$GAME["empire"]->planets->data["food_planets"] += $action[1];
										$GAME["template"]->coord["food_planets"] -= $action[1];
										$query = "UPDATE tb_coordinator SET food_planets=".$GAME["template"]->coord["food_planets"];
										if (!$DB->Execute($query)) trigger_error($DB->ErrorMsg());
									} else {
										$GAME["empire"]->planets->data["food_planets"] += $GAME["template"]->coord["food_planets"];
										$GAME["template"]->coord["food_planets"] = 0;
										$query = "UPDATE tb_coordinator SET food_planets=".$GAME["template"]->coord["food_planets"];
										if (!$DB->Execute($query)) trigger_error($DB->ErrorMsg());
									}
		
									
								}
							}
						}
		
		
		
		
						$advisor = new PopulationAdvisor($GAME["empire"]);
						$reply = $advisor->getAdvise();
		
						$advisor = new MilitaryAdvisor($GAME["empire"]);
						$reply = $advisor->getAdvise();
		
					
		
		
						if ($GAME["empire"]->data["protection_turns_left"] > 0) 
							$GAME["empire"]->data["protection_turns_left"]--;
		
						$GAME["empire"]->data["turns_played"]++;
		
		
		
						$GAME["empire"]->army->data["covert_points"] += CONF_COVERTPOINTS_PER_TURN;
						if ($GAME["empire"]->army->data["covert_points"] > CONF_MAX_COVERTPOINTS) 
							$GAME["empire"]->army->data["covert_points"] = CONF_MAX_COVERTPOINTS;
		
						$GAME["empire"]->data["planets_bought"] = 0;
						$GAME["empire"]->data["food_traded"] = 0;
						$GAME["empire"]->data["already_attacked"] = 0;
						$GAME["empire"]->data["last_turn_date"] = time(NULL);
		
		
		
						if ($GAME["empire"]->army->data["effectiveness"] != 100)
						{
							$GAME["empire"]->army->data["effectiveness"]++;
						}
		
						$GAME["ai_turn"] = true;
						NewTurn_HandlePirateRaid();
						NewTurn_HandleResearchGrowth();
						NewTurn_HandleFoodGrowth();
						NewTurn_HandleMoneyGrowth();
						NewTurn_HandleMilitaryProduction();
						NewTurn_HandlePopulationGrowth();
						NewTurn_PickRandomEvent();
						NewTurn_HandleCivilStatus();
						NewTurn_HandleLotteryPayout();
						NewTurn_handleBlackMarket();
						$GAME["ai_turn"] = false;
		
		
						$GAME["empire"]->data["inflation"] += CONF_COST_INFLATION;
		
		
						// UPDATING production
						$production_food_short = $GAME["empire"]->production->data["food_short"];
						$production_food_long = $GAME["empire"]->production->data["food_long"];
						$production_ore_short = $GAME["empire"]->production->data["ore_short"];
						$production_ore_long = $GAME["empire"]->production->data["ore_long"];
						$production_tourism_short = $GAME["empire"]->production->data["tourism_short"];
						$production_tourism_long = $GAME["empire"]->production->data["tourism_long"];
						$production_supply_short = $GAME["empire"]->production->data["supply_short"];
						$production_supply_long = $GAME["empire"]->production->data["supply_long"];
						$production_edu_short = $GAME["empire"]->production->data["edu_short"];
						$production_edu_long = $GAME["empire"]->production->data["edu_long"];
						$production_research_short = $GAME["empire"]->production->data["research_short"];
						$production_research_long = $GAME["empire"]->production->data["research_long"];
						$production_urban_short = $GAME["empire"]->production->data["urban_short"];
						$production_urban_long = $GAME["empire"]->production->data["urban_long"];
						$production_petro_short = $GAME["empire"]->production->data["petro_short"];
						$production_petro_long = $GAME["empire"]->production->data["petro_long"];
						$production_antipollu_short = $GAME["empire"]->production->data["antipollu_short"];
						$production_antipollu_long = $GAME["empire"]->production->data["antipollu_long"];
		
						$production_food = ($production_food_short - $production_food_long)/10;
						$production_ore = ($production_ore_short - $production_ore_long)/10;
						$production_tourism = ($production_tourism_short - $production_tourism_long)/10;
						$production_supply = ($production_supply_short - $production_supply_long)/10;
						$production_edu = ($production_edu_short - $production_edu_long)/10;
						$production_research = ($production_research_short - $production_research_long)/10;
						$production_urban = ($production_urban_short - $production_urban_long)/10;
						$production_petro = ($production_petro_short - $production_petro_long)/10;
						$production_antipollu = ($production_antipollu_short - $production_antipollu_long)/10;
		
						$GAME["empire"]->production->data["food_short"] -= $production_food;
						$GAME["empire"]->production->data["ore_short"] -= $production_ore;
						$GAME["empire"]->production->data["tourism_short"] -= $production_tourism;
						$GAME["empire"]->production->data["supply_short"] -= $production_supply;
						$GAME["empire"]->production->data["edu_short"] -= $production_edu;
						$GAME["empire"]->production->data["research_short"] -= $production_research;
						$GAME["empire"]->production->data["urban_short"] -= $production_urban;
						$GAME["empire"]->production->data["petro_short"] -= $production_petro;
						$GAME["empire"]->production->data["antipollu_short"] -= $production_antipollu;
		
		
						$GAME["empire"]->updateStats();
						$GAME["empire"]->updateNetworth();
						
					}
					
					$ai_empire = $GAME["empire"];
					$ai_empire->data["turns_left"] -= $turns_left;
		
					if ($ai_empire->data["population"] < 50) {
						$ai_empire->Collapse();
						$ai_empire->Delete();
					}
		
					$ai_empire->save();
				
						
					$rs->MoveNext();
				}
		
		
			*/

	}

	//////////////////////////////////////////////////////////////////////
	// Handle army convoys
	//////////////////////////////////////////////////////////////////////
	function handleArmyConvoys($elapsed) {
		global $GAME;

		$rs = $this->DB->Execute("SELECT * FROM tb_armyconvoy");
		while (!$rs->EOF) {

			$rs2 = $this->DB->Execute("SELECT * FROM tb_empire WHERE id=" . $rs->fields["empire_to"]);
			$rs3 = $this->DB->Execute("SELECT * FROM tb_empire WHERE id=" . $rs->fields["empire_from"]);
			if (($rs2->EOF) || ($rs3->EOF)) {
				$this->DB->Execute("DELETE FROM tb_armyconvoy WHERE id=" . $rs->fields["id"]);
				$rs->MoveNext();
				continue;
			}

			$rs->fields["time_left"] -= $elapsed;
			if ($rs->fields["time_left"] < 0)
				$rs->fields["time_left"] = 0;

			if ($rs->fields["time_left"] <= 0) {

				if (($rs->fields["convoy_type"] == CONF_CONVOY_INVASION_RETREAT) || ($rs->fields["convoy_type"] == CONF_CONVOY_DEFENSE_RETREAT)) {
					// returning convoy!
					/*
					$evt = new EventCreator($this->DB);
					$evt->from = -1;
					$evt->to = $rs->fields["empire_from"];
					$evt->type = CONF_EVENT_CONVOY_BACK;
					$evt->params = $rs->fields;
					$evt->send();
					*/

					$target_empire = new Empire($this->DB, $GAME["template"], $GAME["gameplay_costs"]);
					$result = $target_empire->Load($rs->fields["empire_from"]);
					
					if ($result["code"] == true) {
						$target_empire->army->data["soldiers"] += $rs->fields["convoy_soldiers"];
						$target_empire->army->data["generals"] += $rs->fields["convoy_generals"];
						$target_empire->army->data["fighters"] += $rs->fields["convoy_fighters"];
						$target_empire->army->data["lightcruisers"] += $rs->fields["convoy_lightcruisers"];
						$target_empire->army->data["heavycruisers"] += $rs->fields["convoy_heavycruisers"];
						$target_empire->army->data["carriers"] += $rs->fields["carriers"];
						$target_empire->Save();
					}
					
					$this->DB->Execute("DELETE FROM tb_armyconvoy WHERE id=" . $rs->fields["id"]);

				} else {

					if ($rs->fields["convoy_type"] == CONF_CONVOY_INVASION) {
						$invasion = new Invasion($this->DB, $rs->fields);
						$invasion->Attack();
					} else {

						// make the defense convoy come back if the invasion is done.
						$rs2 = $this->DB->Execute("SELECT * FROM tb_armyconvoy WHERE id=" . $rs->fields["convoy_target"]);
						if (($rs2->EOF) || ($rs2->fields["convoy_type"] == CONF_CONVOY_INVASION_RETREAT)) {
							$this->DB->Execute("UPDATE tb_armyconvoy SET convoy_type=" . CONF_CONVOY_DEFENSE_RETREAT . ",time_left=" . $rs->fields["time_total"] . " WHERE id=" . $rs->fields["id"]);
							/*
							$evt = new EventCreator($this->DB);
							$evt->from = -1;
							$evt->to = $rs->fields["empire_to"];
							$evt->type = CONF_EVENT_CONVOY_RETREAT;
							$evt->params = $rs->fields["empire_from"];
							$evt->send();
							*/

						}
						else
							$this->DB->Execute("UPDATE tb_armyconvoy SET time_left=0 WHERE id=".$rs->fields["id"]);
					}

				}

			} else {

				// update record
				$this->DB->Execute("UPDATE tb_armyconvoy SET time_left=" . $rs->fields["time_left"] . " WHERE id=" . $rs->fields["id"]);

			}

			$rs->MoveNext();
		}

	}

	//////////////////////////////////////////////////////////////////////
	// HTTP redirect
	//////////////////////////////////////////////////////////////////////
	function redirect($path, $args = array ()) {
		ob_clean();

		if ((count($args)) == 0)
			$url = $path;
		else {

			$url = $path . "?";
			reset($args);
			while (list ($key, $value) = each($args)) {
				$url .= $key . "=" . urlencode($value) . "&";
			}
		}

		//	die("Redirect to: $url");

		header("Location: $url");
		die("<script>window.location='$url';</script>");
	}

	//////////////////////////////////////////////////////////////////////
	// Send forgot password email
	//////////////////////////////////////////////////////////////////////
	function sendForgotPasswordEmail($email) {

		$email = addslashes($_POST["email"]);
		if ($email == "")
			die("Error: Empty e-mail address");

		$query = "SELECT * FROM tb_empire WHERE email='$email'";
		$rs = $this->DB->Execute($query);

		if (!$rs->EOF) {

			$newpassword = $this->generatePassword();

			$activation_hash = md5(time(NULL) . $email);

			$mail = new PHPMailer();
			$mail->From = MAIL_FROM;
			$mail->FromName = LANG_FORGOTPASSWORD_MAIL_SUBJECT;
			$mail->Host = MAIL_SMTP;
			$mail->Mailer = "smtp";

			$mail->AddAddress($email);

			$mail->Body = str_replace("{password}", $newpassword, TPL_MAIL_CONTENT_FORGETPASSWORD);
			$mail->Subject = LANG_FORGOTPASSWORD_MAIL_SUBJECT;

			if (!$mail->Send())
				die("Error while sending email, retry or contact administrator");

			$query = "UPDATE tb_empire SET password='" . md5($newpassword) . "' WHERE email='$email'";
			$DB->Execute($query);

			return true;
		}
	}

	//////////////////////////////////////////////////////////////////////
	// Generate Password
	//////////////////////////////////////////////////////////////////////
	function generatePassword() {

		$keyspace = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";

		srand(time(NULL));
		$new_password = "";

		for ($i = 0; $i < 8; $i++) {
			$new_password .= $keyspace[rand() % strlen($keyspace)];
		}

		return $new_password;
	}

	//////////////////////////////////////////////////////////////////////
	// Filter text
	//////////////////////////////////////////////////////////////////////
	function filterText($text, $size = -1) {

		$text = stripslashes($text);
		$text = str_replace("..", ".", $text);
		$text = str_replace("<", "(", $text);
		$text = str_replace(">", ")", $text);
		$text = str_replace("\n", "<br/>", $text);

		if ($size > -1)
			$text = substr($text, 0, $size);

		return $text;
	}

	//////////////////////////////////////////////////////////////////////
	// Send activation email
	//////////////////////////////////////////////////////////////////////
	function sendActivationEmail($email, $activation_hash) {

		$mail = new PHPMailer();
		$mail->From = CONF_MAIL_FROM;
		$mail->FromName = LANG_ACTIVATION_MAIL_SUBJECT;
		$mail->Host = CONF_MAIL_SMTP;
		$mail->Mailer = "smtp";

		$mail->AddAddress($email);

		$mail->Body = str_replace("{activation_hash}", $activation_hash, TPL_MAIL_CONTENT_NEWPLAYER);
		$mail->Subject = LANG_ACTIVATION_MAIL_SUBJECT;

		if (!$mail->Send())
			die("Error while sending email, retry or contact administrator");

	}

	//////////////////////////////////////////////////////////////////////
	//
	//////////////////////////////////////////////////////////////////////
	function alterNumber($number, $percent) {
		$p = rand(0, $percent);

		$r = rand(0, 1);
		if ($r == 0)
			$number += (($number / 100) * $p);
		else
			$number -= (($number / 100) * $p);

		return round($number);
	}

	//////////////////////////////////////////////////////////////////////
	//
	//////////////////////////////////////////////////////////////////////
	function handleVictoryConditions() {

		if ($this->game_status == 1)
			return; // already ended

		// check for research race first
		if (CONF_VICTORY_CONDITION_RESEARCH_RACE) {

			$rs = $this->DB->Execute("SELECT * FROM tb_empire WHERE admin=0 AND research_level>=9 ORDER BY networth DESC");

			if (!$rs->EOF) {

				// we have a winner
				$coord = $this->DB->Execute("SELECT * FROM tb_coordinator");
				$coord = $coord->fields;
				if ($coord["game_status"] == 1)
					return;

				$best_coalition = $this->DB->Execute("SELECT * FROM tb_coalition ORDER BY networth DESC LIMIT 0,1");
				$best_coalition = $best_coalition->fields;
				if ($best_coalition["name"] == "")
					$best_coalition["name"] = "---";

				$winner_empire = $rs->fields["emperor"] . "@" . $rs->fields["name"] . " (Turns:" . $rs->fields["turns_played"] . ")";
				$gold_empire = "";
				$gold_networth = 0;
				$silver_empire = "";
				$silver_networth = 0;
				$bronze_empire = "";
				$bronze_networth = 0;

				$rs2 = $this->DB->Execute("SELECT * FROM tb_empire WHERE admin=0 AND active=1 ORDER BY networth DESC LIMIT 0,3");
				if ($rs2->EOF)
					return;

				$gold_empire = $rs2->fields["emperor"] . "@" . $rs2->fields["name"] . " (Turns:" . $rs2->fields["turns_played"] . ")";
				$gold_networth = $rs2->fields["networth"];

				$rs2->MoveNext();
				if ($rs2->EOF) {
					$silver_empire = $gold_empire;
					$silver_networth = $gold_networth;

				} else {

					$silver_empire = $rs2->fields["emperor"] . "@" . $rs2->fields["name"] . " (Turns:" . $rs2->fields["turns_played"] . ")";

					$silver_networth = $rs2->fields["networth"];

				}
				$rs2->MoveNext();
				if ($rs2->EOF) {
					$bronze_empire = $gold_empire;
					$bronze_networth = $gold_networth;

				} else {

					$bronze_empire = $rs2->fields["emperor"] . "@" . $rs2->fields["name"] . " (Turns:" . $rs2->fields["turns_played"] . ")";
					$bronze_networth = $rs2->fields["networth"];

				}

				$query = "INSERT INTO tb_hall_of_fame (game_name,date,elapsed,victory_condition,winner_coalition,gold_empire,gold_networth,silver_empire,silver_networth,bronze_empire,bronze_networth,winner_empire) " .
				"VALUES(" .
				"'" . CONF_GAME_NAME . "'," .
				time(NULL) . "," .
				 (time(NULL) - $coord["date"]) . "," .
				"'" . LANG_RESEARCH_RACE . "'," .
				"'" . addslashes($best_coalition["name"]) . "'," .
				"'" . addslashes($gold_empire) . "'," .
				addslashes($gold_networth) . "," .
				"'" . addslashes($silver_empire) . "'," .
				addslashes($silver_networth) . "," .
				"'" . addslashes($bronze_empire) . "'," .
				addslashes($bronze_networth) . "," .
				"'" . addslashes($winner_empire) . "'" .
				")";

				$this->DB->Execute($query);

				$this->DB->Execute("UPDATE tb_coordinator SET game_status=1,restart_date=" . (time(NULL) + CONF_GAME_RESTART_DELAY));

				$this->redirect("hall_of_fame.php");
			}

		}

		// Classic 1 month game
		if (CONF_VICTORY_CONDITION_CLASSIC_GAME > 0) {

			$coord = $this->DB->Execute("SELECT * FROM tb_coordinator");
			$coord = $coord->fields;
			if ($coord["game_status"] == 1)
				return;

			if ((time(NULL) - $coord["date"]) >= CONF_VICTORY_CONDITION_CLASSIC_GAME) {

				// we have a winner

				$best_coalition = $this->DB->Execute("SELECT * FROM tb_coalition ORDER BY networth DESC LIMIT 0,1");
				$best_coalition = $best_coalition->fields;
				if ($best_coalition["name"] == "")
					$best_coalition["name"] = "---";

				$gold_empire = "";
				$gold_networth = 0;
				$silver_empire = "";
				$silver_networth = 0;
				$bronze_empire = "";
				$bronze_networth = 0;

				$rs2 = $this->DB->Execute("SELECT * FROM tb_empire WHERE admin=0 AND active=1 ORDER BY networth DESC LIMIT 0,3");
				if ($rs2->EOF)
					return;

				$gold_empire = $rs2->fields["emperor"] . "@" . $rs2->fields["name"] . " (Turns:" . $rs2->fields["turns_played"] . ")";
				$gold_networth = $rs2->fields["networth"];

				$rs2->MoveNext();
				if ($rs2->EOF) {
					$silver_empire = $gold_empire;
					$silver_networth = $gold_networth;

				} else {

					$silver_empire = $rs2->fields["emperor"] . "@" . $rs2->fields["name"] . " (Turns:" . $rs2->fields["turns_played"] . ")";
					;
					$silver_networth = $rs2->fields["networth"];

				}
				$rs2->MoveNext();
				if ($rs2->EOF) {
					$bronze_empire = $gold_empire;
					$bronze_networth = $gold_networth;

				} else {

					$bronze_empire = $rs2->fields["emperor"] . "@" . $rs2->fields["name"] . " (Turns:" . $rs2->fields["turns_played"] . ")";
					;
					$bronze_networth = $rs2->fields["networth"];

				}

				$winner_empire = $gold_empire;

				$query = "INSERT INTO tb_hall_of_fame (game_name,date,elapsed,victory_condition,winner_coalition,gold_empire,gold_networth,silver_empire,silver_networth,bronze_empire,bronze_networth,winner_empire) " .
				"VALUES(" .
				"'" . CONF_GAME_NAME . "'," .
				time(NULL) . "," .
				 (time(NULL) - $coord["date"]) . "," .
				"'" . LANG_CLASSIC_GAME_VICTORY . "'," .
				"'" . addslashes($best_coalition["name"]) . "'," .
				"'" . addslashes($gold_empire) . "'," .
				addslashes($gold_networth) . "," .
				"'" . addslashes($silver_empire) . "'," .
				addslashes($silver_networth) . "," .
				"'" . addslashes($bronze_empire) . "'," .
				addslashes($bronze_networth) . "," .
				"'" . addslashes($winner_empire) . "'" .
				")";

				$this->DB->Execute($query);

				$this->DB->Execute("UPDATE tb_coordinator SET game_status=1,restart_date=" . (time(NULL) + CONF_GAME_RESTART_DELAY));

				$this->redirect("hall_of_fame.php");
			}

		}

	}

	function Reset() {

		$players = $this->DB->Execute("SELECT id,admin FROM tb_empire");
		while (!$players->EOF) {

			if ($players->fields["admin"] == 1) {

				$this->DB->Execute("UPDATE tb_empire SET research_level=0,research_points=0,research_tech_id=-1 WHERE id=" . $players->fields["id"]);

			} else {
				$this->DB->Execute("DELETE FROM tb_army WHERE  empire=" . $players->fields["id"]);
				$this->DB->Execute("DELETE FROM tb_planets WHERE empire=" . $players->fields["id"]);
				$this->DB->Execute("DELETE FROM tb_production WHERE empire=" . $players->fields["id"]);
				$this->DB->Execute("DELETE FROM tb_empire WHERE id=" . $players->fields["id"]);
//					$this->DB->Execute("UPDATE tb_empire SET active=2 WHERE id=" . $players->fields["id"]);
				$this->DB->Execute("DELETE FROM tb_supply WHERE empire=" . $players->fields["id"]);
			}

			$players->MoveNext();
		}

		$this->DB->Execute("DELETE FROM tb_bond");
		$this->DB->Execute("DELETE FROM tb_coalition");
		$this->DB->Execute("DELETE FROM tb_event");
		$this->DB->Execute("DELETE FROM tb_loan");
		$this->DB->Execute("DELETE FROM tb_log_login");
		$this->DB->Execute("DELETE FROM tb_member");
		$this->DB->Execute("DELETE FROM tb_research_done");
		$this->DB->Execute("DELETE FROM tb_session");
		$this->DB->Execute("DELETE FROM tb_shoutbox");
		$this->DB->Execute("DELETE FROM tb_stats");
		$this->DB->Execute("DELETE FROM tb_treaty");
		$this->DB->Execute("DELETE FROM tb_tradeconvoy");

		$this->DB->Execute("DELETE FROM tb_coordinator");
		$this->DB->Execute("INSERT INTO tb_coordinator (taxcash) VALUES(0)");

		$query = "UPDATE tb_coordinator SET
			food_planets=" . CONF_RESET_COORDINATOR_PLANETS . ",
			ore_planets=" . CONF_RESET_COORDINATOR_PLANETS . ",
			tourism_planets=" . CONF_RESET_COORDINATOR_PLANETS . ",
			supply_planets=" . CONF_RESET_COORDINATOR_PLANETS . ",
			gov_planets=" . CONF_RESET_COORDINATOR_PLANETS . ",
			edu_planets=" . CONF_RESET_COORDINATOR_PLANETS . ",
			research_planets=" . CONF_RESET_COORDINATOR_PLANETS . ",
			urban_planets=" . CONF_RESET_COORDINATOR_PLANETS . ",
			petro_planets=" . CONF_RESET_COORDINATOR_PLANETS . ",
			antipollu_planets=" . CONF_RESET_COORDINATOR_PLANETS . ",
			taxcash=0,
			food=" . CONF_RESET_COORDINATOR_FOOD . ",
			date=" . time(NULL) . ",game_status=0,
			lottery_cash=0,
			lottery_date=0
			";

		$this->DB->Execute($query);
		$this->DB->Execute("DELETE FROM tb_bulletin");
		$this->DB->Execute("INSERT INTO tb_bulletin (content) VALUES('" . addslashes(LANG_BULLETIN_RESET) . "')");

		// remove avatars
		$dir = opendir("images/avatars/");
		while ($file = readdir($dir)) {
			if ($file[0] == ".")
				continue;
			if ($file == "dead.jpg")
				continue;
			if ($file == "anonymous.jpg")
				continue;
			unlink("images/avatars/" . $file);
		}
		closedir($dir);

		// remove empire logos
		$dir = opendir("images/empires/");
		while ($file = readdir($dir)) {
			if ($file[0] == ".")
				continue;
			if ($file == "new.jpg")
				continue;
			unlink("images/empires/" . $file);
		}
		closedir($dir);

		// reset pirates
		$rs = $this->DB->Execute("SELECT * FROM tb_pirate");
		srand(time(NULL));

		while (!$rs->EOF) {
			$pirate = $rs->fields;
			$pirate["credits"] = rand(0, CONF_PIRATE_BASEVALUE);
			$pirate["food"] = rand(0, CONF_PIRATE_BASEVALUE);
			$pirate["research"] = rand(0, CONF_PIRATE_BASEVALUE);
			$pirate["covertagents"] = rand(0, CONF_PIRATE_BASEVALUE);
			$pirate["generals"] = rand(0, CONF_PIRATE_BASEVALUE);
			$pirate["stations"] = rand(0, CONF_PIRATE_BASEVALUE);
			$pirate["soldiers"] = rand(0, CONF_PIRATE_BASEVALUE);
			$pirate["fighters"] = rand(0, CONF_PIRATE_BASEVALUE);
			$pirate["lightcruisers"] = rand(0, CONF_PIRATE_BASEVALUE);
			$pirate["heavycruisers"] = rand(0, CONF_PIRATE_BASEVALUE);
			$pirate["carriers"] = rand(0, CONF_PIRATE_BASEVALUE);
			$pirate["food_planets"] = rand(0, CONF_PIRATE_BASEVALUE);
			$pirate["ore_planets"] = rand(0, CONF_PIRATE_BASEVALUE);
			$pirate["tourism_planets"] = rand(0, CONF_PIRATE_BASEVALUE);
			$pirate["supply_planets"] = rand(0, CONF_PIRATE_BASEVALUE);
			$pirate["gov_planets"] = rand(0, CONF_PIRATE_BASEVALUE);
			$pirate["edu_planets"] = rand(0, CONF_PIRATE_BASEVALUE);
			$pirate["urban_planets"] = rand(0, CONF_PIRATE_BASEVALUE);
			$pirate["research_planets"] = rand(0, CONF_PIRATE_BASEVALUE);
			$pirate["petro_planets"] = rand(0, CONF_PIRATE_BASEVALUE);
			$pirate["antipollu_planets"] = rand(0, CONF_PIRATE_BASEVALUE);

			$networth = 0;
			$networth += ($pirate["credits"] * CONF_NETWORTH_CREDITS);

			$planets = 0;
			$planets += $pirate["food_planets"];
			$planets += $pirate["ore_planets"];
			$planets += $pirate["tourism_planets"];
			$planets += $pirate["supply_planets"];
			$planets += $pirate["gov_planets"];
			$planets += $pirate["edu_planets"];
			$planets += $pirate["research_planets"];
			$planets += $pirate["urban_planets"];
			$planets += $pirate["petro_planets"];
			$planets += $pirate["antipollu_planets"];
			$networth += ($planets * CONF_NETWORTH_PLANETS);

			$army = 0;
			$army += $pirate["soldiers"] * CONF_NETWORTH_MILITARY_SOLDIER;
			$army += $pirate["fighters"] * CONF_NETWORTH_MILITARY_FIGHTER;
			$army += $pirate["stations"] * CONF_NETWORTH_MILITARY_STATION;
			$army += $pirate["lightcruisers"] * CONF_NETWORTH_MILITARY_LIGHTCRUISER;
			$army += $pirate["heavycruisers"] * CONF_NETWORTH_MILITARY_HEAVYCRUISER;
			$army += $pirate["carriers"] * CONF_NETWORTH_MILITARY_CARRIER;
			$army += $pirate["generals"] * CONF_NETWORTH_MILITARY_GENERAL;
			$army += $pirate["covertagents"] * CONF_NETWORTH_MILITARY_COVERT;
			$networth += $army;

			$pirate["networth"] = $networth;

			$this->DB->Execute("UPDATE tb_pirate SET soldiers=" . $pirate["soldiers"] . ",
												fighters=" . $pirate["fighters"] . ",
												lightcruisers=" . $pirate["lightcruisers"] . ",
												heavycruisers=" . $pirate["heavycruisers"] . ",
												stations=" . $pirate["stations"] . ",
												carriers=" . $pirate["carriers"] . ",
												generals=" . $pirate["generals"] . ",
												covertagents=" . $pirate["covertagents"] . ",
												food=" . $pirate["food"] . ",
												credits=" . $pirate["credits"] . ",
												research=" . $pirate["research"] . ",
												food_planets=" . $pirate["food_planets"] . ",
												ore_planets=" . $pirate["ore_planets"] . ",
												tourism_planets=" . $pirate["tourism_planets"] . ",
												supply_planets=" . $pirate["supply_planets"] . ",
												gov_planets=" . $pirate["gov_planets"] . ",
												edu_planets=" . $pirate["edu_planets"] . ",
												research_planets=" . $pirate["research_planets"] . ",
												urban_planets=" . $pirate["urban_planets"] . ",
												petro_planets=" . $pirate["petro_planets"] . ",
												antipollu_planets=" . $pirate["antipollu_planets"] . ",
												networth=" . addslashes($pirate["networth"]) . "
											 	WHERE id=" . $pirate["id"]);

			$rs->MoveNext();
		}
	}

}
?>
