<?php

//---------------------------------------------------------------------------//
// author: Yanick Bourbeau <ybourbeau@mrgtech.ca> 
// date:   2006.01.01
// web:    http://www.mrgtech.ca
// info:   Solar Imperium
//---------------------------------------------------------------------------//
// copyleft license
//
// this software is provided 'as-is', without any express or implied
// warranty. in no event will the authors be held liable for any damages
// arising from the use of this software.
//
// permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions: 
//
// 1. the origin of this software must not be misrepresented;
//	  you must not claim that you wrote the original software. 
//	  if you use this software in a product, an acknowledgment 
//	  in the product documentation would be appreciated but is not required.
//
// 2. altered source versions must be plainly marked as such, 
//	  and must not be misrepresented as being the original software.
//
// 3. mail about the fact of using this class in production 
//	  would be very appreciated.
//
// 4. this notice may not be removed or altered from any source distribution.
//
//---------------------------------------------------------------------------//
// CHANGELOG:
//
// 2006.06.01 : ybo : Changed name: Imperium Galactica is (c), moved to Solar Imperium
// 2007.02.04 : ybo : Reviewed the code for public release
//
//---------------------------------------------------------------------------//

function NewTurn_HandlePopulationGrowth()
{
	global $GAME,$DB,$CONF_CIVIL_STATUS;

	srand(time(NULL));

	$initial_population = $GAME["empire"]->data["population"];
	if ($initial_population < 0) $initial_population = 0;

	$final_population = 0;
	$pollution = floor((int) (($GAME["empire"]->planets->data["petro_planets"]/100)*$GAME["empire"]->production->data["petro_short"]) * CONF_PETRO_POLLUTION);
	$pollution += floor((int) $GAME["empire"]->data["population"] * CONF_POP_POLLUTION);
	$pollution = round($pollution,2);
	
	

	if ($GAME["empire"]->planets->data["antipollu_planets"]!=0) {
		$antipollution = $GAME["empire"]->planets->data["antipollu_planets"] * CONF_ANTIPOLLUTION;
		$antipollution = round(($antipollution/100)* $GAME["empire"]->production->data["antipollu_short"]);
	} else $antipollution = 0;
	

	$antipollution = $GAME["system"]->alterNumber($antipollution,5);
	///////////////////////////////////////////////////////////////////
	// BORN
	///////////////////////////////////////////////////////////////////

	$born = 0;

	// First, we calculate the prime growth of the population itself
	$born_prime = $GAME["empire"]->data["population"] * CONF_POP_BORN;
	// Second, we add the growth obtained by the bonus from urban planets
	$born_bonus = $GAME["empire"]->production->data["urban_short"] * CONF_POP_URBAN;
	if ($antipollution == 0) $antipollution = 1;


	$pratio = $pollution / $antipollution;
	// Third, we calculate a malus caused by the pollution;
	$born_malus = ($born_prime * $born_bonus) * ($pratio);

	// civil unrest
	$born_malus += (($born_prime * $born_bonus) * ($GAME["empire"]->data["civil_status"] * CONF_POP_CIVILSTATUS));

	// fourth, tax rates
	$taxrate = $GAME["empire"]->data["taxrate"];
	$taxrate_multi = 1;
	if ($taxrate > 50) $taxrate_multi = 0.5;
	if ($taxrate > 60) $taxrate_multi = 1;
	if ($taxrate > 70) $taxrate_multi = 1.5;
	if ($taxrate > 80) $taxrate_multi = 2;
	if ($taxrate > 90) $taxrate_multi = 3.5;
	if ($taxrate > 100) $taxrate_multi = 4;
	
	$born_taxrate = round($born_prime * $born_bonus * $taxrate_multi) * $taxrate * CONF_POP_TAXRATE * 0.5;

	// calculating total of born
	$born = round(($born_prime * $born_bonus) - $born_malus - $born_taxrate);
	if ($born < 0) $born = 0;

	$born = $GAME["system"]->alterNumber($born,5);

	if (isset($POPULATION_STARVING)) $born = floor($born / 4);

	///////////////////////////////////////////////////////////////////
	// DEAD
	///////////////////////////////////////////////////////////////////

	$dead = 0;

	// First, we calculate the prime body count of the population itself
	$dead_prime = $GAME["empire"]->data["population"] * CONF_POP_DEAD;

	// Second, we add the dead meat obtained by pollution
	
	$dead_pollution = $dead_prime * ($pratio);
	if ($dead_pollution < 0) $dead_pollution = 0;

	
	// Third, we add the civil unrest in the equation
	$dead_civilunrest = ($dead_prime * ($GAME["empire"]->data["civil_status"] * CONF_POP_CIVILSTATUS));


	// calculating total of dead 
	$dead = round($dead_prime + $dead_pollution + $dead_civilunrest);

	$dead = $GAME["system"]->alterNumber($dead,5);

	if ($dead < 0) $dead = 0;

	///////////////////////////////////////////////////////////////////
	// IMMIGRATION
	///////////////////////////////////////////////////////////////////

	$immigration = 0;

	
	
	$immigration_students = $GAME["empire"]->planets->data["edu_planets"] * CONF_POP_EDUCATION;

	// Second, we add pollution
	$immigration_pollution = ($immigration_students * ($pratio));
	
	// Third, then civil unrest
	$immigration_civilunrest = ($immigration_students * ($GAME["empire"]->data["civil_status"] * CONF_POP_CIVILSTATUS));
	
	// fourth, tax rates
	$immigration_taxrate = $immigration_students * $GAME["empire"]->data["taxrate"] * CONF_POP_TAXRATE;

	// calculating total of immigration
	$immigration = round($immigration_students - $immigration_pollution - $immigration_civilunrest - $immigration_taxrate);

	$immigration = $GAME["system"]->alterNumber($immigration,5);


	if ($immigration < 0) $immigration = 0;	

	///////////////////////////////////////////////////////////////////
	// EMMIGRATION
	///////////////////////////////////////////////////////////////////

	$emmigration = 0;


	// First, we calculate the overcrowd factor
	$emmigration_overcrowd = $GAME["empire"]->data["population"] - ($GAME["empire"]->planets->data["urban_planets"] * CONF_POP_OVERCROWD);
	if ($emmigration_overcrowd < 0) $emmigration_overcrowd = 0; else {
		$emmigration_overcrowd *= CONF_POP_OVERCROWD_GETOUT;
	}

	// Second, we add tax rate
	$emmigration_taxrate = $GAME["empire"]->data["population"] * $GAME["empire"]->data["taxrate"] * CONF_POP_TAXRATE_GETOUT;
	
	// Third, civil unrest
	$emmigration_civilunrest = $GAME["empire"]->data["population"] * $GAME["empire"]->data["civil_status"] * CONF_POP_CIVILSTATUS;
	
	// calculating total
	$emmigration = round($emmigration_overcrowd + $emmigration_taxrate + $emmigration_civilunrest);
	$emmigration = $GAME["system"]->alterNumber($emmigration,5);

	if ($emmigration < 0) $emmigration = 0;
	
	///////////////////////////////////////////////////////////////////
	// FINALIZATION
	///////////////////////////////////////////////////////////////////
	
	$final_population = $initial_population + $born + $immigration - $dead - $emmigration;

	if ($initial_population == 0)
		$growrate = 0;
	else
	{
		$growrate = substr((($final_population - $initial_population) / $initial_population) * 100,0,5);
	}

	
	$pratio = $pollution / $antipollution;
	$color = TPL_COLOR_HIGHLIGHT;
	if ($pratio < 0) $color = "green";
	if ($pratio > 0) $color = "lightred";
	
	$evt = new EventCreator($DB);
	$evt->type = CONF_EVENT_POPULATIONGROWTH;
	$evt->from = -1;
	$evt->to = $GAME["empire"]->data["id"];
	$evt->seen = 1;
	$evt->params = array(
		"initial_population"=>$GAME["template"]->formatPopulation($initial_population),
		"final_population"=>$GAME["template"]->formatPopulation($final_population),
		"born"=>$GAME["template"]->formatPopulation($born),
		"dead"=>$GAME["template"]->formatPopulation($dead),
		"born"=>$GAME["template"]->formatPopulation($born),
		"immigration"=>$GAME["template"]->formatPopulation($immigration),
		"emmigration"=>$GAME["template"]->formatPopulation($emmigration),
		"color"=>$color,
		"pratio"=>round($pratio,6),
		"growrate"=>$growrate
	);
	
	$evt->send();



	$GAME["empire"]->data["population"] = $final_population;

	
}

?>